// auto-generated by Georg Sauthoff's eti2wireshark.py

/* packet-eti.c
 * Routines for ETI dissection
 * Copyright 2021, Georg Sauthoff <mail@gms.tf>
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/*
 * The Enhanced Trading Interface 9.1 (ETI) is an electronic trading protocol
 * that is used by a few exchanges (Eurex, Xetra, ...).
 *
 * It's a Length-Tag based protocol consisting of mostly fix sized
 * request/response messages.
 *
 * Links:
 * https://en.wikipedia.org/wiki/List_of_electronic_trading_protocols#Europe
 * https://github.com/gsauthof/python-eti#protocol-descriptions
 * https://github.com/gsauthof/python-eti#protocol-introduction
 *
 */

#include <config.h>


#include <epan/packet.h>   // Should be first Wireshark include (other than config.h)
#include "packet-tcp.h"    // tcp_dissect_pdus()
#include <epan/expert.h>   // expert info

#include <inttypes.h>
#include <stdio.h>         // snprintf()


/* Prototypes */
/* (Required to prevent [-Wmissing-prototypes] warnings */
void proto_reg_handoff_eti(void);
void proto_register_eti(void);

static int proto_eti = -1;
static expert_field ei_eti_counter_overflow = EI_INIT;
static expert_field ei_eti_invalid_template = EI_INIT;
static expert_field ei_eti_invalid_length = EI_INIT;
static expert_field ei_eti_unaligned = EI_INIT;
static expert_field ei_eti_missing = EI_INIT;
static expert_field ei_eti_overused = EI_INIT;

static int hf_eti[] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
static int hf_eti_dscp_exec_summary = -1;
static int hf_eti_dscp_improved = -1;
static int hf_eti_dscp_widened = -1;
enum Field_Handle_Index {
      ACCOUNT_FH_IDX
    , AFFECTEDORDERREQUESTID_FH_IDX
    , ALLOCID_FH_IDX
    , ALLOCQTY_FH_IDX
    , APPLBEGMSGID_FH_IDX
    , APPLBEGSEQNUM_FH_IDX
    , APPLENDMSGID_FH_IDX
    , APPLENDSEQNUM_FH_IDX
    , APPLID_FH_IDX
    , APPLIDSTATUS_FH_IDX
    , APPLMSGID_FH_IDX
    , APPLRESENDFLAG_FH_IDX
    , APPLSEQINDICATOR_FH_IDX
    , APPLSEQNUM_FH_IDX
    , APPLSEQSTATUS_FH_IDX
    , APPLSEQTRADEDATE_FH_IDX
    , APPLSUBID_FH_IDX
    , APPLTOTALMESSAGECOUNT_FH_IDX
    , APPLUSAGEORDERS_FH_IDX
    , APPLUSAGEQUOTES_FH_IDX
    , APPLICATIONSYSTEMNAME_FH_IDX
    , APPLICATIONSYSTEMVENDOR_FH_IDX
    , APPLICATIONSYSTEMVERSION_FH_IDX
    , AUTOAPPROVALRULEID_FH_IDX
    , BASKETEXECID_FH_IDX
    , BASKETPARTYCONTRAFIRM_FH_IDX
    , BASKETPROFILEID_FH_IDX
    , BASKETSIDETRADEREPORTID_FH_IDX
    , BASKETTRADEREPORTTEXT_FH_IDX
    , BASKETTRADEREPORTTYPE_FH_IDX
    , BASKETTRDMATCHID_FH_IDX
    , BESTBIDPX_FH_IDX
    , BESTBIDSIZE_FH_IDX
    , BESTOFFERPX_FH_IDX
    , BESTOFFERSIZE_FH_IDX
    , BIDPX_FH_IDX
    , BIDSIZE_FH_IDX
    , BODYLEN_FH_IDX
    , CLORDID_FH_IDX
    , CLEARINGTRADEPRICE_FH_IDX
    , CLEARINGTRADEQTY_FH_IDX
    , COMPLIANCETEXT_FH_IDX
    , COMPRESSIONACTION_FH_IDX
    , COMPRESSIONID_FH_IDX
    , COMPRESSIONSTATUS_FH_IDX
    , CROSSID_FH_IDX
    , CROSSREQUESTID_FH_IDX
    , CROSSEDINDICATOR_FH_IDX
    , CUMQTY_FH_IDX
    , CUSTORDERHANDLINGINST_FH_IDX
    , CXLQTY_FH_IDX
    , CXLSIZE_FH_IDX
    , DEFAULTCSTMAPPLVERID_FH_IDX
    , DEFAULTCSTMAPPLVERSUBID_FH_IDX
    , DELETEREASON_FH_IDX
    , DELTA_FH_IDX
    , EFFECTONBASKET_FH_IDX
    , EFFECTIVETIME_FH_IDX
    , ENLIGHTRFQAVGRESPRATERANKING_FH_IDX
    , ENLIGHTRFQAVGRESPTIMERANKING_FH_IDX
    , ENRICHMENTRULEID_FH_IDX
    , EUREXVOLUMERANKING_FH_IDX
    , EVENTDATE_FH_IDX
    , EVENTTYPE_FH_IDX
    , EXECID_FH_IDX
    , EXECINST_FH_IDX
    , EXECRESTATEMENTREASON_FH_IDX
    , EXECTYPE_FH_IDX
    , EXECUTINGTRADER_FH_IDX
    , EXECUTINGTRADERQUALIFIER_FH_IDX
    , EXERCISESTYLE_FH_IDX
    , EXPIREDATE_FH_IDX
    , EXPIRETIME_FH_IDX
    , EXPOSUREDURATION_FH_IDX
    , FIXCLORDID_FH_IDX
    , FIXENGINENAME_FH_IDX
    , FIXENGINEVENDOR_FH_IDX
    , FIXENGINEVERSION_FH_IDX
    , FILLEXECID_FH_IDX
    , FILLLIQUIDITYIND_FH_IDX
    , FILLMATCHID_FH_IDX
    , FILLPX_FH_IDX
    , FILLQTY_FH_IDX
    , FILLREFID_FH_IDX
    , FIRMNEGOTIATIONID_FH_IDX
    , FIRMTRADEID_FH_IDX
    , FREETEXT1_FH_IDX
    , FREETEXT2_FH_IDX
    , FREETEXT3_FH_IDX
    , FREETEXT5_FH_IDX
    , FREETEXT5DISCLOSUREINSTRUCTION_FH_IDX
    , HEADLINE_FH_IDX
    , HEARTBTINT_FH_IDX
    , HEDGETYPE_FH_IDX
    , HEDGINGINSTRUCTION_FH_IDX
    , HIGHLIMITPRICE_FH_IDX
    , IMPLIEDMARKETINDICATOR_FH_IDX
    , INDIVIDUALALLOCID_FH_IDX
    , INPUTSOURCE_FH_IDX
    , INSTRATTRIBTYPE_FH_IDX
    , INSTRATTRIBVALUE_FH_IDX
    , INSTRMTMATCHSIDEID_FH_IDX
    , LASTENTITYPROCESSED_FH_IDX
    , LASTFRAGMENT_FH_IDX
    , LASTPX_FH_IDX
    , LASTPXDISCLOSUREINSTRUCTION_FH_IDX
    , LASTQTY_FH_IDX
    , LASTQTYDISCLOSUREINSTRUCTION_FH_IDX
    , LASTUPDATETIME_FH_IDX
    , LEAVESQTY_FH_IDX
    , LEAVESQTYDISCLOSUREINSTRUCTION_FH_IDX
    , LEGACCOUNT_FH_IDX
    , LEGEXECID_FH_IDX
    , LEGINPUTSOURCE_FH_IDX
    , LEGLASTPX_FH_IDX
    , LEGLASTQTY_FH_IDX
    , LEGPOSITIONEFFECT_FH_IDX
    , LEGPRICE_FH_IDX
    , LEGQTY_FH_IDX
    , LEGRATIOQTY_FH_IDX
    , LEGSECURITYID_FH_IDX
    , LEGSECURITYTYPE_FH_IDX
    , LEGSIDE_FH_IDX
    , LEGSYMBOL_FH_IDX
    , LISTUPDATEACTION_FH_IDX
    , LOWLIMITPRICE_FH_IDX
    , MDBOOKTYPE_FH_IDX
    , MDSUBBOOKTYPE_FH_IDX
    , MMPARAMETERREPORTID_FH_IDX
    , MARKETID_FH_IDX
    , MARKETSEGMENTID_FH_IDX
    , MASSACTIONREASON_FH_IDX
    , MASSACTIONREPORTID_FH_IDX
    , MASSACTIONSUBTYPE_FH_IDX
    , MASSACTIONTYPE_FH_IDX
    , MATCHDATE_FH_IDX
    , MATCHINSTCROSSID_FH_IDX
    , MATCHSUBTYPE_FH_IDX
    , MATCHTYPE_FH_IDX
    , MATCHINGENGINESTATUS_FH_IDX
    , MATCHINGENGINETRADEDATE_FH_IDX
    , MATURITYDATE_FH_IDX
    , MATURITYMONTHYEAR_FH_IDX
    , MAXIMUMPRICE_FH_IDX
    , MESSAGEEVENTSOURCE_FH_IDX
    , MSGSEQNUM_FH_IDX
    , MULTILEGREPORTINGTYPE_FH_IDX
    , MULTILEGMODEL_FH_IDX
    , MULTILEGPRICEMODEL_FH_IDX
    , NEGOTIATIONID_FH_IDX
    , NEGOTIATIONSTARTTIME_FH_IDX
    , NETWORKMSGID_FH_IDX
    , NOAFFECTEDORDERREQUESTS_FH_IDX
    , NOBASKETROOTPARTYGRPS_FH_IDX
    , NOBASKETROOTPARTYGRPSBC_FH_IDX
    , NOBASKETSIDEALLOC_FH_IDX
    , NOCROSSLEGS_FH_IDX
    , NOENRICHMENTRULES_FH_IDX
    , NOEVENTS_FH_IDX
    , NOFILLS_FH_IDX
    , NOINSTRATTRIB_FH_IDX
    , NOINSTRMTMATCHSIDES_FH_IDX
    , NOLEGEXECS_FH_IDX
    , NOLEGS_FH_IDX
    , NOMMPARAMETERS_FH_IDX
    , NONOTAFFECTEDORDERS_FH_IDX
    , NONOTAFFECTEDSECURITIES_FH_IDX
    , NOORDERBOOKITEMS_FH_IDX
    , NOPARTYDETAILS_FH_IDX
    , NOQUOTEENTRIES_FH_IDX
    , NOQUOTEEVENTS_FH_IDX
    , NOQUOTEEVENTSINDEX_FH_IDX
    , NOQUOTESIDEENTRIES_FH_IDX
    , NORISKLIMITS_FH_IDX
    , NORISKLIMITSQTY_FH_IDX
    , NOSRQSRELATEDTRADEIDS_FH_IDX
    , NOSESSIONS_FH_IDX
    , NOSIDEALLOCS_FH_IDX
    , NOSIDES_FH_IDX
    , NOTARGETPARTYIDS_FH_IDX
    , NOUNDERLYINGSTIPS_FH_IDX
    , NOTAFFORIGCLORDID_FH_IDX
    , NOTAFFECTEDORDERID_FH_IDX
    , NOTAFFECTEDSECURITYID_FH_IDX
    , NOTIFICATIONIN_FH_IDX
    , NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX
    , NUMBEROFRESPONDENTS_FH_IDX
    , NUMBEROFSECURITIES_FH_IDX
    , OFFERPX_FH_IDX
    , OFFERSIZE_FH_IDX
    , OPTATTRIBUTE_FH_IDX
    , ORDSTATUS_FH_IDX
    , ORDTYPE_FH_IDX
    , ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX
    , ORDERATTRIBUTERISKREDUCTION_FH_IDX
    , ORDERCATEGORY_FH_IDX
    , ORDERID_FH_IDX
    , ORDERORIGINATION_FH_IDX
    , ORDERQTY_FH_IDX
    , ORDERQTYDISCLOSUREINSTRUCTION_FH_IDX
    , ORDERROUTINGINDICATOR_FH_IDX
    , ORDERSIDE_FH_IDX
    , ORIGCLORDID_FH_IDX
    , ORIGTIME_FH_IDX
    , ORIGTRADEID_FH_IDX
    , OWNERSHIPINDICATOR_FH_IDX
    , PACKAGEID_FH_IDX
    , PARTITIONID_FH_IDX
    , PARTYACTIONTYPE_FH_IDX
    , PARTYDETAILDESKID_FH_IDX
    , PARTYDETAILEXECUTINGTRADER_FH_IDX
    , PARTYDETAILEXECUTINGUNIT_FH_IDX
    , PARTYDETAILIDEXECUTINGTRADER_FH_IDX
    , PARTYDETAILIDEXECUTINGUNIT_FH_IDX
    , PARTYDETAILROLEQUALIFIER_FH_IDX
    , PARTYDETAILSTATUS_FH_IDX
    , PARTYDETAILSTATUSINFORMATION_FH_IDX
    , PARTYENTERINGTRADER_FH_IDX
    , PARTYEXECUTINGFIRM_FH_IDX
    , PARTYEXECUTINGTRADER_FH_IDX
    , PARTYEXECUTINGUNIT_FH_IDX
    , PARTYIDBENEFICIARY_FH_IDX
    , PARTYIDCLIENTID_FH_IDX
    , PARTYIDENTERINGFIRM_FH_IDX
    , PARTYIDENTERINGTRADER_FH_IDX
    , PARTYIDEXECUTINGTRADER_FH_IDX
    , PARTYIDEXECUTINGUNIT_FH_IDX
    , PARTYIDLOCATIONID_FH_IDX
    , PARTYIDORDERORIGINATIONFIRM_FH_IDX
    , PARTYIDORIGINATIONMARKET_FH_IDX
    , PARTYIDPOSITIONACCOUNT_FH_IDX
    , PARTYIDSESSIONID_FH_IDX
    , PARTYIDSETTLEMENTLOCATION_FH_IDX
    , PARTYIDTAKEUPTRADINGFIRM_FH_IDX
    , PARTYIDINVESTMENTDECISIONMAKER_FH_IDX
    , PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX
    , PARTYORDERORIGINATIONDISCLOSUREINSTRUCTION_FH_IDX
    , PARTYORDERORIGINATIONTRADER_FH_IDX
    , PARTYSUBIDTYPE_FH_IDX
    , PASSWORD_FH_IDX
    , PCTCOUNT_FH_IDX
    , POSITIONEFFECT_FH_IDX
    , PRICE_FH_IDX
    , PRICEDISCLOSUREINSTRUCTION_FH_IDX
    , PRICEVALIDITYCHECKTYPE_FH_IDX
    , PRODUCTCOMPLEX_FH_IDX
    , PUTORCALL_FH_IDX
    , QUOTECANCELTYPE_FH_IDX
    , QUOTEENTRYREJECTREASON_FH_IDX
    , QUOTEENTRYSTATUS_FH_IDX
    , QUOTEEVENTEXECID_FH_IDX
    , QUOTEEVENTLIQUIDITYIND_FH_IDX
    , QUOTEEVENTMATCHID_FH_IDX
    , QUOTEEVENTPX_FH_IDX
    , QUOTEEVENTQTY_FH_IDX
    , QUOTEEVENTREASON_FH_IDX
    , QUOTEEVENTSIDE_FH_IDX
    , QUOTEEVENTTYPE_FH_IDX
    , QUOTEID_FH_IDX
    , QUOTEINSTRUCTION_FH_IDX
    , QUOTEMSGID_FH_IDX
    , QUOTEREFPRICE_FH_IDX
    , QUOTEREQID_FH_IDX
    , QUOTERESPONSEID_FH_IDX
    , QUOTESIZETYPE_FH_IDX
    , QUOTESTATUS_FH_IDX
    , QUOTESUBTYPE_FH_IDX
    , QUOTETYPE_FH_IDX
    , QUOTINGSTATUS_FH_IDX
    , REFAPPLID_FH_IDX
    , REFAPPLLASTMSGID_FH_IDX
    , REFAPPLLASTSEQNUM_FH_IDX
    , REFAPPLSUBID_FH_IDX
    , REGULATORYTRADEID_FH_IDX
    , RELATEDCLOSEPRICE_FH_IDX
    , RELATEDMARKETSEGMENTID_FH_IDX
    , RELATEDPRODUCTCOMPLEX_FH_IDX
    , RELATEDSECURITYID_FH_IDX
    , RELATEDSYMBOL_FH_IDX
    , RELATEDTRADEID_FH_IDX
    , RELATEDTRADEQUANTITY_FH_IDX
    , REQUESTTIME_FH_IDX
    , REQUESTINGPARTYCLEARINGFIRM_FH_IDX
    , REQUESTINGPARTYENTERINGFIRM_FH_IDX
    , REQUESTINGPARTYIDENTERINGFIRM_FH_IDX
    , REQUESTINGPARTYIDEXECUTINGSYSTEM_FH_IDX
    , REQUESTINGPARTYIDEXECUTINGTRADER_FH_IDX
    , REQUESTINGPARTYSUBIDTYPE_FH_IDX
    , RESPONDENTTYPE_FH_IDX
    , RESPONSEIN_FH_IDX
    , REVERSALAPPROVALTIME_FH_IDX
    , REVERSALCANCELLATIONREASON_FH_IDX
    , REVERSALINDICATOR_FH_IDX
    , REVERSALINITIATIONTIME_FH_IDX
    , REVERSALREASONTEXT_FH_IDX
    , RISKLIMITACTION_FH_IDX
    , RISKLIMITGROUP_FH_IDX
    , RISKLIMITNETPOSITIONQTY_FH_IDX
    , RISKLIMITOPENQTY_FH_IDX
    , RISKLIMITPLATFORM_FH_IDX
    , RISKLIMITQTY_FH_IDX
    , RISKLIMITREPORTID_FH_IDX
    , RISKLIMITREQUESTINGPARTYROLE_FH_IDX
    , RISKLIMITTYPE_FH_IDX
    , RISKLIMITVIOLATIONINDICATOR_FH_IDX
    , ROOTPARTYCLEARINGFIRM_FH_IDX
    , ROOTPARTYCLEARINGORGANIZATION_FH_IDX
    , ROOTPARTYCONTRAFIRM_FH_IDX
    , ROOTPARTYCONTRATRADER_FH_IDX
    , ROOTPARTYENTERINGTRADER_FH_IDX
    , ROOTPARTYEXECUTINGFIRM_FH_IDX
    , ROOTPARTYEXECUTINGTRADER_FH_IDX
    , ROOTPARTYIDBENEFICIARY_FH_IDX
    , ROOTPARTYIDCLEARINGUNIT_FH_IDX
    , ROOTPARTYIDCLIENTID_FH_IDX
    , ROOTPARTYIDEXECUTINGTRADER_FH_IDX
    , ROOTPARTYIDEXECUTINGUNIT_FH_IDX
    , ROOTPARTYIDEXECUTIONVENUE_FH_IDX
    , ROOTPARTYIDINVESTMENTDECISIONMAKER_FH_IDX
    , ROOTPARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX
    , ROOTPARTYIDORDERORIGINATIONFIRM_FH_IDX
    , ROOTPARTYIDPOSITIONACCOUNT_FH_IDX
    , ROOTPARTYIDSESSIONID_FH_IDX
    , ROOTPARTYIDTAKEUPTRADINGFIRM_FH_IDX
    , ROOTPARTYSUBIDTYPE_FH_IDX
    , SRQSRELATEDTRADEID_FH_IDX
    , SECONDARYQUOTEID_FH_IDX
    , SECONDARYTRADEID_FH_IDX
    , SECURITYID_FH_IDX
    , SECURITYRESPONSEID_FH_IDX
    , SECURITYSUBTYPE_FH_IDX
    , SELECTIVEREQUESTFORQUOTESERVICESTATUS_FH_IDX
    , SELECTIVEREQUESTFORQUOTESERVICETRADEDATE_FH_IDX
    , SENDERSUBID_FH_IDX
    , SENDINGTIME_FH_IDX
    , SESSIONINSTANCEID_FH_IDX
    , SESSIONMODE_FH_IDX
    , SESSIONREJECTREASON_FH_IDX
    , SESSIONSTATUS_FH_IDX
    , SESSIONSUBMODE_FH_IDX
    , SETTLMETHOD_FH_IDX
    , SHOWLASTDEALONCLOSURE_FH_IDX
    , SIDE_FH_IDX
    , SIDECOMPLIANCETEXT_FH_IDX
    , SIDEDISCLOSUREINSTRUCTION_FH_IDX
    , SIDELASTPX_FH_IDX
    , SIDELASTQTY_FH_IDX
    , SIDELIQUIDITYIND_FH_IDX
    , SIDEMARKETSEGMENTID_FH_IDX
    , SIDETRADEID_FH_IDX
    , SIDETRDSUBTYP_FH_IDX
    , SIMPLESECURITYID_FH_IDX
    , SKIPVALIDATIONS_FH_IDX
    , STOPPX_FH_IDX
    , STRATEGYLINKID_FH_IDX
    , STRIKEPRICE_FH_IDX
    , SUBSCRIPTIONSCOPE_FH_IDX
    , SWAPCLEARER_FH_IDX
    , T7ENTRYSERVICERTMSTATUS_FH_IDX
    , T7ENTRYSERVICERTMTRADEDATE_FH_IDX
    , T7ENTRYSERVICESTATUS_FH_IDX
    , T7ENTRYSERVICETRADEDATE_FH_IDX
    , TESENRICHMENTRULEID_FH_IDX
    , TESEXECID_FH_IDX
    , TARGETPARTYENTERINGTRADER_FH_IDX
    , TARGETPARTYEXECUTINGFIRM_FH_IDX
    , TARGETPARTYEXECUTINGTRADER_FH_IDX
    , TARGETPARTYIDDESKID_FH_IDX
    , TARGETPARTYIDEXECUTINGTRADER_FH_IDX
    , TARGETPARTYIDSESSIONID_FH_IDX
    , TEMPLATEID_FH_IDX
    , THROTTLEDISCONNECTLIMIT_FH_IDX
    , THROTTLENOMSGS_FH_IDX
    , THROTTLETIMEINTERVAL_FH_IDX
    , TIMEINFORCE_FH_IDX
    , TOTNUMTRADEREPORTS_FH_IDX
    , TRADSESEVENT_FH_IDX
    , TRADSESMODE_FH_IDX
    , TRADEALLOCSTATUS_FH_IDX
    , TRADEDATE_FH_IDX
    , TRADEID_FH_IDX
    , TRADEMANAGERSTATUS_FH_IDX
    , TRADEMANAGERTRADEDATE_FH_IDX
    , TRADEPLATFORM_FH_IDX
    , TRADEPUBLISHINDICATOR_FH_IDX
    , TRADEREPORTID_FH_IDX
    , TRADEREPORTTEXT_FH_IDX
    , TRADEREPORTTYPE_FH_IDX
    , TRADEREQUESTRESULT_FH_IDX
    , TRADETOQUOTERATIO_FH_IDX
    , TRADETOQUOTERATIOPOSITION_FH_IDX
    , TRADETOQUOTERATIORANKING_FH_IDX
    , TRADETOREQUESTRATIO_FH_IDX
    , TRADINGCAPACITY_FH_IDX
    , TRADINGSESSIONSUBID_FH_IDX
    , TRANSBKDTIME_FH_IDX
    , TRANSACTTIME_FH_IDX
    , TRANSACTIONDELAYINDICATOR_FH_IDX
    , TRANSFERREASON_FH_IDX
    , TRDMATCHID_FH_IDX
    , TRDREGTSENTRYTIME_FH_IDX
    , TRDREGTSEXECUTIONTIME_FH_IDX
    , TRDREGTSTIMEIN_FH_IDX
    , TRDREGTSTIMEOUT_FH_IDX
    , TRDREGTSTIMEPRIORITY_FH_IDX
    , TRDRPTSTATUS_FH_IDX
    , TRDTYPE_FH_IDX
    , TRIGGERED_FH_IDX
    , UNDERLYINGCURRENCY_FH_IDX
    , UNDERLYINGDELTAPERCENTAGE_FH_IDX
    , UNDERLYINGEFFECTIVEDELTAPERCENTAGE_FH_IDX
    , UNDERLYINGISSUER_FH_IDX
    , UNDERLYINGMATURITYDATE_FH_IDX
    , UNDERLYINGPRICESTIPVALUE_FH_IDX
    , UNDERLYINGPX_FH_IDX
    , UNDERLYINGQTY_FH_IDX
    , UNDERLYINGSECURITYDESC_FH_IDX
    , UNDERLYINGSECURITYID_FH_IDX
    , UNDERLYINGSETTLEMENTDATE_FH_IDX
    , UNDERLYINGSTIPTYPE_FH_IDX
    , UNDERLYINGSTIPVALUE_FH_IDX
    , USERSTATUS_FH_IDX
    , USERNAME_FH_IDX
    , VALIDUNTILTIME_FH_IDX
    , VALUECHECKTYPEMINLOTSIZE_FH_IDX
    , VALUECHECKTYPEVALUE_FH_IDX
    , VARTEXT_FH_IDX
    , VARTEXTLEN_FH_IDX
    , VEGA_FH_IDX
};
static const value_string template_id_vals[] = { // TemplateID
    { 10000, "LogonRequest" },
    { 10001, "LogonResponse" },
    { 10002, "LogoutRequest" },
    { 10003, "LogoutResponse" },
    { 10004, "Unknown" },
    { 10005, "SubscribeResponse" },
    { 10006, "UnsubscribeRequest" },
    { 10007, "UnsubscribeResponse" },
    { 10008, "RetransmitRequest" },
    { 10009, "RetransmitResponse" },
    { 10010, "Reject" },
    { 10011, "Heartbeat" },
    { 10012, "ForcedLogoutNotification" },
    { 10013, "Unknown" },
    { 10014, "Unknown" },
    { 10015, "Unknown" },
    { 10016, "Unknown" },
    { 10017, "Unknown" },
    { 10018, "UserLoginRequest" },
    { 10019, "UserLoginResponse" },
    { 10020, "Unknown" },
    { 10021, "Unknown" },
    { 10022, "Unknown" },
    { 10023, "HeartbeatNotification" },
    { 10024, "UserLogoutResponse" },
    { 10025, "SubscribeRequest" },
    { 10026, "RetransmitMEMessageRequest" },
    { 10027, "RetransmitMEMessageResponse" },
    { 10028, "ThrottleUpdateNotification" },
    { 10029, "UserLogoutRequest" },
    { 10030, "ServiceAvailabilityBroadcast" },
    { 10031, "NewsBroadcast" },
    { 10032, "BroadcastErrorNotification" },
    { 10033, "RiskNotificationBroadcast" },
    { 10034, "PartyEntitlementsUpdateReport" },
    { 10035, "InquireSessionListRequest" },
    { 10036, "InquireSessionListResponse" },
    { 10037, "LegalNotificationBroadcast" },
    { 10038, "InquireUserRequest" },
    { 10039, "InquireUserResponse" },
    { 10040, "InquireEnrichmentRuleIDListRequest" },
    { 10041, "InquireEnrichmentRuleIDListResponse" },
    { 10042, "PartyActionReport" },
    { 10043, "ForcedUserLogoutNotification" },
    { 10044, "ServiceAvailabilityMarketBroadcast" },
    { 10045, "Unknown" },
    { 10046, "Unknown" },
    { 10047, "Unknown" },
    { 10048, "Unknown" },
    { 10049, "Unknown" },
    { 10050, "Unknown" },
    { 10051, "Unknown" },
    { 10052, "Unknown" },
    { 10053, "Unknown" },
    { 10054, "Unknown" },
    { 10055, "Unknown" },
    { 10056, "Unknown" },
    { 10057, "Unknown" },
    { 10058, "Unknown" },
    { 10059, "Unknown" },
    { 10060, "Unknown" },
    { 10061, "Unknown" },
    { 10062, "Unknown" },
    { 10063, "Unknown" },
    { 10064, "Unknown" },
    { 10065, "Unknown" },
    { 10066, "Unknown" },
    { 10067, "Unknown" },
    { 10068, "Unknown" },
    { 10069, "Unknown" },
    { 10070, "Unknown" },
    { 10071, "Unknown" },
    { 10072, "Unknown" },
    { 10073, "Unknown" },
    { 10074, "Unknown" },
    { 10075, "Unknown" },
    { 10076, "Unknown" },
    { 10077, "Unknown" },
    { 10078, "Unknown" },
    { 10079, "Unknown" },
    { 10080, "Unknown" },
    { 10081, "Unknown" },
    { 10082, "Unknown" },
    { 10083, "Unknown" },
    { 10084, "Unknown" },
    { 10085, "Unknown" },
    { 10086, "Unknown" },
    { 10087, "Unknown" },
    { 10088, "Unknown" },
    { 10089, "Unknown" },
    { 10090, "Unknown" },
    { 10091, "Unknown" },
    { 10092, "Unknown" },
    { 10093, "Unknown" },
    { 10094, "Unknown" },
    { 10095, "Unknown" },
    { 10096, "Unknown" },
    { 10097, "Unknown" },
    { 10098, "Unknown" },
    { 10099, "Unknown" },
    { 10100, "NewOrderSingleRequest" },
    { 10101, "NewOrderResponse" },
    { 10102, "NewOrderNRResponse" },
    { 10103, "OrderExecResponse" },
    { 10104, "OrderExecNotification" },
    { 10105, "Unknown" },
    { 10106, "ModifyOrderSingleRequest" },
    { 10107, "ModifyOrderResponse" },
    { 10108, "ModifyOrderNRResponse" },
    { 10109, "DeleteOrderSingleRequest" },
    { 10110, "DeleteOrderResponse" },
    { 10111, "DeleteOrderNRResponse" },
    { 10112, "DeleteOrderBroadcast" },
    { 10113, "NewOrderComplexRequest" },
    { 10114, "ModifyOrderComplexRequest" },
    { 10115, "Unknown" },
    { 10116, "Unknown" },
    { 10117, "OrderExecReportBroadcast" },
    { 10118, "CrossRequest" },
    { 10119, "CrossRequestResponse" },
    { 10120, "DeleteAllOrderRequest" },
    { 10121, "DeleteAllOrderResponse" },
    { 10122, "DeleteAllOrderBroadcast" },
    { 10123, "DeleteOrderComplexRequest" },
    { 10124, "DeleteAllOrderNRResponse" },
    { 10125, "NewOrderSingleShortRequest" },
    { 10126, "ModifyOrderSingleShortRequest" },
    { 10127, "Unknown" },
    { 10128, "Unknown" },
    { 10129, "NewOrderComplexShortRequest" },
    { 10130, "ModifyOrderComplexShortRequest" },
    { 10131, "EnterCLIPRequest" },
    { 10132, "DeleteCLIPRequest" },
    { 10133, "CLIPResponse" },
    { 10134, "CLIPDeletionNotification" },
    { 10135, "CLIPExecutionNotification" },
    { 10136, "Unknown" },
    { 10137, "Unknown" },
    { 10138, "Unknown" },
    { 10139, "Unknown" },
    { 10140, "Unknown" },
    { 10141, "Unknown" },
    { 10142, "Unknown" },
    { 10143, "Unknown" },
    { 10144, "Unknown" },
    { 10145, "Unknown" },
    { 10146, "Unknown" },
    { 10147, "Unknown" },
    { 10148, "Unknown" },
    { 10149, "Unknown" },
    { 10150, "Unknown" },
    { 10151, "Unknown" },
    { 10152, "Unknown" },
    { 10153, "Unknown" },
    { 10154, "Unknown" },
    { 10155, "Unknown" },
    { 10156, "Unknown" },
    { 10157, "Unknown" },
    { 10158, "Unknown" },
    { 10159, "Unknown" },
    { 10160, "Unknown" },
    { 10161, "Unknown" },
    { 10162, "Unknown" },
    { 10163, "Unknown" },
    { 10164, "Unknown" },
    { 10165, "Unknown" },
    { 10166, "Unknown" },
    { 10167, "Unknown" },
    { 10168, "Unknown" },
    { 10169, "Unknown" },
    { 10170, "Unknown" },
    { 10171, "Unknown" },
    { 10172, "Unknown" },
    { 10173, "Unknown" },
    { 10174, "Unknown" },
    { 10175, "Unknown" },
    { 10176, "Unknown" },
    { 10177, "Unknown" },
    { 10178, "Unknown" },
    { 10179, "Unknown" },
    { 10180, "Unknown" },
    { 10181, "Unknown" },
    { 10182, "Unknown" },
    { 10183, "Unknown" },
    { 10184, "Unknown" },
    { 10185, "Unknown" },
    { 10186, "Unknown" },
    { 10187, "Unknown" },
    { 10188, "Unknown" },
    { 10189, "Unknown" },
    { 10190, "Unknown" },
    { 10191, "Unknown" },
    { 10192, "Unknown" },
    { 10193, "Unknown" },
    { 10194, "Unknown" },
    { 10195, "Unknown" },
    { 10196, "Unknown" },
    { 10197, "Unknown" },
    { 10198, "Unknown" },
    { 10199, "Unknown" },
    { 10200, "Unknown" },
    { 10201, "Unknown" },
    { 10202, "Unknown" },
    { 10203, "Unknown" },
    { 10204, "Unknown" },
    { 10205, "Unknown" },
    { 10206, "Unknown" },
    { 10207, "Unknown" },
    { 10208, "Unknown" },
    { 10209, "Unknown" },
    { 10210, "Unknown" },
    { 10211, "Unknown" },
    { 10212, "Unknown" },
    { 10213, "Unknown" },
    { 10214, "Unknown" },
    { 10215, "Unknown" },
    { 10216, "Unknown" },
    { 10217, "Unknown" },
    { 10218, "Unknown" },
    { 10219, "Unknown" },
    { 10220, "Unknown" },
    { 10221, "Unknown" },
    { 10222, "Unknown" },
    { 10223, "Unknown" },
    { 10224, "Unknown" },
    { 10225, "Unknown" },
    { 10226, "Unknown" },
    { 10227, "Unknown" },
    { 10228, "Unknown" },
    { 10229, "Unknown" },
    { 10230, "Unknown" },
    { 10231, "Unknown" },
    { 10232, "Unknown" },
    { 10233, "Unknown" },
    { 10234, "Unknown" },
    { 10235, "Unknown" },
    { 10236, "Unknown" },
    { 10237, "Unknown" },
    { 10238, "Unknown" },
    { 10239, "Unknown" },
    { 10240, "Unknown" },
    { 10241, "Unknown" },
    { 10242, "Unknown" },
    { 10243, "Unknown" },
    { 10244, "Unknown" },
    { 10245, "Unknown" },
    { 10246, "Unknown" },
    { 10247, "Unknown" },
    { 10248, "Unknown" },
    { 10249, "Unknown" },
    { 10250, "Unknown" },
    { 10251, "Unknown" },
    { 10252, "Unknown" },
    { 10253, "Unknown" },
    { 10254, "Unknown" },
    { 10255, "Unknown" },
    { 10256, "Unknown" },
    { 10257, "Unknown" },
    { 10258, "Unknown" },
    { 10259, "Unknown" },
    { 10260, "Unknown" },
    { 10261, "Unknown" },
    { 10262, "Unknown" },
    { 10263, "Unknown" },
    { 10264, "Unknown" },
    { 10265, "Unknown" },
    { 10266, "Unknown" },
    { 10267, "Unknown" },
    { 10268, "Unknown" },
    { 10269, "Unknown" },
    { 10270, "Unknown" },
    { 10271, "Unknown" },
    { 10272, "Unknown" },
    { 10273, "Unknown" },
    { 10274, "Unknown" },
    { 10275, "Unknown" },
    { 10276, "Unknown" },
    { 10277, "Unknown" },
    { 10278, "Unknown" },
    { 10279, "Unknown" },
    { 10280, "Unknown" },
    { 10281, "Unknown" },
    { 10282, "Unknown" },
    { 10283, "Unknown" },
    { 10284, "Unknown" },
    { 10285, "Unknown" },
    { 10286, "Unknown" },
    { 10287, "Unknown" },
    { 10288, "Unknown" },
    { 10289, "Unknown" },
    { 10290, "Unknown" },
    { 10291, "Unknown" },
    { 10292, "Unknown" },
    { 10293, "Unknown" },
    { 10294, "Unknown" },
    { 10295, "Unknown" },
    { 10296, "Unknown" },
    { 10297, "Unknown" },
    { 10298, "Unknown" },
    { 10299, "Unknown" },
    { 10300, "Unknown" },
    { 10301, "AddComplexInstrumentRequest" },
    { 10302, "AddComplexInstrumentResponse" },
    { 10303, "MMParameterDefinitionRequest" },
    { 10304, "MMParameterDefinitionResponse" },
    { 10305, "InquireMMParameterRequest" },
    { 10306, "InquireMMParameterResponse" },
    { 10307, "TradingSessionStatusBroadcast" },
    { 10308, "DeleteAllOrderQuoteEventBroadcast" },
    { 10309, "AddFlexibleInstrumentRequest" },
    { 10310, "AddFlexibleInstrumentResponse" },
    { 10311, "InquirePreTradeRiskLimitsRequest" },
    { 10312, "PreTradeRiskLimitsDefinitionRequest" },
    { 10313, "PreTradeRiskLimitResponse" },
    { 10314, "Unknown" },
    { 10315, "Unknown" },
    { 10316, "Unknown" },
    { 10317, "Unknown" },
    { 10318, "Unknown" },
    { 10319, "Unknown" },
    { 10320, "PingRequest" },
    { 10321, "PingResponse" },
    { 10322, "Unknown" },
    { 10323, "Unknown" },
    { 10324, "Unknown" },
    { 10325, "Unknown" },
    { 10326, "Unknown" },
    { 10327, "Unknown" },
    { 10328, "Unknown" },
    { 10329, "Unknown" },
    { 10330, "Unknown" },
    { 10331, "Unknown" },
    { 10332, "Unknown" },
    { 10333, "Unknown" },
    { 10334, "Unknown" },
    { 10335, "Unknown" },
    { 10336, "Unknown" },
    { 10337, "Unknown" },
    { 10338, "Unknown" },
    { 10339, "Unknown" },
    { 10340, "Unknown" },
    { 10341, "Unknown" },
    { 10342, "Unknown" },
    { 10343, "Unknown" },
    { 10344, "Unknown" },
    { 10345, "Unknown" },
    { 10346, "Unknown" },
    { 10347, "Unknown" },
    { 10348, "Unknown" },
    { 10349, "Unknown" },
    { 10350, "Unknown" },
    { 10351, "Unknown" },
    { 10352, "Unknown" },
    { 10353, "Unknown" },
    { 10354, "Unknown" },
    { 10355, "Unknown" },
    { 10356, "Unknown" },
    { 10357, "Unknown" },
    { 10358, "Unknown" },
    { 10359, "Unknown" },
    { 10360, "Unknown" },
    { 10361, "Unknown" },
    { 10362, "Unknown" },
    { 10363, "Unknown" },
    { 10364, "Unknown" },
    { 10365, "Unknown" },
    { 10366, "Unknown" },
    { 10367, "Unknown" },
    { 10368, "Unknown" },
    { 10369, "Unknown" },
    { 10370, "Unknown" },
    { 10371, "Unknown" },
    { 10372, "Unknown" },
    { 10373, "Unknown" },
    { 10374, "Unknown" },
    { 10375, "Unknown" },
    { 10376, "Unknown" },
    { 10377, "Unknown" },
    { 10378, "Unknown" },
    { 10379, "Unknown" },
    { 10380, "Unknown" },
    { 10381, "Unknown" },
    { 10382, "Unknown" },
    { 10383, "Unknown" },
    { 10384, "Unknown" },
    { 10385, "Unknown" },
    { 10386, "Unknown" },
    { 10387, "Unknown" },
    { 10388, "Unknown" },
    { 10389, "Unknown" },
    { 10390, "Unknown" },
    { 10391, "Unknown" },
    { 10392, "Unknown" },
    { 10393, "Unknown" },
    { 10394, "Unknown" },
    { 10395, "Unknown" },
    { 10396, "Unknown" },
    { 10397, "Unknown" },
    { 10398, "Unknown" },
    { 10399, "Unknown" },
    { 10400, "Unknown" },
    { 10401, "RFQRequest" },
    { 10402, "RFQResponse" },
    { 10403, "QuoteActivationRequest" },
    { 10404, "QuoteActivationResponse" },
    { 10405, "MassQuoteRequest" },
    { 10406, "MassQuoteResponse" },
    { 10407, "QuoteExecutionReport" },
    { 10408, "DeleteAllQuoteRequest" },
    { 10409, "DeleteAllQuoteResponse" },
    { 10410, "DeleteAllQuoteBroadcast" },
    { 10411, "QuoteActivationNotification" },
    { 10412, "Unknown" },
    { 10413, "Unknown" },
    { 10414, "Unknown" },
    { 10415, "Unknown" },
    { 10416, "Unknown" },
    { 10417, "Unknown" },
    { 10418, "Unknown" },
    { 10419, "Unknown" },
    { 10420, "Unknown" },
    { 10421, "Unknown" },
    { 10422, "Unknown" },
    { 10423, "Unknown" },
    { 10424, "Unknown" },
    { 10425, "Unknown" },
    { 10426, "Unknown" },
    { 10427, "Unknown" },
    { 10428, "Unknown" },
    { 10429, "Unknown" },
    { 10430, "Unknown" },
    { 10431, "Unknown" },
    { 10432, "Unknown" },
    { 10433, "Unknown" },
    { 10434, "Unknown" },
    { 10435, "Unknown" },
    { 10436, "Unknown" },
    { 10437, "Unknown" },
    { 10438, "Unknown" },
    { 10439, "Unknown" },
    { 10440, "Unknown" },
    { 10441, "Unknown" },
    { 10442, "Unknown" },
    { 10443, "Unknown" },
    { 10444, "Unknown" },
    { 10445, "Unknown" },
    { 10446, "Unknown" },
    { 10447, "Unknown" },
    { 10448, "Unknown" },
    { 10449, "Unknown" },
    { 10450, "Unknown" },
    { 10451, "Unknown" },
    { 10452, "Unknown" },
    { 10453, "Unknown" },
    { 10454, "Unknown" },
    { 10455, "Unknown" },
    { 10456, "Unknown" },
    { 10457, "Unknown" },
    { 10458, "Unknown" },
    { 10459, "Unknown" },
    { 10460, "Unknown" },
    { 10461, "Unknown" },
    { 10462, "Unknown" },
    { 10463, "Unknown" },
    { 10464, "Unknown" },
    { 10465, "Unknown" },
    { 10466, "Unknown" },
    { 10467, "Unknown" },
    { 10468, "Unknown" },
    { 10469, "Unknown" },
    { 10470, "Unknown" },
    { 10471, "Unknown" },
    { 10472, "Unknown" },
    { 10473, "Unknown" },
    { 10474, "Unknown" },
    { 10475, "Unknown" },
    { 10476, "Unknown" },
    { 10477, "Unknown" },
    { 10478, "Unknown" },
    { 10479, "Unknown" },
    { 10480, "Unknown" },
    { 10481, "Unknown" },
    { 10482, "Unknown" },
    { 10483, "Unknown" },
    { 10484, "Unknown" },
    { 10485, "Unknown" },
    { 10486, "Unknown" },
    { 10487, "Unknown" },
    { 10488, "Unknown" },
    { 10489, "Unknown" },
    { 10490, "Unknown" },
    { 10491, "Unknown" },
    { 10492, "Unknown" },
    { 10493, "Unknown" },
    { 10494, "Unknown" },
    { 10495, "Unknown" },
    { 10496, "Unknown" },
    { 10497, "Unknown" },
    { 10498, "Unknown" },
    { 10499, "Unknown" },
    { 10500, "TradeBroadcast" },
    { 10501, "TMTradingSessionStatusBroadcast" },
    { 10502, "Unknown" },
    { 10503, "Unknown" },
    { 10504, "Unknown" },
    { 10505, "Unknown" },
    { 10506, "Unknown" },
    { 10507, "Unknown" },
    { 10508, "Unknown" },
    { 10509, "Unknown" },
    { 10510, "Unknown" },
    { 10511, "Unknown" },
    { 10512, "Unknown" },
    { 10513, "Unknown" },
    { 10514, "Unknown" },
    { 10515, "Unknown" },
    { 10516, "Unknown" },
    { 10517, "Unknown" },
    { 10518, "Unknown" },
    { 10519, "Unknown" },
    { 10520, "Unknown" },
    { 10521, "Unknown" },
    { 10522, "Unknown" },
    { 10523, "Unknown" },
    { 10524, "Unknown" },
    { 10525, "Unknown" },
    { 10526, "Unknown" },
    { 10527, "Unknown" },
    { 10528, "Unknown" },
    { 10529, "Unknown" },
    { 10530, "Unknown" },
    { 10531, "Unknown" },
    { 10532, "Unknown" },
    { 10533, "Unknown" },
    { 10534, "Unknown" },
    { 10535, "Unknown" },
    { 10536, "Unknown" },
    { 10537, "Unknown" },
    { 10538, "Unknown" },
    { 10539, "Unknown" },
    { 10540, "Unknown" },
    { 10541, "Unknown" },
    { 10542, "Unknown" },
    { 10543, "Unknown" },
    { 10544, "Unknown" },
    { 10545, "Unknown" },
    { 10546, "Unknown" },
    { 10547, "Unknown" },
    { 10548, "Unknown" },
    { 10549, "Unknown" },
    { 10550, "Unknown" },
    { 10551, "Unknown" },
    { 10552, "Unknown" },
    { 10553, "Unknown" },
    { 10554, "Unknown" },
    { 10555, "Unknown" },
    { 10556, "Unknown" },
    { 10557, "Unknown" },
    { 10558, "Unknown" },
    { 10559, "Unknown" },
    { 10560, "Unknown" },
    { 10561, "Unknown" },
    { 10562, "Unknown" },
    { 10563, "Unknown" },
    { 10564, "Unknown" },
    { 10565, "Unknown" },
    { 10566, "Unknown" },
    { 10567, "Unknown" },
    { 10568, "Unknown" },
    { 10569, "Unknown" },
    { 10570, "Unknown" },
    { 10571, "Unknown" },
    { 10572, "Unknown" },
    { 10573, "Unknown" },
    { 10574, "Unknown" },
    { 10575, "Unknown" },
    { 10576, "Unknown" },
    { 10577, "Unknown" },
    { 10578, "Unknown" },
    { 10579, "Unknown" },
    { 10580, "Unknown" },
    { 10581, "Unknown" },
    { 10582, "Unknown" },
    { 10583, "Unknown" },
    { 10584, "Unknown" },
    { 10585, "Unknown" },
    { 10586, "Unknown" },
    { 10587, "Unknown" },
    { 10588, "Unknown" },
    { 10589, "Unknown" },
    { 10590, "Unknown" },
    { 10591, "Unknown" },
    { 10592, "Unknown" },
    { 10593, "Unknown" },
    { 10594, "Unknown" },
    { 10595, "Unknown" },
    { 10596, "Unknown" },
    { 10597, "Unknown" },
    { 10598, "Unknown" },
    { 10599, "Unknown" },
    { 10600, "EnterTESTradeRequest" },
    { 10601, "ModifyTESTradeRequest" },
    { 10602, "DeleteTESTradeRequest" },
    { 10603, "ApproveTESTradeRequest" },
    { 10604, "TESBroadcast" },
    { 10605, "Unknown" },
    { 10606, "TESDeleteBroadcast" },
    { 10607, "TESApproveBroadcast" },
    { 10608, "Unknown" },
    { 10609, "Unknown" },
    { 10610, "TESExecutionBroadcast" },
    { 10611, "TESResponse" },
    { 10612, "UploadTESTradeRequest" },
    { 10613, "TESUploadBroadcast" },
    { 10614, "TESTradeBroadcast" },
    { 10615, "TESTradingSessionStatusBroadcast" },
    { 10616, "TESCompressionRunStatusRequest" },
    { 10617, "TESCompressionRunStatusResponse" },
    { 10618, "TESCompressionRunStatusBroadcast" },
    { 10619, "Unknown" },
    { 10620, "EnterBasketTradeRequest" },
    { 10621, "ModifyBasketTradeRequest" },
    { 10622, "DeleteBasketTradeRequest" },
    { 10623, "ApproveBasketTradeRequest" },
    { 10624, "BasketResponse" },
    { 10625, "BasketBroadcast" },
    { 10626, "BasketDeleteBroadcast" },
    { 10627, "BasketApproveBroadcast" },
    { 10628, "BasketExecutionBroadcast" },
    { 10629, "AmendBasketTradeRequest" },
    { 10630, "ReverseTESTradeRequest" },
    { 10631, "ApproveReverseTESTradeRequest" },
    { 10632, "TESReversalBroadcast" },
    { 10633, "Unknown" },
    { 10634, "Unknown" },
    { 10635, "Unknown" },
    { 10636, "Unknown" },
    { 10637, "Unknown" },
    { 10638, "Unknown" },
    { 10639, "Unknown" },
    { 10640, "Unknown" },
    { 10641, "Unknown" },
    { 10642, "Unknown" },
    { 10643, "Unknown" },
    { 10644, "Unknown" },
    { 10645, "Unknown" },
    { 10646, "Unknown" },
    { 10647, "Unknown" },
    { 10648, "Unknown" },
    { 10649, "Unknown" },
    { 10650, "Unknown" },
    { 10651, "Unknown" },
    { 10652, "Unknown" },
    { 10653, "Unknown" },
    { 10654, "Unknown" },
    { 10655, "Unknown" },
    { 10656, "Unknown" },
    { 10657, "Unknown" },
    { 10658, "Unknown" },
    { 10659, "Unknown" },
    { 10660, "Unknown" },
    { 10661, "Unknown" },
    { 10662, "Unknown" },
    { 10663, "Unknown" },
    { 10664, "Unknown" },
    { 10665, "Unknown" },
    { 10666, "Unknown" },
    { 10667, "Unknown" },
    { 10668, "Unknown" },
    { 10669, "Unknown" },
    { 10670, "Unknown" },
    { 10671, "Unknown" },
    { 10672, "Unknown" },
    { 10673, "Unknown" },
    { 10674, "Unknown" },
    { 10675, "Unknown" },
    { 10676, "Unknown" },
    { 10677, "Unknown" },
    { 10678, "Unknown" },
    { 10679, "Unknown" },
    { 10680, "Unknown" },
    { 10681, "Unknown" },
    { 10682, "Unknown" },
    { 10683, "Unknown" },
    { 10684, "Unknown" },
    { 10685, "Unknown" },
    { 10686, "Unknown" },
    { 10687, "Unknown" },
    { 10688, "Unknown" },
    { 10689, "Unknown" },
    { 10690, "Unknown" },
    { 10691, "Unknown" },
    { 10692, "Unknown" },
    { 10693, "Unknown" },
    { 10694, "Unknown" },
    { 10695, "Unknown" },
    { 10696, "Unknown" },
    { 10697, "Unknown" },
    { 10698, "Unknown" },
    { 10699, "Unknown" },
    { 10700, "SRQSOpenNegotiationRequest" },
    { 10701, "SRQSUpdateNegotiationRequest" },
    { 10702, "SRQSEnterQuoteRequest" },
    { 10703, "SRQSQuoteResponse" },
    { 10704, "SRQSHitQuoteRequest" },
    { 10705, "SRQSDealResponse" },
    { 10706, "SRQSUpdateDealStatusRequest" },
    { 10707, "SRQSQuoteNotification" },
    { 10708, "SRQSCreateDealNotification" },
    { 10709, "SRQSDealNotification" },
    { 10710, "SRQSOpenNegotiationRequesterNotification" },
    { 10711, "SRQSOpenNegotiationNotification" },
    { 10712, "SRQSNegotiationRequesterNotification" },
    { 10713, "SRQSNegotiationNotification" },
    { 10714, "SRQSStatusBroadcast" },
    { 10715, "SRQSNegotiationStatusNotification" },
    { 10716, "Unknown" },
    { 10717, "SRQSQuotingStatusRequest" },
    { 10718, "SRQSInquireSmartRespondentRequest" },
    { 10719, "SRQSInquireSmartRespondentResponse" },
    { 0, NULL }
};
static value_string_ext template_id_vals_ext = VALUE_STRING_EXT_INIT(template_id_vals);
static const value_string appl_id_vals[] = { // ApplID
    { 1, "Trade" },
    { 2, "News" },
    { 3, "Service_availability" },
    { 4, "Session_data" },
    { 5, "Listener_data" },
    { 6, "RiskControl" },
    { 7, "TES_Maintenance" },
    { 8, "TES_Trade" },
    { 9, "SRQS_Maintenance" },
    { 10, "Service_Availability_Market" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext appl_id_vals_ext = VALUE_STRING_EXT_INIT(appl_id_vals);
static const value_string appl_idstatus_vals[] = { // ApplIDStatus
    { 105, "Outbound_conversion_error" },
    { 0xFFFFFFFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string appl_resend_flag_vals[] = { // ApplResendFlag
    { 0, "False" },
    { 1, "True" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string appl_seq_indicator_vals[] = { // ApplSeqIndicator
    { 0, "No_Recovery_Required" },
    { 1, "Recovery_Required" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string appl_seq_status_vals[] = { // ApplSeqStatus
    { 0, "Unavailable" },
    { 1, "Available" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string appl_usage_orders_vals[] = { // ApplUsageOrders
    { 0, "NO_VALUE" },
    { 'A', "Automated" },
    { 'B', "AutoSelect" },
    { 'M', "Manual" },
    { 'N', "None" },
    { 0, NULL }
};
// ApplUsageQuotes aliased by ApplUsageOrders
static const value_string basket_trade_report_type_vals[] = { // BasketTradeReportType
    { 0, "Submit" },
    { 4, "Addendum" },
    { 5, "No_Was_Substitue" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string compression_action_vals[] = { // CompressionAction
    { 1, "Start_Compression" },
    { 2, "Commit_Compression" },
    { 3, "Cancel_Compression" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string compression_status_vals[] = { // CompressionStatus
    { 1, "Open" },
    { 2, "Executed" },
    { 3, "Cancelled_By_User" },
    { 4, "Cancelled_By_System" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string crossed_indicator_vals[] = { // CrossedIndicator
    { 0, "No_crossing" },
    { 1, "Cross_rejected" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string cust_order_handling_inst_vals[] = { // CustOrderHandlingInst
    { 0, "NO_VALUE" },
    { 'C', "C" },
    { 'D', "D" },
    { 'G', "G" },
    { 'H', "H" },
    { 'W', "W" },
    { 'Y', "Y" },
    { 0, NULL }
};
static const value_string delete_reason_vals[] = { // DeleteReason
    { 100, "No_special_reason" },
    { 101, "TAS_Change" },
    { 102, "Intraday_Expiration" },
    { 103, "Risk_Event" },
    { 104, "Stop_Trading" },
    { 105, "Instrument_Deletion" },
    { 106, "Instrument_Suspension" },
    { 107, "PreTrade_Risk_Event" },
    { 108, "Amendment_Reset" },
    { 109, "Amendment_User_Cancelled" },
    { 110, "Compression_Cancelled_By_User" },
    { 111, "Compression_Cancelled_By_System" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext delete_reason_vals_ext = VALUE_STRING_EXT_INIT(delete_reason_vals);
static const value_string effect_on_basket_vals[] = { // EffectOnBasket
    { 1, "Add_Volume" },
    { 2, "Remove_Volume" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string enlight_rfqavg_resp_rate_ranking_vals[] = { // EnlightRFQAvgRespRateRanking
    { 1, "Low" },
    { 2, "Medium" },
    { 3, "High" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// EnlightRFQAvgRespTimeRanking aliased by EnlightRFQAvgRespRateRanking
// EurexVolumeRanking aliased by EnlightRFQAvgRespRateRanking
static const value_string event_type_vals[] = { // EventType
    { 8, "Swap_Start_Date" },
    { 9, "Swap_End_Date" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string exec_inst_vals[] = { // ExecInst
    { 1, "H" },
    { 2, "Q" },
    { 3, "H_Q" },
    { 5, "H_6" },
    { 6, "Q_6" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string exec_restatement_reason_vals[] = { // ExecRestatementReason
    { 1, "Order_Book_Restatement" },
    { 101, "Order_Added" },
    { 102, "Order_Modified" },
    { 103, "Order_Cancelled" },
    { 105, "IOC_Order_Cancelled" },
    { 108, "Book_Order_Executed" },
    { 114, "Changed_to_IOC" },
    { 122, "Instrument_State_Change" },
    { 135, "Market_Order_Triggered" },
    { 149, "CAO_Order_Activated" },
    { 150, "CAO_Order_Inactivated" },
    { 164, "OCO_Order_Triggered" },
    { 172, "Stop_Order_Triggered" },
    { 181, "Ownership_Changed" },
    { 197, "Order_Cancellation_Pending" },
    { 199, "Pending_Cancellation_Executed" },
    { 212, "BOC_Order_Cancelled" },
    { 261, "Panic_Cancel" },
    { 340, "CLIP_Execution" },
    { 343, "CLIP_Arrangement_time_out" },
    { 344, "CLIP_Arrangement_Validation" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext exec_restatement_reason_vals_ext = VALUE_STRING_EXT_INIT(exec_restatement_reason_vals);
static const value_string exec_type_vals[] = { // ExecType
    { 0, "NO_VALUE" },
    { '0', "New" },
    { '4', "Canceled" },
    { '5', "Replaced" },
    { '6', "Pending_Cancel_e" },
    { '9', "Suspended" },
    { 'D', "Restated" },
    { 'F', "Trade" },
    { 'L', "Triggered" },
    { 0, NULL }
};
static value_string_ext exec_type_vals_ext = VALUE_STRING_EXT_INIT(exec_type_vals);
static const value_string executing_trader_qualifier_vals[] = { // ExecutingTraderQualifier
    { 22, "Algo" },
    { 24, "Human" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string exercise_style_vals[] = { // ExerciseStyle
    { 0, "European" },
    { 1, "American" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string fill_liquidity_ind_vals[] = { // FillLiquidityInd
    { 1, "Added_Liquidity" },
    { 2, "Removed_Liquidity" },
    { 4, "Auction" },
    { 5, "Triggered_Stop_Order" },
    { 6, "Triggered_OCO_Order" },
    { 7, "Triggered_Market_Order" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string free_text5_disclosure_instruction_vals[] = { // FreeText5DisclosureInstruction
    { 0, "No" },
    { 1, "Yes" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string hedge_type_vals[] = { // HedgeType
    { 0, "Duration_Hedge" },
    { 1, "Nominal_Hedge" },
    { 2, "Price_Factor_Hedge" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string hedging_instruction_vals[] = { // HedgingInstruction
    { 1, "On_Close" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string implied_market_indicator_vals[] = { // ImpliedMarketIndicator
    { 0, "Not_implied" },
    { 3, "Implied_in_out" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string input_source_vals[] = { // InputSource
    { 1, "Client_Broker" },
    { 2, "Proprietary_Broker" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string instr_attrib_type_vals[] = { // InstrAttribType
    { 5, "Variable_Rate" },
    { 100, "Coupon_Rate" },
    { 101, "Offset_to_the_variable_coupon_rate" },
    { 102, "Swap_Customer_1" },
    { 103, "Swap_Customer_2" },
    { 104, "Cash_Basket_Reference" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string last_fragment_vals[] = { // LastFragment
    { 0, "Not_Last_Message" },
    { 1, "Last_Message" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// LastPxDisclosureInstruction aliased by FreeText5DisclosureInstruction
// LastQtyDisclosureInstruction aliased by FreeText5DisclosureInstruction
// LeavesQtyDisclosureInstruction aliased by FreeText5DisclosureInstruction
// LegInputSource aliased by InputSource
static const value_string leg_position_effect_vals[] = { // LegPositionEffect
    { 0, "NO_VALUE" },
    { 'C', "Close" },
    { 'O', "Open" },
    { 0, NULL }
};
static const value_string leg_security_type_vals[] = { // LegSecurityType
    { 1, "Multileg_Instrument" },
    { 2, "Underlying_Leg" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string leg_side_vals[] = { // LegSide
    { 1, "Buy" },
    { 2, "Sell" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string list_update_action_vals[] = { // ListUpdateAction
    { 0, "NO_VALUE" },
    { 'A', "Add" },
    { 'D', "Delete" },
    { 0, NULL }
};
static const value_string mdbook_type_vals[] = { // MDBookType
    { 1, "TopOfBook" },
    { 2, "PriceDepth" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string mdsub_book_type_vals[] = { // MDSubBookType
    { 1, "Implied" },
    { 2, "VolumeWeightedAverage" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string market_id_vals[] = { // MarketID
    { 1, "XEUR" },
    { 2, "XEEE" },
    { 12, "NODX" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string mass_action_reason_vals[] = { // MassActionReason
    { 0, "No_Special_Reason" },
    { 1, "Stop_Trading" },
    { 2, "Emergency" },
    { 3, "Market_Maker_Protection" },
    { 6, "Session_Loss" },
    { 7, "Duplicate_Session_Login" },
    { 8, "Clearing_Risk_Control" },
    { 100, "Internal_Connection_Loss" },
    { 105, "Product_State_Halt" },
    { 106, "Product_State_Holiday" },
    { 107, "Instrument_Suspended" },
    { 109, "Complex_Instrument_Deletion" },
    { 110, "Volatility_Interruption" },
    { 111, "Product_temporarily_not_tradeable" },
    { 117, "Member_disable" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext mass_action_reason_vals_ext = VALUE_STRING_EXT_INIT(mass_action_reason_vals);
static const value_string mass_action_sub_type_vals[] = { // MassActionSubType
    { 1, "Book_or_Cancel" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string mass_action_type_vals[] = { // MassActionType
    { 1, "Suspend_quotes" },
    { 2, "Release_quotes" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string match_sub_type_vals[] = { // MatchSubType
    { 1, "Opening_Auction" },
    { 2, "Closing_Auction" },
    { 3, "Intraday_Auction" },
    { 4, "Circuit_Breaker_Auction" },
    { 6, "Outside_BBO" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string match_type_vals[] = { // MatchType
    { 3, "Confirmed_Trade_Report" },
    { 4, "Auto_match_incoming" },
    { 5, "Cross_Auction" },
    { 7, "Call_Auction" },
    { 11, "Auto_match_resting" },
    { 13, "Liquidity_Improvement_Cross" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// MatchingEngineStatus aliased by ApplSeqStatus
static const value_string message_event_source_vals[] = { // MessageEventSource
    { 0, "NO_VALUE" },
    { 'A', "Broadcast_to_Approver" },
    { 'I', "Broadcast_to_Initiator" },
    { 'Q', "Broadcast_to_Quote_Submitter" },
    { 'R', "Broadcast_to_Requester" },
    { 0, NULL }
};
static const value_string multi_leg_reporting_type_vals[] = { // MultiLegReportingType
    { 1, "Single_security" },
    { 2, "Individual_leg_of_a_multileg_security" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string multileg_model_vals[] = { // MultilegModel
    { 0, "Predefined_Multileg_Security" },
    { 1, "User_defined_Multleg" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string multileg_price_model_vals[] = { // MultilegPriceModel
    { 0, "Standard" },
    { 1, "UserDefined" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// NumberOfRespDisclosureInstruction aliased by FreeText5DisclosureInstruction
static const value_string ord_status_vals[] = { // OrdStatus
    { 0, "NO_VALUE" },
    { '0', "New" },
    { '1', "Partially_filled" },
    { '2', "Filled" },
    { '4', "Canceled" },
    { '6', "Pending_Cancel" },
    { '9', "Suspended" },
    { 0, NULL }
};
static const value_string ord_type_vals[] = { // OrdType
    { 1, "Market" },
    { 2, "Limit" },
    { 3, "Stop" },
    { 4, "Stop_Limit" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string order_attribute_liquidity_provision_vals[] = { // OrderAttributeLiquidityProvision
    { 0, "N" },
    { 1, "Y" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// OrderAttributeRiskReduction aliased by OrderAttributeLiquidityProvision
static const value_string order_category_vals[] = { // OrderCategory
    { 0, "NO_VALUE" },
    { '1', "Order" },
    { '2', "Quote" },
    { 0, NULL }
};
static const value_string order_origination_vals[] = { // OrderOrigination
    { 5, "Direct_access_or_sponsored_access_customer" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// OrderQtyDisclosureInstruction aliased by FreeText5DisclosureInstruction
static const value_string order_routing_indicator_vals[] = { // OrderRoutingIndicator
    { 0, "NO_VALUE" },
    { 'N', "No" },
    { 'Y', "Yes" },
    { 0, NULL }
};
// OrderSide aliased by LegSide
static const value_string ownership_indicator_vals[] = { // OwnershipIndicator
    { 0, "No_Change_of_Ownership" },
    { 1, "Change_to_Executing_Trader" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_action_type_vals[] = { // PartyActionType
    { 1, "Halt_Trading" },
    { 2, "Reinstate" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_detail_role_qualifier_vals[] = { // PartyDetailRoleQualifier
    { 10, "Trader" },
    { 11, "Head_Trader" },
    { 12, "Supervisor" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_detail_status_vals[] = { // PartyDetailStatus
    { 0, "Active" },
    { 1, "Suspend" },
    { 2, "Rejected" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_detail_status_information_vals[] = { // PartyDetailStatusInformation
    { 1, "TRR_THRESHOLD_TOO_BIG" },
    { 2, "BLOCK_ALL_ANONYMOUS" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_identering_firm_vals[] = { // PartyIDEnteringFirm
    { 1, "Participant" },
    { 2, "MarketSupervision" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_idorigination_market_vals[] = { // PartyIDOriginationMarket
    { 1, "XKFE" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string party_idsettlement_location_vals[] = { // PartyIDSettlementLocation
    { 1, "Clearstrem_Banking_Frankfurt" },
    { 2, "Clearstrem_Banking_Luxemburg" },
    { 3, "CLS_Group" },
    { 4, "Euroclear" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// PartyIdInvestmentDecisionMakerQualifier aliased by ExecutingTraderQualifier
// PartyOrderOriginationDisclosureInstruction aliased by FreeText5DisclosureInstruction
static const value_string party_sub_idtype_vals[] = { // PartySubIDType
    { 14001, "Buyer" },
    { 14002, "Seller" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
// PositionEffect aliased by LegPositionEffect
// PriceDisclosureInstruction aliased by FreeText5DisclosureInstruction
static const value_string price_validity_check_type_vals[] = { // PriceValidityCheckType
    { 0, "None" },
    { 1, "Optional" },
    { 2, "Mandatory" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string product_complex_vals[] = { // ProductComplex
    { 1, "simple_instrument" },
    { 2, "standard_option_strategy" },
    { 3, "non_standard_option_strategy" },
    { 4, "volatility_strategy" },
    { 5, "futures_Spread" },
    { 6, "inter_product_spread" },
    { 7, "standard_future_strategy" },
    { 8, "pack_and_bundle" },
    { 9, "strip" },
    { 10, "flexible_simple_instrument" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext product_complex_vals_ext = VALUE_STRING_EXT_INIT(product_complex_vals);
static const value_string put_or_call_vals[] = { // PutOrCall
    { 0, "Put" },
    { 1, "Call" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_cancel_type_vals[] = { // QuoteCancelType
    { 4, "Cancel_All_Quotes" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_entry_reject_reason_vals[] = { // QuoteEntryRejectReason
    { 1, "Unknown_Security" },
    { 6, "Duplicate_Quote" },
    { 8, "Invalid_Price" },
    { 16, "No_Reference_Price_Available" },
    { 100, "No_Single_Sided_Quotes" },
    { 103, "Invalid_Quoting_Model" },
    { 106, "Invalid_Size" },
    { 107, "Invalid_Underlying_Price" },
    { 108, "Bid_Price_Not_Reasonable" },
    { 109, "Ask_Price_Not_Reasonable" },
    { 110, "Bid_Price_Exceeds_Range" },
    { 111, "Ask_Price_Exceeds_Range" },
    { 115, "Instrument_State_Freeze" },
    { 116, "Deletion_Already_Pending" },
    { 119, "Entitlement_Not_Assigned_For_Underlying" },
    { 124, "Currently_Not_Tradeable_On_Book" },
    { 125, "Quantity_Limit_Exceeded" },
    { 126, "Value_Limit_Exceeded" },
    { 127, "Invalid_Quote_Spread" },
    { 128, "Book_or_Cancel" },
    { 131, "Cant_Proc_In_Curr_Instr_State" },
    { 134, "Invalid_Quote_Type" },
    { 143, "Trading_indication_running_for_trader" },
    { 144, "On_Book_Trading_disabled_for_Instrument_Type" },
    { 147, "Liquidity_provider_protection_bid_side_cancelled" },
    { 148, "Liquidity_provider_protection_ask_side_cancelled" },
    { 155, "Outside_Quoting_Period" },
    { 156, "Match_Price_Not_On_Price_Step" },
    { 161, "Quantity_Limit_Exceeds_TSL" },
    { 163, "Too_Many_Orders_and_Quotes_in_Order_Book" },
    { 0xFFFFFFFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext quote_entry_reject_reason_vals_ext = VALUE_STRING_EXT_INIT(quote_entry_reject_reason_vals);
static const value_string quote_entry_status_vals[] = { // QuoteEntryStatus
    { 0, "Accepted" },
    { 5, "Rejected" },
    { 6, "Removed_and_Rejected" },
    { 10, "Pending" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_event_liquidity_ind_vals[] = { // QuoteEventLiquidityInd
    { 1, "Added_Liquidity" },
    { 2, "Removed_Liquidity" },
    { 4, "Auction" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_event_reason_vals[] = { // QuoteEventReason
    { 14, "Pending_cancellation_executed" },
    { 15, "Invalid_price" },
    { 16, "Cross_rejected" },
    { 17, "Book_or_Cancel" },
    { 18, "PLP" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// QuoteEventSide aliased by LegSide
static const value_string quote_event_type_vals[] = { // QuoteEventType
    { 2, "Modified_quote_side" },
    { 3, "Removed_quote_side" },
    { 4, "Partially_filled" },
    { 5, "Filled" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_instruction_vals[] = { // QuoteInstruction
    { 0, "Do_Not_Quote" },
    { 1, "Quote" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_size_type_vals[] = { // QuoteSizeType
    { 1, "TotalSize" },
    { 2, "OpenSize" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_status_vals[] = { // QuoteStatus
    { 6, "Removed" },
    { 7, "Expired" },
    { 16, "Active" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_sub_type_vals[] = { // QuoteSubType
    { 1, "WorkingDelta" },
    { 2, "BasisTrade" },
    { 3, "Regular" },
    { 4, "NegotiateUnderlyingOutsideExchange" },
    { 5, "VolaStrategyFix" },
    { 6, "VolaStrategyNegotiateUnderlying" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quote_type_vals[] = { // QuoteType
    { 0, "Indicative" },
    { 1, "Tradeable" },
    { 100, "Tradeable_BOC" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string quoting_status_vals[] = { // QuotingStatus
    { 1, "Open_Active" },
    { 2, "Open_Idle" },
    { 3, "Closed_Inactive" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// RefApplID aliased by ApplID
static const value_string related_product_complex_vals[] = { // RelatedProductComplex
    { 2, "standard_option_strategy" },
    { 3, "non_standard_option_strategy" },
    { 4, "volatility_strategy" },
    { 5, "futures_Spread" },
    { 6, "inter_product_spread" },
    { 7, "standard_future_strategy" },
    { 8, "pack_and_bundle" },
    { 9, "strip" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext related_product_complex_vals_ext = VALUE_STRING_EXT_INIT(related_product_complex_vals);
// RequestingPartyIDEnteringFirm aliased by PartyIDEnteringFirm
static const value_string requesting_party_idexecuting_system_vals[] = { // RequestingPartyIDExecutingSystem
    { 1, "EurexClearing" },
    { 2, "T7" },
    { 0xFFFFFFFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string requesting_party_sub_idtype_vals[] = { // RequestingPartySubIDType
    { 5000, "Requester" },
    { 5001, "Responder" },
    { 5002, "Both" },
    { 5003, "System" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string respondent_type_vals[] = { // RespondentType
    { 1, "AllMarketParticipants" },
    { 102, "Anonymous" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string reversal_cancellation_reason_vals[] = { // ReversalCancellationReason
    { 1, "TAS_Change" },
    { 2, "Intraday_Expiration" },
    { 3, "Instrument_Deletion" },
    { 4, "Instrument_Suspension" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// ReversalIndicator aliased by FreeText5DisclosureInstruction
static const value_string risk_limit_action_vals[] = { // RiskLimitAction
    { 0, "QueueInbound" },
    { 2, "Reject" },
    { 4, "Warning" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string risk_limit_platform_vals[] = { // RiskLimitPlatform
    { 0, "On_Book" },
    { 1, "Off_Book" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string risk_limit_requesting_party_role_vals[] = { // RiskLimitRequestingPartyRole
    { 4, "Clearing_firm" },
    { 22, "Exchange" },
    { 59, "Executing_unit" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string risk_limit_type_vals[] = { // RiskLimitType
    { 4, "Long_limit" },
    { 5, "Short_limit" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// RiskLimitViolationIndicator aliased by FreeText5DisclosureInstruction
// RootPartyIDInvestmentDecisionMakerQualifier aliased by ExecutingTraderQualifier
static const value_string root_party_sub_idtype_vals[] = { // RootPartySubIDType
    { 4001, "Buyer" },
    { 4002, "Seller" },
    { 4003, "Broker" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
// SelectiveRequestForQuoteServiceStatus aliased by ApplSeqStatus
static const value_string session_mode_vals[] = { // SessionMode
    { 1, "HF" },
    { 2, "LF" },
    { 3, "GUI" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string session_reject_reason_vals[] = { // SessionRejectReason
    { 1, "Required_Tag_Missing" },
    { 5, "Value_is_incorrect" },
    { 7, "Decryption_problem" },
    { 11, "Invalid_MsgID" },
    { 16, "Incorrect_NumInGroup_count" },
    { 99, "Other" },
    { 100, "Throttle_Limit_Exceeded" },
    { 101, "Exposure_Limit_Exceeded" },
    { 102, "Service_Temporarily_Not_Available" },
    { 103, "Service_Not_Available" },
    { 105, "Outbound_conversion_error" },
    { 152, "Heartbeat_Violation" },
    { 200, "Internal_technical_error" },
    { 210, "Validation_Error" },
    { 211, "User_Already_Logged_In" },
    { 216, "Gateway_Is_Standby" },
    { 217, "Session_Login_Limit_Reached" },
    { 223, "User_Entitlement_Data_Timeout" },
    { 224, "PSGateway_Session_Limit_Reached" },
    { 225, "User_Login_Limit_Reached" },
    { 226, "Outstanding_Logins_Bu_Limit_Reached" },
    { 227, "Outstanding_Logins_Session_Limit_Reached" },
    { 10000, "Order_Not_Found" },
    { 10001, "Price_Not_Reasonable" },
    { 10002, "ClientOrderID_Not_Unique" },
    { 10003, "Quote_Activation_In_Progress" },
    { 10006, "Stop_Bid_Price_Not_Reasonable" },
    { 10007, "Stop_Ask_Price_Not_Reasonable" },
    { 10008, "Order_Not_Executable_Within_Validity" },
    { 10010, "Create_CI_Throttle_Exceeded" },
    { 10011, "Transaction_Not_Allowed_In_Current_State" },
    { 0xFFFFFFFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext session_reject_reason_vals_ext = VALUE_STRING_EXT_INIT(session_reject_reason_vals);
static const value_string session_status_vals[] = { // SessionStatus
    { 0, "Active" },
    { 4, "Logout" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string session_sub_mode_vals[] = { // SessionSubMode
    { 0, "Regular_trading_session" },
    { 1, "FIX_trading_session" },
    { 2, "Regular_Back_Office_session" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string settl_method_vals[] = { // SettlMethod
    { 0, "NO_VALUE" },
    { 'C', "Cash_Settlement" },
    { 'P', "Physical_Settlement" },
    { 0, NULL }
};
// ShowLastDealOnClosure aliased by FreeText5DisclosureInstruction
// Side aliased by LegSide
// SideDisclosureInstruction aliased by FreeText5DisclosureInstruction
// SideLiquidityInd aliased by QuoteEventLiquidityInd
static const value_string side_trd_sub_typ_vals[] = { // SideTrdSubTyp
    { 2001, "Block_Trade" },
    { 2004, "Trade_at_Market" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
// SkipValidations aliased by ApplResendFlag
static const value_string swap_clearer_vals[] = { // SwapClearer
    { 0, "ECAG" },
    { 1, "NON_ECAG" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// T7EntryServiceRtmStatus aliased by ApplSeqStatus
// T7EntryServiceStatus aliased by ApplSeqStatus
static const value_string time_in_force_vals[] = { // TimeInForce
    { 0, "Day" },
    { 1, "GTC" },
    { 3, "IOC" },
    { 6, "GTD" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trad_ses_event_vals[] = { // TradSesEvent
    { 101, "Start_of_Service" },
    { 102, "Market_Reset" },
    { 103, "End_of_Restatement" },
    { 104, "End_of_Day_Service" },
    { 105, "Service_Resumed" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trad_ses_mode_vals[] = { // TradSesMode
    { 1, "Testing" },
    { 2, "Simulated" },
    { 3, "Production" },
    { 4, "Acceptance" },
    { 5, "Disaster_Recovery" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trade_alloc_status_vals[] = { // TradeAllocStatus
    { 1, "Pending" },
    { 2, "Approved" },
    { 3, "Auto_Approved" },
    { 4, "Uploaded" },
    { 5, "Canceled" },
    { 6, "Pending_Execution" },
    { 7, "Pending_Reversal" },
    { 8, "Approved_Reversal" },
    { 9, "Reversed" },
    { 10, "Cancelled_Reversal" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext trade_alloc_status_vals_ext = VALUE_STRING_EXT_INIT(trade_alloc_status_vals);
// TradeManagerStatus aliased by ApplSeqStatus
static const value_string trade_platform_vals[] = { // TradePlatform
    { 0, "Off_Book" },
    { 1, "On_Book" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trade_publish_indicator_vals[] = { // TradePublishIndicator
    { 0, "Do_Not_Publish_Trade" },
    { 1, "Publish_Trade" },
    { 2, "Deferred_Publication" },
    { 3, "Published" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trade_report_type_vals[] = { // TradeReportType
    { 0, "Submit" },
    { 1, "Alleged" },
    { 2, "Accept" },
    { 3, "Decline" },
    { 5, "No_Was_Replaced" },
    { 6, "Trade_Report_Cancel" },
    { 7, "Trade_Break" },
    { 11, "Alleged_New" },
    { 13, "Alleged_No_Was" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext trade_report_type_vals_ext = VALUE_STRING_EXT_INIT(trade_report_type_vals);
static const value_string trade_request_result_vals[] = { // TradeRequestResult
    { 100, "Cancel_pending" },
    { 101, "Cancel_declined" },
    { 102, "Cancel_approved" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// TradeToQuoteRatioRanking aliased by EnlightRFQAvgRespRateRanking
static const value_string trading_capacity_vals[] = { // TradingCapacity
    { 1, "Customer" },
    { 5, "Principal" },
    { 6, "Market_Maker" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trading_session_sub_id_vals[] = { // TradingSessionSubID
    { 4, "Closing_auction" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string transaction_delay_indicator_vals[] = { // TransactionDelayIndicator
    { 0, "Not_delayed" },
    { 1, "Delayed" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string transfer_reason_vals[] = { // TransferReason
    { 1, "Owner" },
    { 2, "Clearer" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string trd_rpt_status_vals[] = { // TrdRptStatus
    { 0, "Accepted" },
    { 1, "Rejected" },
    { 2, "Cancelled" },
    { 4, "Pending_New" },
    { 5, "Pending_Cancel" },
    { 7, "Terminated" },
    { 9, "Deemed_Verified" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext trd_rpt_status_vals_ext = VALUE_STRING_EXT_INIT(trd_rpt_status_vals);
static const value_string trd_type_vals[] = { // TrdType
    { 1, "Block_Trade" },
    { 12, "Exchange_for_Swap" },
    { 50, "Compression" },
    { 1000, "Vola_Trade" },
    { 1001, "EFP_Fin_Trade" },
    { 1002, "EFP_Index_Futures_Trade" },
    { 1004, "Trade_at_Market" },
    { 1006, "Enlight" },
    { 1007, "BLOCK_QTPIP" },
    { 1010, "BTRF" },
    { 1011, "EBB" },
    { 0xFFFF, "NO_VALUE" },
    { 0, NULL }
};
static value_string_ext trd_type_vals_ext = VALUE_STRING_EXT_INIT(trd_type_vals);
static const value_string triggered_vals[] = { // Triggered
    { 0, "Not_triggered" },
    { 1, "Triggered_Stop" },
    { 2, "Triggered_OCO" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string user_status_vals[] = { // UserStatus
    { 7, "User_forced_logout" },
    { 10, "User_stopped" },
    { 11, "User_released" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
static const value_string value_check_type_min_lot_size_vals[] = { // ValueCheckTypeMinLotSize
    { 0, "Do_not_check" },
    { 1, "Check" },
    { 0xFF, "NO_VALUE" },
    { 0, NULL }
};
// ValueCheckTypeValue aliased by ValueCheckTypeMinLotSize

enum ETI_Type {
    ETI_EOF,
    ETI_PADDING,
    ETI_UINT,
    ETI_INT,
    ETI_UINT_ENUM,
    ETI_INT_ENUM,
    ETI_COUNTER,
    ETI_FIXED_POINT,
    ETI_TIMESTAMP_NS,
    ETI_CHAR,
    ETI_STRING,
    ETI_VAR_STRING,
    ETI_STRUCT,
    ETI_VAR_STRUCT,
    ETI_DSCP
};

struct ETI_Field {
    uint8_t  type;
    uint8_t  counter_off;      // offset into counter array
                               // if ETI_COUNTER => storage
                               // if ETI_VAR_STRING or ETI_VAR_STRUCT => load
                               // to get length or repeat count
                               // if ETI_FIXED_POINT: #fractional digits
    uint16_t size;             // or offset into struct_names if ETI_STRUCT/ETI_VAR_STRUCT
    uint16_t field_handle_idx; // or index into fields array if ETI_STRUCT/ETI_VAR_STRUT
    uint16_t ett_idx;          // index into ett array if ETI_STRUCT/ETI_VAR_STRUCT
                               // or max value if ETI_COUNTER
};

static gint ett_eti[] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
static gint ett_eti_dscp = -1;
/* This method dissects fully reassembled messages */
static int
dissect_eti_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
{
    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ETI");
    col_clear(pinfo->cinfo, COL_INFO);
    guint16 templateid = tvb_get_letohs(tvb, 4);
    const char *template_str = val_to_str_ext(templateid, &template_id_vals_ext, "Unknown ETI template: 0x%04x");
    col_add_fstr(pinfo->cinfo, COL_INFO, "%s", template_str);

    /* create display subtree for the protocol */
    proto_item *ti = proto_tree_add_item(tree, proto_eti, tvb, 0, -1, ENC_NA);
    guint32 bodylen= tvb_get_letohl(tvb, 0);
    proto_item_append_text(ti, ", %s (%" PRIu16 "), BodyLen: %u", template_str, templateid, bodylen);
    proto_tree *root = proto_item_add_subtree(ti, ett_eti[0]);

    static const char struct_names[] = "AffectedOrderRequestsGrp\0BasketExecGrp\0BasketRootPartyGrp\0BasketSideAllocExtBCGrp\0BasketSideAllocExtGrp\0BasketSideAllocGrp\0CrossRequestAckSideGrp\0CrossRequestSideGrp\0EnrichmentRulesGrp\0FillsGrp\0InstrmntLegExecGrp\0InstrmtLegGrp\0InstrmtMatchSideGrp\0InstrumentAttributeGrp\0InstrumentEventGrp\0LegOrdGrp\0MMParameterGrp\0MessageHeaderIn\0MessageHeaderOut\0NRBCHeader\0NRResponseHeaderME\0NotAffectedOrdersGrp\0NotAffectedSecuritiesGrp\0NotifHeader\0OrderBookItemGrp\0PartyDetailsGrp\0QuotReqLegsGrp\0QuoteEntryAckGrp\0QuoteEntryGrp\0QuoteEventGrp\0QuoteLegExecGrp\0RBCHeader\0RBCHeaderME\0RequestHeader\0ResponseHeader\0ResponseHeaderME\0RiskLimitQtyGrp\0RiskLimitsRptGrp\0SRQSRelatedTradeIDGrp\0SessionsGrp\0SideAllocExtGrp\0SideAllocGrp\0SideAllocGrpBC\0SideCrossLegGrp\0SmartPartyDetailGrp\0TargetParties\0TrdInstrmntLegGrp\0UnderlyingStipGrp";
    static const struct ETI_Field fields[] = {
        // AffectedOrderRequestsGrpComp@0
          { ETI_UINT, 0, 4, AFFECTEDORDERREQUESTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketExecGrpComp@3
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SIDEMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, SIDETRDSUBTYP_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketRootPartyGrpComp@9
        , { ETI_UINT_ENUM, 0, 2, ROOTPARTYSUBIDTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYCONTRAFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYCONTRATRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETSIDETRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketSideAllocExtBCGrpComp@15
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SIDEMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, SIDETRDSUBTYP_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, PARTYSUBIDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EFFECTONBASKET_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEALLOCSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketSideAllocExtGrpComp@47
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SIDEMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketSideAllocGrpComp@77
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, INDIVIDUALALLOCID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, PARTYSUBIDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, INSTRMTMATCHSIDEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEALLOCSTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // CrossRequestAckSideGrpComp@87
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, INPUTSOURCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // CrossRequestSideGrpComp@92
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, MAXIMUMPRICE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, INPUTSOURCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, SIDECOMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // EnrichmentRulesGrpComp@116
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDORIGINATIONMARKET_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // FillsGrpComp@128
        , { ETI_FIXED_POINT, 8, 8, FILLPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, FILLQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, FILLMATCHID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, FILLEXECID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, FILLLIQUIDITYIND_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InstrmntLegExecGrpComp@135
        , { ETI_INT, 0, 8, LEGSECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LEGLASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEGLASTQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, LEGEXECID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSIDE_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, FILLREFID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InstrmtLegGrpComp@143
        , { ETI_INT, 0, 8, LEGSECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LEGPRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, LEGSYMBOL_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, LEGRATIOQTY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSECURITYTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InstrmtMatchSideGrpComp@151
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SIDEMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, SIDETRDSUBTYP_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, INSTRMTMATCHSIDEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EFFECTONBASKET_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InstrumentAttributeGrpComp@165
        , { ETI_UINT_ENUM, 0, 1, INSTRATTRIBTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, INSTRATTRIBVALUE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InstrumentEventGrpComp@169
        , { ETI_UINT, 0, 4, EVENTDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EVENTTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // LegOrdGrpComp@173
        , { ETI_STRING, 0, 2, LEGACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, LEGPOSITIONEFFECT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // MMParameterGrpComp@177
        , { ETI_INT, 0, 8, EXPOSUREDURATION_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, DELTA_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, VEGA_FH_IDX, 0 }
        , { ETI_INT, 0, 4, PCTCOUNT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // MessageHeaderInComp@184
        , { ETI_UINT, 0, 4, BODYLEN_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, TEMPLATEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 8, NETWORKMSGID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // MessageHeaderOutComp@189
        , { ETI_UINT, 0, 4, BODYLEN_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, TEMPLATEID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NRBCHeaderComp@193
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, APPLSUBID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTFRAGMENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NRResponseHeaderMEComp@199
        , { ETI_TIMESTAMP_NS, 0, 8, REQUESTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEIN_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEOUT_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, RESPONSEIN_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MSGSEQNUM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTFRAGMENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NotAffectedOrdersGrpComp@208
        , { ETI_UINT, 0, 8, NOTAFFECTEDORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, NOTAFFORIGCLORDID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NotAffectedSecuritiesGrpComp@211
        , { ETI_UINT, 0, 8, NOTAFFECTEDSECURITYID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NotifHeaderComp@213
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // OrderBookItemGrpComp@215
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BESTBIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, BESTBIDSIZE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BESTOFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, BESTOFFERSIZE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MDBOOKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MDSUBBOOKTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // PartyDetailsGrpComp@224
        , { ETI_UINT, 0, 4, PARTYDETAILIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYDETAILEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILROLEQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, PARTYDETAILDESKID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuotReqLegsGrpComp@231
        , { ETI_INT, 0, 8, LEGSECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, LEGRATIOQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, LEGSYMBOL_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSECURITYTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSIDE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteEntryAckGrpComp@238
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLSIZE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 4, QUOTEENTRYREJECTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEENTRYSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteEntryGrpComp@245
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, BIDSIZE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, OFFERSIZE_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteEventGrpComp@251
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEEVENTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, QUOTEEVENTQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, QUOTEMSGID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, QUOTEEVENTMATCHID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, QUOTEEVENTEXECID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEEVENTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEEVENTSIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEEVENTLIQUIDITYIND_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEEVENTREASON_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteLegExecGrpComp@263
        , { ETI_INT, 0, 8, LEGSECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LEGLASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEGLASTQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, LEGEXECID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEGSIDE_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, NOQUOTEEVENTSINDEX_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RBCHeaderComp@271
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, APPLSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, APPLSUBID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLRESENDFLAG_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTFRAGMENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RBCHeaderMEComp@280
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEOUT_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, NOTIFICATIONIN_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, APPLSUBID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, APPLMSGID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLRESENDFLAG_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTFRAGMENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RequestHeaderComp@291
        , { ETI_UINT, 0, 4, MSGSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SENDERSUBID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ResponseHeaderComp@294
        , { ETI_TIMESTAMP_NS, 0, 8, REQUESTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MSGSEQNUM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ResponseHeaderMEComp@299
        , { ETI_TIMESTAMP_NS, 0, 8, REQUESTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEIN_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEOUT_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, RESPONSEIN_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, SENDINGTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MSGSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLID_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, APPLMSGID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTFRAGMENT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RiskLimitQtyGrpComp@310
        , { ETI_FIXED_POINT, 4, 8, RISKLIMITQTY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RiskLimitsRptGrpComp@314
        , { ETI_FIXED_POINT, 4, 8, RISKLIMITQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RISKLIMITOPENQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RISKLIMITNETPOSITIONQTY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITREQUESTINGPARTYROLE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITVIOLATIONINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, RISKLIMITGROUP_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSRelatedTradeIDGrpComp@323
        , { ETI_UINT, 0, 4, SRQSRELATEDTRADEID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SessionsGrpComp@326
        , { ETI_UINT, 0, 4, PARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SESSIONMODE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SESSIONSUBMODE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SideAllocExtGrpComp@331
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, INDIVIDUALALLOCID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_UINT, 0, 4, TESENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEALLOCSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SideAllocGrpComp@362
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, INDIVIDUALALLOCID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SideAllocGrpBCComp@370
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, REVERSALAPPROVALTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, INDIVIDUALALLOCID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEALLOCSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SideCrossLegGrpComp@380
        , { ETI_UINT_ENUM, 0, 1, LEGINPUTSOURCE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, LEGPOSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, LEGACCOUNT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SmartPartyDetailGrpComp@385
        , { ETI_STRING, 0, 5, PARTYDETAILEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYDETAILEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TargetPartiesComp@389
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDEDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LEAVESQTYDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTPXDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, LASTQTYDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, FREETEXT5DISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYORDERORIGINATIONDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEINSTRUCTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, TARGETPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUSINFORMATION_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TrdInstrmntLegGrpComp@404
        , { ETI_INT, 0, 8, LEGSECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LEGPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEGQTY_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // UnderlyingStipGrpComp@408
        , { ETI_STRING, 0, 32, UNDERLYINGSTIPVALUE_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, UNDERLYINGSTIPTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // AddComplexInstrumentRequest@412
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SECURITYSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 213, 143, 12 } // InstrmtLegGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // AddComplexInstrumentResponse@422
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_FIXED_POINT, 8, 8, LOWLIMITPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, HIGHLIMITPRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, LASTUPDATETIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, SECURITYRESPONSEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, NUMBEROFSECURITIES_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SECURITYSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MULTILEGMODEL_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, IMPLIEDMARKETINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_VAR_STRUCT, 0, 213, 143, 12 } // InstrmtLegGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // AddFlexibleInstrumentRequest@438
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, STRIKEPRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYDATE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, SETTLMETHOD_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, OPTATTRIBUTE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PUTORCALL_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXERCISESTYLE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // AddFlexibleInstrumentResponse@449
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, SECURITYRESPONSEID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, STRIKEPRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, SETTLMETHOD_FH_IDX, 0 }
        , { ETI_UINT, 0, 1, OPTATTRIBUTE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PUTORCALL_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXERCISESTYLE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // AmendBasketTradeRequest@463
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYMONTHYEAR_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, BASKETTRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOBASKETROOTPARTYGRPS_FH_IDX, 2 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOINSTRMTMATCHSIDES_FH_IDX, 199 } // <- counter@2
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_VAR_STRUCT, 1, 39, 9, 3 } // BasketRootPartyGrp
        , { ETI_VAR_STRUCT, 2, 227, 151, 13 } // InstrmtMatchSideGrp
        , { ETI_VAR_STRUCT, 0, 104, 77, 6 } // BasketSideAllocGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ApproveBasketTradeRequest@482
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, ROOTPARTYSUBIDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETSIDETRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 82, 47, 5 } // BasketSideAllocExtGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ApproveReverseTESTradeRequest@497
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ApproveTESTradeRequest@508
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketApproveBroadcast@545
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, BASKETTRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOBASKETROOTPARTYGRPS_FH_IDX, 2 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 39, 9, 3 } // BasketRootPartyGrp
        , { ETI_VAR_STRUCT, 0, 58, 15, 4 } // BasketSideAllocExtBCGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketBroadcast@566
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYMONTHYEAR_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, BASKETTRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOBASKETROOTPARTYGRPSBC_FH_IDX, 3 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOINSTRMTMATCHSIDES_FH_IDX, 199 } // <- counter@2
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 39, 9, 3 } // BasketRootPartyGrp
        , { ETI_VAR_STRUCT, 2, 227, 151, 13 } // InstrmtMatchSideGrp
        , { ETI_VAR_STRUCT, 0, 104, 77, 6 } // BasketSideAllocGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketDeleteBroadcast@588
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, DELETEREASON_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketExecutionBroadcast@601
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOINSTRMTMATCHSIDES_FH_IDX, 199 } // <- counter@0
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETSIDETRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 25, 3, 2 } // BasketExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // BasketResponse@616
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // BroadcastErrorNotification@621
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 423, 213, 24 } // NotifHeader
        , { ETI_UINT_ENUM, 0, 4, APPLIDSTATUS_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, REFAPPLSUBID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, REFAPPLID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SESSIONSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // CLIPDeletionNotification@631
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // CLIPExecutionNotification@645
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOLEGEXECS_FH_IDX, 600 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MATCHTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOFILLS_FH_IDX, 100 } // <- counter@1
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 185, 128, 10 } // FillsGrp
        , { ETI_VAR_STRUCT, 0, 194, 135, 11 } // InstrmntLegExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // CLIPResponse@666
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, CROSSREQUESTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDES_FH_IDX, 2 } // <- counter@0
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 123, 87, 7 } // CrossRequestAckSideGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // CrossRequest@676
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // CrossRequestResponse@683
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllOrderBroadcast@687
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDENTERINGTRADER_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDORDERS_FH_IDX, 500 } // <- counter@0
        , { ETI_COUNTER, 1, 2, NOAFFECTEDORDERREQUESTS_FH_IDX, 500 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_VAR_STRUCT, 0, 377, 208, 22 } // NotAffectedOrdersGrp
        , { ETI_VAR_STRUCT, 1, 0, 0, 1 } // AffectedOrderRequestsGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllOrderNRResponse@705
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllOrderQuoteEventBroadcast@709
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllOrderRequest@718
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllOrderResponse@732
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 595, 299, 36 } // ResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDORDERS_FH_IDX, 500 } // <- counter@0
        , { ETI_COUNTER, 1, 2, NOAFFECTEDORDERREQUESTS_FH_IDX, 500 } // <- counter@1
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 377, 208, 22 } // NotAffectedOrdersGrp
        , { ETI_VAR_STRUCT, 1, 0, 0, 1 } // AffectedOrderRequestsGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllQuoteBroadcast@741
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDENTERINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDSECURITIES_FH_IDX, 500 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, TARGETPARTYIDDESKID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 398, 211, 23 } // NotAffectedSecuritiesGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllQuoteRequest@756
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteAllQuoteResponse@766
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDSECURITIES_FH_IDX, 500 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 398, 211, 23 } // NotAffectedSecuritiesGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteBasketTradeRequest@773
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteCLIPRequest@783
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, CROSSREQUESTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteOrderBroadcast@795
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDENTERINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteOrderComplexRequest@816
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteOrderNRResponse@832
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteOrderResponse@848
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 595, 299, 36 } // ResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteOrderSingleRequest@864
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIMPLESECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // DeleteTESTradeRequest@880
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // EnterBasketTradeRequest@892
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYMONTHYEAR_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, BASKETTRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOBASKETROOTPARTYGRPS_FH_IDX, 2 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOINSTRMTMATCHSIDES_FH_IDX, 199 } // <- counter@2
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 39, 9, 3 } // BasketRootPartyGrp
        , { ETI_VAR_STRUCT, 2, 227, 151, 13 } // InstrmtMatchSideGrp
        , { ETI_VAR_STRUCT, 0, 104, 77, 6 } // BasketSideAllocGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // EnterCLIPRequest@910
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, CROSSID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, CROSSREQUESTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDES_FH_IDX, 2 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOCROSSLEGS_FH_IDX, 40 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, SIDEDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERQTYDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYCONTRAFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYCONTRATRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 146, 92, 8 } // CrossRequestSideGrp
        , { ETI_VAR_STRUCT, 1, 723, 380, 44 } // SideCrossLegGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // EnterTESTradeRequest@929
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RELATEDTRADEQUANTITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGSETTLEMENTDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGMATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, RELATEDTRADEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOEVENTS_FH_IDX, 2 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOLEGS_FH_IDX, 20 } // <- counter@2
        , { ETI_COUNTER, 3, 1, NOINSTRATTRIB_FH_IDX, 6 } // <- counter@3
        , { ETI_COUNTER, 4, 1, NOUNDERLYINGSTIPS_FH_IDX, 1 } // <- counter@4
        , { ETI_UINT_ENUM, 0, 1, PARTYIDSETTLEMENTLOCATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SWAPCLEARER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, UNDERLYINGSECURITYID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGSECURITYDESC_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, UNDERLYINGCURRENCY_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGISSUER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 695, 362, 43 } // SideAllocGrp
        , { ETI_VAR_STRUCT, 2, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_VAR_STRUCT, 1, 270, 169, 15 } // InstrumentEventGrp
        , { ETI_VAR_STRUCT, 3, 247, 165, 14 } // InstrumentAttributeGrp
        , { ETI_VAR_STRUCT, 4, 791, 408, 48 } // UnderlyingStipGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ForcedLogoutNotification@969
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 423, 213, 24 } // NotifHeader
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ForcedUserLogoutNotification@975
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 423, 213, 24 } // NotifHeader
        , { ETI_UINT, 0, 4, USERNAME_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, USERSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // Heartbeat@983
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_EOF, 0, 0, 0, 0 }
        // HeartbeatNotification@985
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 423, 213, 24 } // NotifHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireEnrichmentRuleIDListRequest@988
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_STRING, 0, 16, LASTENTITYPROCESSED_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireEnrichmentRuleIDListResponse@992
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_STRING, 0, 16, LASTENTITYPROCESSED_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOENRICHMENTRULES_FH_IDX, 400 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 166, 116, 9 } // EnrichmentRulesGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireMMParameterRequest@999
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireMMParameterResponse@1004
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MMPARAMETERREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOMMPARAMETERS_FH_IDX, 9 } // <- counter@0
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 299, 177, 17 } // MMParameterGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquirePreTradeRiskLimitsRequest@1012
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITPLATFORM_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, RISKLIMITGROUP_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireSessionListRequest@1020
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireSessionListResponse@1023
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_COUNTER, 0, 2, NOSESSIONS_FH_IDX, 1000 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 667, 326, 40 } // SessionsGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireUserRequest@1029
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_STRING, 0, 16, LASTENTITYPROCESSED_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // InquireUserResponse@1033
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_STRING, 0, 16, LASTENTITYPROCESSED_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOPARTYDETAILS_FH_IDX, 1000 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 452, 224, 26 } // PartyDetailsGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // LegalNotificationBroadcast@1040
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, USERSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // LogonRequest@1048
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, HEARTBTINT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, DEFAULTCSTMAPPLVERID_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PASSWORD_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, APPLUSAGEORDERS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, APPLUSAGEQUOTES_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDERROUTINGINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, FIXENGINENAME_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, FIXENGINEVERSION_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, FIXENGINEVENDOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, APPLICATIONSYSTEMNAME_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, APPLICATIONSYSTEMVERSION_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, APPLICATIONSYSTEMVENDOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // LogonResponse@1065
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_INT, 0, 8, THROTTLETIMEINTERVAL_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, THROTTLENOMSGS_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, THROTTLEDISCONNECTLIMIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, HEARTBTINT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SESSIONINSTANCEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, MARKETID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADSESMODE_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, DEFAULTCSTMAPPLVERID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, DEFAULTCSTMAPPLVERSUBID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // LogoutRequest@1078
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // LogoutResponse@1081
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // MMParameterDefinitionRequest@1084
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, EXPOSUREDURATION_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, DELTA_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, VEGA_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, PCTCOUNT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // MMParameterDefinitionResponse@1095
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // MassQuoteRequest@1099
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESIZETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOQUOTEENTRIES_FH_IDX, 100 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 500, 245, 29 } // QuoteEntryGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // MassQuoteResponse@1118
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, QUOTERESPONSEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOQUOTESIDEENTRIES_FH_IDX, 200 } // <- counter@0
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 483, 238, 28 } // QuoteEntryAckGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyBasketTradeRequest@1127
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATURITYMONTHYEAR_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOBASKETSIDEALLOC_FH_IDX, 398 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOBASKETROOTPARTYGRPS_FH_IDX, 2 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOINSTRMTMATCHSIDES_FH_IDX, 199 } // <- counter@2
        , { ETI_STRING, 0, 20, BASKETTRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 39, 9, 3 } // BasketRootPartyGrp
        , { ETI_VAR_STRUCT, 2, 227, 151, 13 } // InstrmtMatchSideGrp
        , { ETI_VAR_STRUCT, 0, 104, 77, 6 } // BasketSideAllocGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderComplexRequest@1146
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, EXPIREDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, OWNERSHIPINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 289, 173, 16 } // LegOrdGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderComplexShortRequest@1190
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderNRResponse@1217
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderResponse@1235
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 595, 299, 36 } // ResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEPRIORITY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderSingleRequest@1254
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, STOPPX_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, EXPIREDATE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIMPLESECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGSESSIONSUBID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, OWNERSHIPINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyOrderSingleShortRequest@1299
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIMPLESECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ModifyTESTradeRequest@1324
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOLEGS_FH_IDX, 20 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, SWAPCLEARER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 695, 362, 43 } // SideAllocGrp
        , { ETI_VAR_STRUCT, 1, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderComplexRequest@1345
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, EXPIREDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 289, 173, 16 } // LegOrdGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderComplexShortRequest@1386
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderNRResponse@1412
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderResponse@1426
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 595, 299, 36 } // ResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSENTRYTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEPRIORITY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderSingleRequest@1442
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, STOPPX_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, EXPIREDATE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIMPLESECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGSESSIONSUBID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewOrderSingleShortRequest@1484
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIMPLESECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, ENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRICEVALIDITYCHECKTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEVALUE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // NewsBroadcast@1508
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, ORIGTIME_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_STRING, 0, 256, HEADLINE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // OrderExecNotification@1516
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOLEGEXECS_FH_IDX, 600 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOFILLS_FH_IDX, 100 } // <- counter@1
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 185, 128, 10 } // FillsGrp
        , { ETI_VAR_STRUCT, 0, 194, 135, 11 } // InstrmntLegExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // OrderExecReportBroadcast@1541
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSENTRYTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEPRIORITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, STOPPX_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, EXPIREDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHINSTCROSSID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDENTERINGTRADER_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOLEGEXECS_FH_IDX, 600 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TIMEINFORCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECINST_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGSESSIONSUBID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIXCLORDID_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOFILLS_FH_IDX, 100 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOLEGS_FH_IDX, 20 } // <- counter@2
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_VAR_STRUCT, 2, 289, 173, 16 } // LegOrdGrp
        , { ETI_VAR_STRUCT, 1, 185, 128, 10 } // FillsGrp
        , { ETI_VAR_STRUCT, 0, 194, 135, 11 } // InstrmntLegExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // OrderExecResponse@1598
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 595, 299, 36 } // ResponseHeaderME
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORIGCLORDID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSENTRYTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSTIMEPRIORITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CXLQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOLEGEXECS_FH_IDX, 600 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 2, EXECRESTATEMENTREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, EXECTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRIGGERED_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, CROSSEDINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSACTIONDELAYINDICATOR_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOFILLS_FH_IDX, 100 } // <- counter@1
        , { ETI_VAR_STRUCT, 1, 185, 128, 10 } // FillsGrp
        , { ETI_VAR_STRUCT, 0, 194, 135, 11 } // InstrmntLegExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // PartyActionReport@1624
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, REQUESTINGPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 4, REQUESTINGPARTYIDEXECUTINGSYSTEM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, MARKETID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYACTIONTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REQUESTINGPARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // PartyEntitlementsUpdateReport@1636
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYDETAILIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 4, REQUESTINGPARTYIDEXECUTINGSYSTEM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, MARKETID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, LISTUPDATEACTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, REQUESTINGPARTYENTERINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, REQUESTINGPARTYCLEARINGFIRM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // PingRequest@1649
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // PingResponse@1654
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // PreTradeRiskLimitResponse@1658
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT, 0, 8, RISKLIMITREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NORISKLIMITS_FH_IDX, 64 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITPLATFORM_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYDETAILEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 628, 314, 38 } // RiskLimitsRptGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // PreTradeRiskLimitsDefinitionRequest@1669
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITPLATFORM_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NORISKLIMITSQTY_FH_IDX, 2 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, PARTYDETAILSTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, RISKLIMITGROUP_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYDETAILEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 612, 310, 37 } // RiskLimitQtyGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteActivationNotification@1680
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYIDENTERINGTRADER_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDSECURITIES_FH_IDX, 500 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONREASON_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 398, 211, 23 } // NotAffectedSecuritiesGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteActivationRequest@1693
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MASSACTIONSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteActivationResponse@1705
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, MASSACTIONREPORTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NONOTAFFECTEDSECURITIES_FH_IDX, 500 } // <- counter@0
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 398, 211, 23 } // NotAffectedSecuritiesGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // QuoteExecutionReport@1712
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOLEGEXECS_FH_IDX, 600 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOQUOTEEVENTS_FH_IDX, 100 } // <- counter@1
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 514, 251, 30 } // QuoteEventGrp
        , { ETI_VAR_STRUCT, 0, 528, 263, 31 } // QuoteLegExecGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // RFQRequest@1722
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RFQResponse@1731
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_TIMESTAMP_NS, 0, 8, EXECID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // Reject@1735
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 358, 199, 21 } // NRResponseHeaderME
        , { ETI_UINT_ENUM, 0, 4, SESSIONREJECTREASON_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, SESSIONSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RetransmitMEMessageRequest@1743
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, SUBSCRIPTIONSCOPE_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REFAPPLID_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, APPLBEGMSGID_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, APPLENDMSGID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RetransmitMEMessageResponse@1752
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 2, APPLTOTALMESSAGECOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, APPLENDMSGID_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, REFAPPLLASTMSGID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RetransmitRequest@1759
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, APPLBEGSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, APPLENDSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REFAPPLID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RetransmitResponse@1767
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 8, APPLENDSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, REFAPPLLASTSEQNUM_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, APPLTOTALMESSAGECOUNT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ReverseTESTradeRequest@1774
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, REVERSALREASONTEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // RiskNotificationBroadcast@1785
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PARTYDETAILIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 4, REQUESTINGPARTYIDEXECUTINGSYSTEM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, MARKETID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, LISTUPDATEACTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RISKLIMITACTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, REQUESTINGPARTYENTERINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, REQUESTINGPARTYCLEARINGFIRM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 6, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSCreateDealNotification@1798
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXPIRETIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGEFFECTIVEDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPRICESTIPVALUE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ORIGTRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOORDERBOOKITEMS_FH_IDX, 26 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGINGINSTRUCTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, TARGETPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 435, 215, 25 } // OrderBookItemGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSDealNotification@1846
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPRICESTIPVALUE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TARGETPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, REQUESTINGPARTYSUBIDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREQUESTRESULT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, TARGETPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSDealResponse@1884
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SECONDARYTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSEnterQuoteRequest@1895
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, BIDSIZE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, OFFERSIZE_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSHitQuoteRequest@1929
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, VALIDUNTILTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPRICESTIPVALUE_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, PARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGINGINSTRUCTION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSInquireSmartRespondentRequest@1967
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EUREXVOLUMERANKING_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ENLIGHTRFQAVGRESPTIMERANKING_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ENLIGHTRFQAVGRESPRATERANKING_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADETOQUOTERATIORANKING_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSInquireSmartRespondentResponse@1975
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, NOPARTYDETAILS_FH_IDX, 1000 } // <- counter@0
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 739, 385, 45 } // SmartPartyDetailGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSNegotiationNotification@1982
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EFFECTIVETIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, LASTUPDATETIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, TRADETOQUOTERATIO_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NUMBEROFRESPONDENTS_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, TRADETOQUOTERATIOPOSITION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTEINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, TARGETPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSNegotiationRequesterNotification@2013
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRDREGTSEXECUTIONTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EFFECTIVETIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, LASTUPDATETIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NUMBEROFRESPONDENTS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESTATUS_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOTARGETPARTYIDS_FH_IDX, 50 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SHOWLASTDEALONCLOSURE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 759, 389, 46 } // TargetParties
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSNegotiationStatusNotification@2045
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EFFECTIVETIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSOpenNegotiationNotification@2054
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, NEGOTIATIONSTARTTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXPIRETIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, TRADETOREQUESTRATIO_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, TRADETOQUOTERATIO_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SECURITYSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NUMBEROFRESPONDENTS_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, TRADETOQUOTERATIOPOSITION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESTATUS_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RESPONDENTTYPE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, TARGETPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, TARGETPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 468, 231, 27 } // QuotReqLegsGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSOpenNegotiationRequest@2092
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, VALIDUNTILTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SECURITYSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESUBTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOTARGETPARTYIDS_FH_IDX, 50 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RESPONDENTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SHOWLASTDEALONCLOSURE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, QUOTEREQID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_VAR_STRUCT, 0, 468, 231, 27 } // QuotReqLegsGrp
        , { ETI_VAR_STRUCT, 1, 759, 389, 46 } // TargetParties
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSOpenNegotiationRequesterNotification@2120
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, EXPIRETIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, TRADETOREQUESTRATIO_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, SECURITYSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NUMBEROFRESPONDENTS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTESTATUS_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOLEGS_FH_IDX, 20 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOTARGETPARTYIDS_FH_IDX, 50 } // <- counter@1
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RESPONDENTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SHOWLASTDEALONCLOSURE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, FIRMNEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 468, 231, 27 } // QuotReqLegsGrp
        , { ETI_VAR_STRUCT, 1, 759, 389, 46 } // TargetParties
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSQuoteNotification@2157
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, SECONDARYQUOTEID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, BIDSIZE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, OFFERSIZE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTINGSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_UINT, 0, 4, PARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, QUOTEREQID_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSQuoteResponse@2191
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 8, QUOTEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, QUOTEREQID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSQuotingStatusRequest@2197
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTINGSTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSStatusBroadcast@2206
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADSESEVENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSUpdateDealStatusRequest@2212
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPRICESTIPVALUE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SRQSUpdateNegotiationRequest@2227
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_FIXED_POINT, 8, 8, QUOTEREFPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGDELTAPERCENTAGE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, BIDPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, OFFERPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ORDERQTY_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOTARGETPARTYIDS_FH_IDX, 50 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, QUOTECANCELTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SHOWLASTDEALONCLOSURE_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, FREETEXT5_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, PARTYORDERORIGINATIONTRADER_FH_IDX, 0 }
        , { ETI_VAR_STRUCT, 0, 759, 389, 46 } // TargetParties
        , { ETI_EOF, 0, 0, 0, 0 }
        // ServiceAvailabilityBroadcast@2247
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 347, 193, 20 } // NRBCHeader
        , { ETI_UINT, 0, 4, MATCHINGENGINETRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEMANAGERTRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, APPLSEQTRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, T7ENTRYSERVICETRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, T7ENTRYSERVICERTMTRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 2, PARTITIONID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MATCHINGENGINESTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEMANAGERSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, APPLSEQSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, T7ENTRYSERVICESTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, T7ENTRYSERVICERTMSTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ServiceAvailabilityMarketBroadcast@2262
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 347, 193, 20 } // NRBCHeader
        , { ETI_UINT, 0, 4, SELECTIVEREQUESTFORQUOTESERVICETRADEDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SELECTIVEREQUESTFORQUOTESERVICESTATUS_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SubscribeRequest@2268
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, SUBSCRIPTIONSCOPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REFAPPLID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // SubscribeResponse@2274
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 4, APPLSUBID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESApproveBroadcast@2279
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, ALLOCQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RELATEDTRADEQUANTITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGSETTLEMENTDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGMATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, RELATEDTRADEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESENRICHMENTRULEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, AUTOAPPROVALRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDSETTLEMENTLOCATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEALLOCSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGETYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOLEGS_FH_IDX, 20 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOEVENTS_FH_IDX, 2 } // <- counter@2
        , { ETI_COUNTER, 3, 1, NOINSTRATTRIB_FH_IDX, 6 } // <- counter@3
        , { ETI_COUNTER, 4, 1, NOUNDERLYINGSTIPS_FH_IDX, 1 } // <- counter@4
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDENTERINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, PARTYENTERINGTRADER_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, PARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, PARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, PARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, PARTYIDLOCATIONID_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, COMPLIANCETEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, UNDERLYINGSECURITYID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGSECURITYDESC_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, UNDERLYINGCURRENCY_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGISSUER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 1, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_VAR_STRUCT, 2, 270, 169, 15 } // InstrumentEventGrp
        , { ETI_VAR_STRUCT, 3, 247, 165, 14 } // InstrumentAttributeGrp
        , { ETI_VAR_STRUCT, 4, 791, 408, 48 } // UnderlyingStipGrp
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESBroadcast@2348
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RELATEDTRADEQUANTITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGSETTLEMENTDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGMATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, RELATEDTRADEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, AUTOAPPROVALRULEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 2, VARTEXTLEN_FH_IDX, 2000 } // <- counter@0
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_COUNTER, 1, 1, NOEVENTS_FH_IDX, 2 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOINSTRATTRIB_FH_IDX, 6 } // <- counter@2
        , { ETI_COUNTER, 3, 1, NOUNDERLYINGSTIPS_FH_IDX, 1 } // <- counter@3
        , { ETI_COUNTER, 4, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@4
        , { ETI_COUNTER, 5, 1, NOLEGS_FH_IDX, 20 } // <- counter@5
        , { ETI_UINT_ENUM, 0, 1, PARTYIDSETTLEMENTLOCATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SWAPCLEARER_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, UNDERLYINGSECURITYID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGSECURITYDESC_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, UNDERLYINGCURRENCY_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGISSUER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 4, 708, 370, 42 } // SideAllocGrpBC
        , { ETI_VAR_STRUCT, 5, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_VAR_STRUCT, 1, 270, 169, 15 } // InstrumentEventGrp
        , { ETI_VAR_STRUCT, 2, 247, 165, 14 } // InstrumentAttributeGrp
        , { ETI_VAR_STRUCT, 3, 791, 408, 48 } // UnderlyingStipGrp
        , { ETI_VAR_STRING, 0, 2000, VARTEXT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESCompressionRunStatusBroadcast@2398
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, COMPRESSIONACTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, COMPRESSIONSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESCompressionRunStatusRequest@2407
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, COMPRESSIONACTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESCompressionRunStatusResponse@2415
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, COMPRESSIONACTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, COMPRESSIONSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESDeleteBroadcast@2424
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, DELETEREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESExecutionBroadcast@2439
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ALLOCID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESResponse@2454
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESReversalBroadcast@2459
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_TIMESTAMP_NS, 0, 8, REVERSALINITIATIONTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REVERSALCANCELLATIONREASON_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@0
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 132, REVERSALREASONTEXT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 708, 370, 42 } // SideAllocGrpBC
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESTradeBroadcast@2475
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, CLEARINGTRADEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CLEARINGTRADEQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_INT, 0, 8, RELATEDSECURITYID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ROOTPARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ROOTPARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, BASKETTRDMATCHID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, SIDELASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, SIDELASTQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIDETRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ORIGTRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDCLEARINGUNIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, STRATEGYLINKID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDSYMBOL_FH_IDX, 0 }
        , { ETI_INT, 0, 4, TOTNUMTRADEREPORTS_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, NEGOTIATIONID_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSRQSRELATEDTRADEIDS_FH_IDX, 12 } // <- counter@0
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_UINT, 0, 4, BASKETPROFILEID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RELATEDPRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSFERREASON_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MULTILEGREPORTINGTYPE_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MULTILEGPRICEMODEL_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ROOTPARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, REVERSALINDICATOR_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, ROOTPARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYCLEARINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 4, ROOTPARTYCLEARINGORGANIZATION_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, ROOTPARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, ROOTPARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 4, ROOTPARTYIDEXECUTIONVENUE_FH_IDX, 0 }
        , { ETI_STRING, 0, 52, REGULATORYTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, BASKETPARTYCONTRAFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, BASKETSIDETRADEREPORTID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 5, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 645, 323, 39 } // SRQSRelatedTradeIDGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESTradingSessionStatusBroadcast@2545
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADSESEVENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TESUploadBroadcast@2551
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RELATEDTRADEQUANTITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, PACKAGEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TESEXECID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGSETTLEMENTDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGMATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, RELATEDTRADEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPLATFORM_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOLEGS_FH_IDX, 20 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOEVENTS_FH_IDX, 2 } // <- counter@2
        , { ETI_COUNTER, 3, 1, NOINSTRATTRIB_FH_IDX, 6 } // <- counter@3
        , { ETI_COUNTER, 4, 1, NOUNDERLYINGSTIPS_FH_IDX, 1 } // <- counter@4
        , { ETI_UINT_ENUM, 0, 1, HEDGETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDSETTLEMENTLOCATION_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, MESSAGEEVENTSOURCE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, UNDERLYINGSECURITYID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGSECURITYDESC_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, UNDERLYINGCURRENCY_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGISSUER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 3, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 679, 331, 41 } // SideAllocExtGrp
        , { ETI_VAR_STRUCT, 1, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_VAR_STRUCT, 2, 270, 169, 15 } // InstrumentEventGrp
        , { ETI_VAR_STRUCT, 3, 247, 165, 14 } // InstrumentAttributeGrp
        , { ETI_VAR_STRUCT, 4, 791, 408, 48 } // UnderlyingStipGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // TMTradingSessionStatusBroadcast@2597
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_UINT_ENUM, 0, 1, TRADSESEVENT_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // ThrottleUpdateNotification@2602
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 423, 213, 24 } // NotifHeader
        , { ETI_INT, 0, 8, THROTTLETIMEINTERVAL_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, THROTTLENOMSGS_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, THROTTLEDISCONNECTLIMIT_FH_IDX, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TradeBroadcast@2608
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 544, 271, 32 } // RBCHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_INT, 0, 8, RELATEDSECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, PRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LASTQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, SIDELASTPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, SIDELASTQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, CLEARINGTRADEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CLEARINGTRADEQTY_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSACTTIME_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ORDERID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, CLORDID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, LEAVESQTY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, CUMQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ROOTPARTYIDCLIENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, EXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, ROOTPARTYIDINVESTMENTDECISIONMAKER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ORIGTRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDEXECUTINGUNIT_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDSESSIONID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, ROOTPARTYIDCLEARINGUNIT_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDSYMBOL_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, SIDETRADEID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, MATCHDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRDMATCHID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, STRATEGYLINKID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, TOTNUMTRADEREPORTS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MULTILEGREPORTINGTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRANSFERREASON_FH_IDX, 0 }
        , { ETI_STRING, 0, 9, ROOTPARTYIDBENEFICIARY_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYIDTAKEUPTRADINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 7, ROOTPARTYIDORDERORIGINATIONFIRM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MATCHTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, MATCHSUBTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, SIDELIQUIDITYIND_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADINGCAPACITY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERORIGINATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERATTRIBUTERISKREDUCTION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, EXECUTINGTRADERQUALIFIER_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ROOTPARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX, 0 }
        , { ETI_STRING, 0, 2, ACCOUNT_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, ROOTPARTYIDPOSITIONACCOUNT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, POSITIONEFFECT_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, CUSTORDERHANDLINGINST_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT1_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT2_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, FREETEXT3_FH_IDX, 0 }
        , { ETI_CHAR, 0, 1, ORDERCATEGORY_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, RELATEDPRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, ORDERSIDE_FH_IDX, 0 }
        , { ETI_STRING, 0, 4, ROOTPARTYCLEARINGORGANIZATION_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYEXECUTINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 6, ROOTPARTYEXECUTINGTRADER_FH_IDX, 0 }
        , { ETI_STRING, 0, 5, ROOTPARTYCLEARINGFIRM_FH_IDX, 0 }
        , { ETI_STRING, 0, 52, REGULATORYTRADEID_FH_IDX, 0 }
        , { ETI_STRING, 0, 4, ROOTPARTYIDEXECUTIONVENUE_FH_IDX, 0 }
        , { ETI_PADDING, 0, 2, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // TradingSessionStatusBroadcast@2675
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 554, 280, 33 } // RBCHeaderME
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, TRADEDATE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADSESEVENT_FH_IDX, 0 }
        , { ETI_STRING, 0, 16, REFAPPLLASTMSGID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 7, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // UnsubscribeRequest@2683
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, REFAPPLSUBID_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // UnsubscribeResponse@2688
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // UploadTESTradeRequest@2691
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_INT, 0, 8, SECURITYID_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, LASTPX_FH_IDX, 0 }
        , { ETI_TIMESTAMP_NS, 0, 8, TRANSBKDTIME_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 8, 8, UNDERLYINGPX_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 6, 8, RELATEDCLOSEPRICE_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, RELATEDTRADEQUANTITY_FH_IDX, 0 }
        , { ETI_FIXED_POINT, 4, 8, UNDERLYINGQTY_FH_IDX, 0 }
        , { ETI_UINT, 0, 8, COMPRESSIONID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, MARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGSETTLEMENTDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, UNDERLYINGMATURITYDATE_FH_IDX, 0 }
        , { ETI_UINT, 0, 4, RELATEDTRADEID_FH_IDX, 0 }
        , { ETI_INT, 0, 4, RELATEDMARKETSEGMENTID_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 2, TRDTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PRODUCTCOMPLEX_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEREPORTTYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPUBLISHINDICATOR_FH_IDX, 0 }
        , { ETI_COUNTER, 0, 1, NOSIDEALLOCS_FH_IDX, 99 } // <- counter@0
        , { ETI_COUNTER, 1, 1, NOLEGS_FH_IDX, 20 } // <- counter@1
        , { ETI_COUNTER, 2, 1, NOEVENTS_FH_IDX, 2 } // <- counter@2
        , { ETI_COUNTER, 3, 1, NOINSTRATTRIB_FH_IDX, 6 } // <- counter@3
        , { ETI_COUNTER, 4, 1, NOUNDERLYINGSTIPS_FH_IDX, 1 } // <- counter@4
        , { ETI_UINT_ENUM, 0, 1, SKIPVALIDATIONS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRDRPTSTATUS_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, TRADEPLATFORM_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, HEDGETYPE_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, PARTYIDSETTLEMENTLOCATION_FH_IDX, 0 }
        , { ETI_UINT_ENUM, 0, 1, VALUECHECKTYPEMINLOTSIZE_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTID_FH_IDX, 0 }
        , { ETI_STRING, 0, 20, TRADEREPORTTEXT_FH_IDX, 0 }
        , { ETI_STRING, 0, 12, UNDERLYINGSECURITYID_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGSECURITYDESC_FH_IDX, 0 }
        , { ETI_STRING, 0, 3, UNDERLYINGCURRENCY_FH_IDX, 0 }
        , { ETI_STRING, 0, 30, UNDERLYINGISSUER_FH_IDX, 0 }
        , { ETI_PADDING, 0, 1, 0, 0 }
        , { ETI_VAR_STRUCT, 0, 679, 331, 41 } // SideAllocExtGrp
        , { ETI_VAR_STRUCT, 1, 773, 404, 47 } // TrdInstrmntLegGrp
        , { ETI_VAR_STRUCT, 2, 270, 169, 15 } // InstrumentEventGrp
        , { ETI_VAR_STRUCT, 3, 247, 165, 14 } // InstrumentAttributeGrp
        , { ETI_VAR_STRUCT, 4, 791, 408, 48 } // UnderlyingStipGrp
        , { ETI_EOF, 0, 0, 0, 0 }
        // UserLoginRequest@2734
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, USERNAME_FH_IDX, 0 }
        , { ETI_STRING, 0, 32, PASSWORD_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // UserLoginResponse@2740
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_EOF, 0, 0, 0, 0 }
        // UserLogoutRequest@2743
        , { ETI_STRUCT, 0, 314, 184, 18 } // MessageHeaderIn
        , { ETI_STRUCT, 0, 566, 291, 34 } // RequestHeader
        , { ETI_UINT, 0, 4, USERNAME_FH_IDX, 0 }
        , { ETI_PADDING, 0, 4, 0, 0 }
        , { ETI_EOF, 0, 0, 0, 0 }
        // UserLogoutResponse@2748
        , { ETI_STRUCT, 0, 330, 189, 19 } // MessageHeaderOut
        , { ETI_STRUCT, 0, 580, 294, 35 } // ResponseHeader
        , { ETI_EOF, 0, 0, 0, 0 }
    };
    static const int16_t tid2fidx[] = {
          1048 /* LogonRequest */
        , 1065 /* LogonResponse */
        , 1078 /* LogoutRequest */
        , 1081 /* LogoutResponse */
        , -1
        , 2274 /* SubscribeResponse */
        , 2683 /* UnsubscribeRequest */
        , 2688 /* UnsubscribeResponse */
        , 1759 /* RetransmitRequest */
        , 1767 /* RetransmitResponse */
        , 1735 /* Reject */
        , 983 /* Heartbeat */
        , 969 /* ForcedLogoutNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , 2734 /* UserLoginRequest */
        , 2740 /* UserLoginResponse */
        , -1
        , -1
        , -1
        , 985 /* HeartbeatNotification */
        , 2748 /* UserLogoutResponse */
        , 2268 /* SubscribeRequest */
        , 1743 /* RetransmitMEMessageRequest */
        , 1752 /* RetransmitMEMessageResponse */
        , 2602 /* ThrottleUpdateNotification */
        , 2743 /* UserLogoutRequest */
        , 2247 /* ServiceAvailabilityBroadcast */
        , 1508 /* NewsBroadcast */
        , 621 /* BroadcastErrorNotification */
        , 1785 /* RiskNotificationBroadcast */
        , 1636 /* PartyEntitlementsUpdateReport */
        , 1020 /* InquireSessionListRequest */
        , 1023 /* InquireSessionListResponse */
        , 1040 /* LegalNotificationBroadcast */
        , 1029 /* InquireUserRequest */
        , 1033 /* InquireUserResponse */
        , 988 /* InquireEnrichmentRuleIDListRequest */
        , 992 /* InquireEnrichmentRuleIDListResponse */
        , 1624 /* PartyActionReport */
        , 975 /* ForcedUserLogoutNotification */
        , 2262 /* ServiceAvailabilityMarketBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1442 /* NewOrderSingleRequest */
        , 1426 /* NewOrderResponse */
        , 1412 /* NewOrderNRResponse */
        , 1598 /* OrderExecResponse */
        , 1516 /* OrderExecNotification */
        , -1
        , 1254 /* ModifyOrderSingleRequest */
        , 1235 /* ModifyOrderResponse */
        , 1217 /* ModifyOrderNRResponse */
        , 864 /* DeleteOrderSingleRequest */
        , 848 /* DeleteOrderResponse */
        , 832 /* DeleteOrderNRResponse */
        , 795 /* DeleteOrderBroadcast */
        , 1345 /* NewOrderComplexRequest */
        , 1146 /* ModifyOrderComplexRequest */
        , -1
        , -1
        , 1541 /* OrderExecReportBroadcast */
        , 676 /* CrossRequest */
        , 683 /* CrossRequestResponse */
        , 718 /* DeleteAllOrderRequest */
        , 732 /* DeleteAllOrderResponse */
        , 687 /* DeleteAllOrderBroadcast */
        , 816 /* DeleteOrderComplexRequest */
        , 705 /* DeleteAllOrderNRResponse */
        , 1484 /* NewOrderSingleShortRequest */
        , 1299 /* ModifyOrderSingleShortRequest */
        , -1
        , -1
        , 1386 /* NewOrderComplexShortRequest */
        , 1190 /* ModifyOrderComplexShortRequest */
        , 910 /* EnterCLIPRequest */
        , 783 /* DeleteCLIPRequest */
        , 666 /* CLIPResponse */
        , 631 /* CLIPDeletionNotification */
        , 645 /* CLIPExecutionNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 412 /* AddComplexInstrumentRequest */
        , 422 /* AddComplexInstrumentResponse */
        , 1084 /* MMParameterDefinitionRequest */
        , 1095 /* MMParameterDefinitionResponse */
        , 999 /* InquireMMParameterRequest */
        , 1004 /* InquireMMParameterResponse */
        , 2675 /* TradingSessionStatusBroadcast */
        , 709 /* DeleteAllOrderQuoteEventBroadcast */
        , 438 /* AddFlexibleInstrumentRequest */
        , 449 /* AddFlexibleInstrumentResponse */
        , 1012 /* InquirePreTradeRiskLimitsRequest */
        , 1669 /* PreTradeRiskLimitsDefinitionRequest */
        , 1658 /* PreTradeRiskLimitResponse */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1649 /* PingRequest */
        , 1654 /* PingResponse */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1722 /* RFQRequest */
        , 1731 /* RFQResponse */
        , 1693 /* QuoteActivationRequest */
        , 1705 /* QuoteActivationResponse */
        , 1099 /* MassQuoteRequest */
        , 1118 /* MassQuoteResponse */
        , 1712 /* QuoteExecutionReport */
        , 756 /* DeleteAllQuoteRequest */
        , 766 /* DeleteAllQuoteResponse */
        , 741 /* DeleteAllQuoteBroadcast */
        , 1680 /* QuoteActivationNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 2608 /* TradeBroadcast */
        , 2597 /* TMTradingSessionStatusBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 929 /* EnterTESTradeRequest */
        , 1324 /* ModifyTESTradeRequest */
        , 880 /* DeleteTESTradeRequest */
        , 508 /* ApproveTESTradeRequest */
        , 2348 /* TESBroadcast */
        , -1
        , 2424 /* TESDeleteBroadcast */
        , 2279 /* TESApproveBroadcast */
        , -1
        , -1
        , 2439 /* TESExecutionBroadcast */
        , 2454 /* TESResponse */
        , 2691 /* UploadTESTradeRequest */
        , 2551 /* TESUploadBroadcast */
        , 2475 /* TESTradeBroadcast */
        , 2545 /* TESTradingSessionStatusBroadcast */
        , 2407 /* TESCompressionRunStatusRequest */
        , 2415 /* TESCompressionRunStatusResponse */
        , 2398 /* TESCompressionRunStatusBroadcast */
        , -1
        , 892 /* EnterBasketTradeRequest */
        , 1127 /* ModifyBasketTradeRequest */
        , 773 /* DeleteBasketTradeRequest */
        , 482 /* ApproveBasketTradeRequest */
        , 616 /* BasketResponse */
        , 566 /* BasketBroadcast */
        , 588 /* BasketDeleteBroadcast */
        , 545 /* BasketApproveBroadcast */
        , 601 /* BasketExecutionBroadcast */
        , 463 /* AmendBasketTradeRequest */
        , 1774 /* ReverseTESTradeRequest */
        , 497 /* ApproveReverseTESTradeRequest */
        , 2459 /* TESReversalBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 2092 /* SRQSOpenNegotiationRequest */
        , 2227 /* SRQSUpdateNegotiationRequest */
        , 1895 /* SRQSEnterQuoteRequest */
        , 2191 /* SRQSQuoteResponse */
        , 1929 /* SRQSHitQuoteRequest */
        , 1884 /* SRQSDealResponse */
        , 2212 /* SRQSUpdateDealStatusRequest */
        , 2157 /* SRQSQuoteNotification */
        , 1798 /* SRQSCreateDealNotification */
        , 1846 /* SRQSDealNotification */
        , 2120 /* SRQSOpenNegotiationRequesterNotification */
        , 2054 /* SRQSOpenNegotiationNotification */
        , 2013 /* SRQSNegotiationRequesterNotification */
        , 1982 /* SRQSNegotiationNotification */
        , 2206 /* SRQSStatusBroadcast */
        , 2045 /* SRQSNegotiationStatusNotification */
        , -1
        , 2197 /* SRQSQuotingStatusRequest */
        , 1967 /* SRQSInquireSmartRespondentRequest */
        , 1975 /* SRQSInquireSmartRespondentResponse */
    };
    static const uint32_t tid2size[720][2] = {
          { 280, 280 } /* LogonRequest */
        , { 96, 96 } /* LogonResponse */
        , { 24, 24 } /* LogoutRequest */
        , { 32, 32 } /* LogoutResponse */
        , { 0, 0}
        , { 40, 40 } /* SubscribeResponse */
        , { 32, 32 } /* UnsubscribeRequest */
        , { 32, 32 } /* UnsubscribeResponse */
        , { 48, 48 } /* RetransmitRequest */
        , { 56, 56 } /* RetransmitResponse */
        , { 64, 2064 } /* Reject */
        , { 16, 16 } /* Heartbeat */
        , { 24, 2024 } /* ForcedLogoutNotification */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 64, 64 } /* UserLoginRequest */
        , { 32, 32 } /* UserLoginResponse */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 16, 16 } /* HeartbeatNotification */
        , { 32, 32 } /* UserLogoutResponse */
        , { 32, 32 } /* SubscribeRequest */
        , { 64, 64 } /* RetransmitMEMessageRequest */
        , { 72, 72 } /* RetransmitMEMessageResponse */
        , { 32, 32 } /* ThrottleUpdateNotification */
        , { 32, 32 } /* UserLogoutRequest */
        , { 56, 56 } /* ServiceAvailabilityBroadcast */
        , { 312, 2312 } /* NewsBroadcast */
        , { 32, 2032 } /* BroadcastErrorNotification */
        , { 88, 88 } /* RiskNotificationBroadcast */
        , { 88, 88 } /* PartyEntitlementsUpdateReport */
        , { 24, 24 } /* InquireSessionListRequest */
        , { 48, 8040 } /* InquireSessionListResponse */
        , { 56, 2056 } /* LegalNotificationBroadcast */
        , { 40, 40 } /* InquireUserRequest */
        , { 56, 16056 } /* InquireUserResponse */
        , { 40, 40 } /* InquireEnrichmentRuleIDListRequest */
        , { 56, 25656 } /* InquireEnrichmentRuleIDListResponse */
        , { 72, 72 } /* PartyActionReport */
        , { 24, 2024 } /* ForcedUserLogoutNotification */
        , { 32, 32 } /* ServiceAvailabilityMarketBroadcast */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 248, 248 } /* NewOrderSingleRequest */
        , { 128, 128 } /* NewOrderResponse */
        , { 96, 96 } /* NewOrderNRResponse */
        , { 168, 22568 } /* OrderExecResponse */
        , { 168, 22568 } /* OrderExecNotification */
        , { 0, 0}
        , { 272, 272 } /* ModifyOrderSingleRequest */
        , { 152, 152 } /* ModifyOrderResponse */
        , { 128, 128 } /* ModifyOrderNRResponse */
        , { 104, 104 } /* DeleteOrderSingleRequest */
        , { 136, 136 } /* DeleteOrderResponse */
        , { 120, 120 } /* DeleteOrderNRResponse */
        , { 160, 160 } /* DeleteOrderBroadcast */
        , { 240, 400 } /* NewOrderComplexRequest */
        , { 264, 424 } /* ModifyOrderComplexRequest */
        , { 0, 0}
        , { 0, 0}
        , { 352, 22912 } /* OrderExecReportBroadcast */
        , { 64, 64 } /* CrossRequest */
        , { 64, 64 } /* CrossRequestResponse */
        , { 72, 72 } /* DeleteAllOrderRequest */
        , { 88, 12088 } /* DeleteAllOrderResponse */
        , { 112, 12112 } /* DeleteAllOrderBroadcast */
        , { 104, 104 } /* DeleteOrderComplexRequest */
        , { 64, 64 } /* DeleteAllOrderNRResponse */
        , { 96, 96 } /* NewOrderSingleShortRequest */
        , { 104, 104 } /* ModifyOrderSingleShortRequest */
        , { 0, 0}
        , { 0, 0}
        , { 104, 104 } /* NewOrderComplexShortRequest */
        , { 112, 112 } /* ModifyOrderComplexShortRequest */
        , { 240, 720 } /* EnterCLIPRequest */
        , { 72, 72 } /* DeleteCLIPRequest */
        , { 104, 120 } /* CLIPResponse */
        , { 112, 112 } /* CLIPDeletionNotification */
        , { 128, 22528 } /* CLIPExecutionNotification */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 56, 696 } /* AddComplexInstrumentRequest */
        , { 112, 752 } /* AddComplexInstrumentResponse */
        , { 72, 72 } /* MMParameterDefinitionRequest */
        , { 64, 64 } /* MMParameterDefinitionResponse */
        , { 32, 32 } /* InquireMMParameterRequest */
        , { 72, 432 } /* InquireMMParameterResponse */
        , { 96, 96 } /* TradingSessionStatusBroadcast */
        , { 88, 88 } /* DeleteAllOrderQuoteEventBroadcast */
        , { 64, 64 } /* AddFlexibleInstrumentRequest */
        , { 96, 96 } /* AddFlexibleInstrumentResponse */
        , { 40, 40 } /* InquirePreTradeRiskLimitsRequest */
        , { 40, 72 } /* PreTradeRiskLimitsDefinitionRequest */
        , { 80, 2128 } /* PreTradeRiskLimitResponse */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 32, 32 } /* PingRequest */
        , { 64, 64 } /* PingResponse */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 72, 72 } /* RFQRequest */
        , { 64, 64 } /* RFQResponse */
        , { 56, 56 } /* QuoteActivationRequest */
        , { 72, 4072 } /* QuoteActivationResponse */
        , { 72, 4072 } /* MassQuoteRequest */
        , { 80, 4880 } /* MassQuoteResponse */
        , { 128, 24080 } /* QuoteExecutionReport */
        , { 56, 56 } /* DeleteAllQuoteRequest */
        , { 72, 4072 } /* DeleteAllQuoteResponse */
        , { 104, 4104 } /* DeleteAllQuoteBroadcast */
        , { 88, 4088 } /* QuoteActivationNotification */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 416, 416 } /* TradeBroadcast */
        , { 48, 48 } /* TMTradingSessionStatusBroadcast */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 272, 4184 } /* EnterTESTradeRequest */
        , { 144, 3760 } /* ModifyTESTradeRequest */
        , { 72, 72 } /* DeleteTESTradeRequest */
        , { 240, 240 } /* ApproveTESTradeRequest */
        , { 328, 7024 } /* TESBroadcast */
        , { 0, 0}
        , { 96, 96 } /* TESDeleteBroadcast */
        , { 424, 3200 } /* TESApproveBroadcast */
        , { 0, 0}
        , { 0, 0}
        , { 80, 80 } /* TESExecutionBroadcast */
        , { 56, 56 } /* TESResponse */
        , { 416, 18440 } /* UploadTESTradeRequest */
        , { 440, 18464 } /* TESUploadBroadcast */
        , { 432, 528 } /* TESTradeBroadcast */
        , { 48, 48 } /* TESTradingSessionStatusBroadcast */
        , { 40, 40 } /* TESCompressionRunStatusRequest */
        , { 48, 48 } /* TESCompressionRunStatusResponse */
        , { 56, 56 } /* TESCompressionRunStatusBroadcast */
        , { 0, 0}
        , { 272, 27232 } /* EnterBasketTradeRequest */
        , { 280, 27240 } /* ModifyBasketTradeRequest */
        , { 64, 64 } /* DeleteBasketTradeRequest */
        , { 296, 73344 } /* ApproveBasketTradeRequest */
        , { 56, 56 } /* BasketResponse */
        , { 352, 27312 } /* BasketBroadcast */
        , { 96, 96 } /* BasketDeleteBroadcast */
        , { 400, 76624 } /* BasketApproveBroadcast */
        , { 112, 3280 } /* BasketExecutionBroadcast */
        , { 280, 27240 } /* AmendBasketTradeRequest */
        , { 200, 200 } /* ReverseTESTradeRequest */
        , { 72, 72 } /* ApproveReverseTESTradeRequest */
        , { 264, 4184 } /* TESReversalBroadcast */
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 0, 0}
        , { 392, 2472 } /* SRQSOpenNegotiationRequest */
        , { 352, 1952 } /* SRQSUpdateNegotiationRequest */
        , { 368, 368 } /* SRQSEnterQuoteRequest */
        , { 64, 64 } /* SRQSQuoteResponse */
        , { 384, 384 } /* SRQSHitQuoteRequest */
        , { 104, 104 } /* SRQSDealResponse */
        , { 208, 208 } /* SRQSUpdateDealStatusRequest */
        , { 408, 408 } /* SRQSQuoteNotification */
        , { 472, 1720 } /* SRQSCreateDealNotification */
        , { 424, 424 } /* SRQSDealNotification */
        , { 456, 2536 } /* SRQSOpenNegotiationRequesterNotification */
        , { 488, 968 } /* SRQSOpenNegotiationNotification */
        , { 456, 2056 } /* SRQSNegotiationRequesterNotification */
        , { 456, 456 } /* SRQSNegotiationNotification */
        , { 48, 48 } /* SRQSStatusBroadcast */
        , { 88, 88 } /* SRQSNegotiationStatusNotification */
        , { 0, 0}
        , { 176, 176 } /* SRQSQuotingStatusRequest */
        , { 32, 32 } /* SRQSInquireSmartRespondentRequest */
        , { 40, 16040 } /* SRQSInquireSmartRespondentResponse */
    };
    static const unsigned char usages[] = {
        // AddComplexInstrumentRequest
        //// MessageHeaderInComp
          0 // BodyLen#0
        , 0 // TemplateID#1
        , 2 // NetworkMsgID#2
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#3
        , 0 // SenderSubID#4
        ///
        , 0 // MarketSegmentID#5
        , 1 // SecuritySubType#6
        , 0 // ProductComplex#7
        , 0 // NoLegs#8
        , 1 // ComplianceText#9
        //// InstrmtLegGrpComp
        , 0 // LegSecurityID#10
        , 1 // LegPrice#11
        , 1 // LegSymbol#12
        , 0 // LegRatioQty#13
        , 0 // LegSide#14
        , 0 // LegSecurityType#15
        ///
        // AddComplexInstrumentResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#16
        , 0 // TemplateID#17
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#18
        , 0 // TrdRegTSTimeIn#19
        , 0 // TrdRegTSTimeOut#20
        , 0 // ResponseIn#21
        , 0 // SendingTime#22
        , 0 // MsgSeqNum#23
        , 0 // LastFragment#24
        ///
        , 1 // LowLimitPrice#25
        , 1 // HighLimitPrice#26
        , 0 // SecurityID#27
        , 0 // LastUpdateTime#28
        , 0 // SecurityResponseID#29
        , 0 // MarketSegmentID#30
        , 0 // NumberOfSecurities#31
        , 1 // SecuritySubType#32
        , 0 // MultilegModel#33
        , 0 // ImpliedMarketIndicator#34
        , 0 // ProductComplex#35
        , 0 // NoLegs#36
        //// InstrmtLegGrpComp
        , 0 // LegSecurityID#37
        , 1 // LegPrice#38
        , 1 // LegSymbol#39
        , 0 // LegRatioQty#40
        , 0 // LegSide#41
        , 0 // LegSecurityType#42
        ///
        // AddFlexibleInstrumentRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#43
        , 0 // TemplateID#44
        , 2 // NetworkMsgID#45
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#46
        , 0 // SenderSubID#47
        ///
        , 1 // StrikePrice#48
        , 0 // MarketSegmentID#49
        , 0 // MaturityDate#50
        , 0 // SettlMethod#51
        , 1 // OptAttribute#52
        , 1 // PutOrCall#53
        , 1 // ExerciseStyle#54
        , 1 // ComplianceText#55
        // AddFlexibleInstrumentResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#56
        , 0 // TemplateID#57
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#58
        , 0 // TrdRegTSTimeIn#59
        , 0 // TrdRegTSTimeOut#60
        , 0 // ResponseIn#61
        , 0 // SendingTime#62
        , 0 // MsgSeqNum#63
        , 0 // LastFragment#64
        ///
        , 0 // SecurityResponseID#65
        , 0 // SecurityID#66
        , 1 // StrikePrice#67
        , 0 // MarketSegmentID#68
        , 0 // MaturityDate#69
        , 1 // ProductComplex#70
        , 0 // SettlMethod#71
        , 1 // OptAttribute#72
        , 1 // PutOrCall#73
        , 1 // ExerciseStyle#74
        // AmendBasketTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#75
        , 0 // TemplateID#76
        , 2 // NetworkMsgID#77
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#78
        , 0 // SenderSubID#79
        ///
        , 0 // BasketTrdMatchID#80
        , 0 // BasketExecID#81
        , 0 // MarketSegmentID#82
        , 0 // MaturityMonthYear#83
        , 1 // BasketProfileID#84
        , 0 // TrdType#85
        , 0 // NoBasketSideAlloc#86
        , 0 // TradeReportType#87
        , 0 // BasketTradeReportType#88
        , 0 // NoBasketRootPartyGrps#89
        , 0 // NoInstrmtMatchSides#90
        , 1 // BasketTradeReportText#91
        , 1 // TradeReportID#92
        //// BasketRootPartyGrpComp
        , 0 // RootPartySubIDType#93
        , 0 // RootPartyContraFirm#94
        , 0 // RootPartyContraTrader#95
        , 1 // BasketSideTradeReportID#96
        ///
        //// InstrmtMatchSideGrpComp
        , 0 // SecurityID#97
        , 0 // LastPx#98
        , 1 // TransBkdTime#99
        , 1 // RelatedClosePrice#100
        , 2 // PackageID#101
        , 0 // SideMarketSegmentID#102
        , 0 // SideTrdSubTyp#103
        , 0 // ProductComplex#104
        , 0 // TradePublishIndicator#105
        , 0 // InstrmtMatchSideID#106
        , 1 // EffectOnBasket#107
        , 1 // TradeReportText#108
        ///
        //// BasketSideAllocGrpComp
        , 0 // AllocQty#109
        , 2 // IndividualAllocID#110
        , 0 // PartySubIDType#111
        , 0 // Side#112
        , 0 // InstrmtMatchSideID#113
        , 2 // TradeAllocStatus#114
        , 0 // PartyExecutingFirm#115
        , 0 // PartyExecutingTrader#116
        ///
        // ApproveBasketTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#117
        , 0 // TemplateID#118
        , 2 // NetworkMsgID#119
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#120
        , 0 // SenderSubID#121
        ///
        , 0 // BasketTrdMatchID#122
        , 0 // BasketExecID#123
        , 0 // MarketSegmentID#124
        , 0 // RootPartySubIDType#125
        , 0 // NoBasketSideAlloc#126
        , 0 // TrdType#127
        , 0 // TradeReportType#128
        , 1 // BasketTradeReportText#129
        , 1 // TradeReportID#130
        , 1 // BasketSideTradeReportID#131
        //// BasketSideAllocExtGrpComp
        , 0 // AllocQty#132
        , 1 // PartyIDClientID#133
        , 1 // PartyIdInvestmentDecisionMaker#134
        , 1 // ExecutingTrader#135
        , 0 // PackageID#136
        , 0 // SideMarketSegmentID#137
        , 0 // AllocID#138
        , 0 // Side#139
        , 0 // PositionEffect#140
        , 0 // TradingCapacity#141
        , 0 // OrderAttributeLiquidityProvision#142
        , 0 // ExecutingTraderQualifier#143
        , 1 // PartyIdInvestmentDecisionMakerQualifier#144
        , 1 // OrderAttributeRiskReduction#145
        , 1 // OrderOrigination#146
        , 0 // PartyExecutingFirm#147
        , 0 // PartyExecutingTrader#148
        , 1 // Account#149
        , 1 // FreeText1#150
        , 1 // FreeText2#151
        , 1 // FreeText3#152
        , 1 // PartyIDTakeUpTradingFirm#153
        , 1 // PartyIDOrderOriginationFirm#154
        , 1 // PartyIDBeneficiary#155
        , 1 // PartyIDPositionAccount#156
        , 1 // PartyIDLocationID#157
        , 1 // CustOrderHandlingInst#158
        , 1 // ComplianceText#159
        ///
        // ApproveReverseTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#160
        , 0 // TemplateID#161
        , 2 // NetworkMsgID#162
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#163
        , 0 // SenderSubID#164
        ///
        , 0 // MarketSegmentID#165
        , 0 // PackageID#166
        , 0 // AllocID#167
        , 0 // TESExecID#168
        , 1 // RelatedMarketSegmentID#169
        , 0 // TrdType#170
        , 1 // TradeReportID#171
        // ApproveTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#172
        , 0 // TemplateID#173
        , 2 // NetworkMsgID#174
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#175
        , 0 // SenderSubID#176
        ///
        , 1 // PartyIDClientID#177
        , 1 // PartyIdInvestmentDecisionMaker#178
        , 1 // ExecutingTrader#179
        , 0 // AllocQty#180
        , 1 // CompressionID#181
        , 0 // PackageID#182
        , 0 // AllocID#183
        , 0 // TESExecID#184
        , 0 // MarketSegmentID#185
        , 1 // RelatedMarketSegmentID#186
        , 0 // TrdType#187
        , 0 // TradingCapacity#188
        , 0 // TradeReportType#189
        , 0 // Side#190
        , 0 // OrderAttributeLiquidityProvision#191
        , 1 // PartyIdInvestmentDecisionMakerQualifier#192
        , 0 // ExecutingTraderQualifier#193
        , 1 // OrderAttributeRiskReduction#194
        , 1 // OrderOrigination#195
        , 1 // TradeReportID#196
        , 0 // PositionEffect#197
        , 0 // PartyExecutingFirm#198
        , 0 // PartyExecutingTrader#199
        , 1 // Account#200
        , 1 // FreeText1#201
        , 1 // FreeText2#202
        , 1 // FreeText3#203
        , 1 // PartyIDTakeUpTradingFirm#204
        , 1 // PartyIDPositionAccount#205
        , 1 // PartyIDOrderOriginationFirm#206
        , 1 // PartyIDBeneficiary#207
        , 1 // PartyIDLocationID#208
        , 1 // CustOrderHandlingInst#209
        , 1 // ComplianceText#210
        // BasketApproveBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#211
        , 0 // TemplateID#212
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#213
        , 0 // ApplSeqNum#214
        , 1 // ApplSubID#215
        , 0 // PartitionID#216
        , 0 // ApplResendFlag#217
        , 0 // ApplID#218
        , 0 // LastFragment#219
        ///
        , 0 // BasketTrdMatchID#220
        , 0 // TransactTime#221
        , 0 // BasketExecID#222
        , 0 // MarketSegmentID#223
        , 1 // BasketProfileID#224
        , 0 // TrdType#225
        , 0 // NoBasketSideAlloc#226
        , 0 // TradeReportType#227
        , 0 // BasketTradeReportType#228
        , 0 // MessageEventSource#229
        , 0 // NoBasketRootPartyGrps#230
        , 0 // PartyIDEnteringFirm#231
        , 0 // PartyEnteringTrader#232
        , 1 // BasketTradeReportText#233
        , 1 // TradeReportID#234
        //// BasketRootPartyGrpComp
        , 0 // RootPartySubIDType#235
        , 0 // RootPartyContraFirm#236
        , 0 // RootPartyContraTrader#237
        , 1 // BasketSideTradeReportID#238
        ///
        //// BasketSideAllocExtBCGrpComp
        , 0 // AllocQty#239
        , 0 // SecurityID#240
        , 0 // LastPx#241
        , 1 // TransBkdTime#242
        , 1 // RelatedClosePrice#243
        , 0 // PackageID#244
        , 0 // SideMarketSegmentID#245
        , 0 // AllocID#246
        , 0 // SideTrdSubTyp#247
        , 0 // PartySubIDType#248
        , 0 // Side#249
        , 0 // PositionEffect#250
        , 1 // EffectOnBasket#251
        , 0 // TradingCapacity#252
        , 0 // TradeAllocStatus#253
        , 0 // ProductComplex#254
        , 0 // TradePublishIndicator#255
        , 0 // PartyExecutingFirm#256
        , 0 // PartyExecutingTrader#257
        , 1 // Account#258
        , 1 // FreeText1#259
        , 1 // FreeText2#260
        , 1 // FreeText3#261
        , 1 // PartyIDTakeUpTradingFirm#262
        , 1 // PartyIDOrderOriginationFirm#263
        , 1 // PartyIDBeneficiary#264
        , 1 // PartyIDPositionAccount#265
        , 1 // PartyIDLocationID#266
        , 1 // CustOrderHandlingInst#267
        , 1 // ComplianceText#268
        ///
        // BasketBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#269
        , 0 // TemplateID#270
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#271
        , 0 // ApplSeqNum#272
        , 1 // ApplSubID#273
        , 0 // PartitionID#274
        , 0 // ApplResendFlag#275
        , 0 // ApplID#276
        , 0 // LastFragment#277
        ///
        , 0 // BasketTrdMatchID#278
        , 0 // TransactTime#279
        , 0 // BasketExecID#280
        , 0 // MarketSegmentID#281
        , 0 // MaturityMonthYear#282
        , 1 // BasketProfileID#283
        , 0 // NoBasketSideAlloc#284
        , 0 // TrdType#285
        , 0 // TradeReportType#286
        , 0 // BasketTradeReportType#287
        , 0 // MessageEventSource#288
        , 0 // NoBasketRootPartyGrpsBC#289
        , 0 // NoInstrmtMatchSides#290
        , 1 // BasketTradeReportText#291
        , 1 // TradeReportID#292
        //// BasketRootPartyGrpComp
        , 0 // RootPartySubIDType#293
        , 0 // RootPartyContraFirm#294
        , 0 // RootPartyContraTrader#295
        , 1 // BasketSideTradeReportID#296
        ///
        //// InstrmtMatchSideGrpComp
        , 0 // SecurityID#297
        , 0 // LastPx#298
        , 1 // TransBkdTime#299
        , 1 // RelatedClosePrice#300
        , 0 // PackageID#301
        , 0 // SideMarketSegmentID#302
        , 0 // SideTrdSubTyp#303
        , 0 // ProductComplex#304
        , 0 // TradePublishIndicator#305
        , 0 // InstrmtMatchSideID#306
        , 1 // EffectOnBasket#307
        , 1 // TradeReportText#308
        ///
        //// BasketSideAllocGrpComp
        , 0 // AllocQty#309
        , 0 // IndividualAllocID#310
        , 0 // PartySubIDType#311
        , 0 // Side#312
        , 0 // InstrmtMatchSideID#313
        , 0 // TradeAllocStatus#314
        , 0 // PartyExecutingFirm#315
        , 0 // PartyExecutingTrader#316
        ///
        // BasketDeleteBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#317
        , 0 // TemplateID#318
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#319
        , 0 // ApplSeqNum#320
        , 1 // ApplSubID#321
        , 0 // PartitionID#322
        , 0 // ApplResendFlag#323
        , 0 // ApplID#324
        , 0 // LastFragment#325
        ///
        , 0 // BasketTrdMatchID#326
        , 0 // TransactTime#327
        , 0 // MarketSegmentID#328
        , 0 // BasketExecID#329
        , 1 // BasketProfileID#330
        , 0 // TrdType#331
        , 0 // DeleteReason#332
        , 0 // MessageEventSource#333
        , 1 // TradeReportID#334
        // BasketExecutionBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#335
        , 0 // TemplateID#336
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#337
        , 0 // ApplSeqNum#338
        , 1 // ApplSubID#339
        , 0 // PartitionID#340
        , 0 // ApplResendFlag#341
        , 0 // ApplID#342
        , 0 // LastFragment#343
        ///
        , 0 // BasketTrdMatchID#344
        , 0 // TransactTime#345
        , 0 // BasketExecID#346
        , 0 // MarketSegmentID#347
        , 1 // BasketProfileID#348
        , 0 // TrdType#349
        , 0 // TradeReportType#350
        , 0 // NoInstrmtMatchSides#351
        , 0 // MessageEventSource#352
        , 1 // BasketSideTradeReportID#353
        //// BasketExecGrpComp
        , 0 // PackageID#354
        , 0 // SideMarketSegmentID#355
        , 0 // AllocID#356
        , 0 // SideTrdSubTyp#357
        ///
        // BasketResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#358
        , 0 // TemplateID#359
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#360
        , 0 // SendingTime#361
        , 0 // MsgSeqNum#362
        ///
        , 0 // BasketExecID#363
        , 1 // TradeReportID#364
        // BroadcastErrorNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#365
        , 0 // TemplateID#366
        ///
        //// NotifHeaderComp
        , 0 // SendingTime#367
        ///
        , 0 // ApplIDStatus#368
        , 1 // RefApplSubID#369
        , 0 // VarTextLen#370
        , 0 // RefApplID#371
        , 0 // SessionStatus#372
        , 0 // VarText#373
        // CLIPDeletionNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#374
        , 0 // TemplateID#375
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#376
        , 1 // NotificationIn#377
        , 0 // SendingTime#378
        , 2 // ApplSubID#379
        , 0 // PartitionID#380
        , 2 // ApplMsgID#381
        , 0 // ApplID#382
        , 2 // ApplResendFlag#383
        , 0 // LastFragment#384
        ///
        , 0 // OrderID#385
        , 0 // ExecID#386
        , 0 // SecurityID#387
        , 0 // CxlQty#388
        , 0 // MarketSegmentID#389
        , 0 // ExecRestatementReason#390
        , 0 // ProductComplex#391
        , 0 // Side#392
        , 0 // OrdStatus#393
        , 0 // ExecType#394
        // CLIPExecutionNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#395
        , 0 // TemplateID#396
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#397
        , 1 // NotificationIn#398
        , 0 // SendingTime#399
        , 2 // ApplSubID#400
        , 0 // PartitionID#401
        , 0 // ApplMsgID#402
        , 0 // ApplID#403
        , 0 // ApplResendFlag#404
        , 0 // LastFragment#405
        ///
        , 0 // OrderID#406
        , 0 // ExecID#407
        , 0 // SecurityID#408
        , 0 // CxlQty#409
        , 0 // LeavesQty#410
        , 0 // CumQty#411
        , 0 // MarketSegmentID#412
        , 0 // ExecRestatementReason#413
        , 0 // NoLegExecs#414
        , 0 // ProductComplex#415
        , 0 // Side#416
        , 0 // OrdStatus#417
        , 0 // ExecType#418
        , 0 // MatchType#419
        , 0 // NoFills#420
        //// FillsGrpComp
        , 0 // FillPx#421
        , 0 // FillQty#422
        , 0 // FillMatchID#423
        , 0 // FillExecID#424
        , 1 // FillLiquidityInd#425
        ///
        //// InstrmntLegExecGrpComp
        , 0 // LegSecurityID#426
        , 0 // LegLastPx#427
        , 0 // LegLastQty#428
        , 0 // LegExecID#429
        , 0 // LegSide#430
        , 0 // FillRefID#431
        ///
        // CLIPResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#432
        , 0 // TemplateID#433
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#434
        , 0 // TrdRegTSTimeIn#435
        , 0 // TrdRegTSTimeOut#436
        , 0 // ResponseIn#437
        , 0 // SendingTime#438
        , 0 // MsgSeqNum#439
        , 0 // LastFragment#440
        ///
        , 0 // ExecID#441
        , 0 // SecurityID#442
        , 0 // MarketSegmentID#443
        , 1 // CrossRequestID#444
        , 0 // NoSides#445
        //// CrossRequestAckSideGrpComp
        , 0 // OrderID#446
        , 0 // InputSource#447
        , 0 // Side#448
        ///
        // CrossRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#449
        , 0 // TemplateID#450
        , 2 // NetworkMsgID#451
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#452
        , 0 // SenderSubID#453
        ///
        , 0 // SecurityID#454
        , 0 // OrderQty#455
        , 0 // MarketSegmentID#456
        , 1 // ComplianceText#457
        // CrossRequestResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#458
        , 0 // TemplateID#459
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#460
        , 0 // TrdRegTSTimeIn#461
        , 0 // TrdRegTSTimeOut#462
        , 0 // ResponseIn#463
        , 0 // SendingTime#464
        , 0 // MsgSeqNum#465
        , 0 // LastFragment#466
        ///
        , 0 // ExecID#467
        // DeleteAllOrderBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#468
        , 0 // TemplateID#469
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#470
        , 1 // NotificationIn#471
        , 0 // SendingTime#472
        , 1 // ApplSubID#473
        , 0 // PartitionID#474
        , 0 // ApplMsgID#475
        , 0 // ApplID#476
        , 0 // ApplResendFlag#477
        , 0 // LastFragment#478
        ///
        , 0 // MassActionReportID#479
        , 1 // SecurityID#480
        , 1 // Price#481
        , 0 // MarketSegmentID#482
        , 0 // TargetPartyIDSessionID#483
        , 1 // TargetPartyIDExecutingTrader#484
        , 1 // PartyIDEnteringTrader#485
        , 0 // NoNotAffectedOrders#486
        , 0 // NoAffectedOrderRequests#487
        , 1 // PartyIDEnteringFirm#488
        , 0 // MassActionReason#489
        , 0 // ExecInst#490
        , 1 // Side#491
        //// NotAffectedOrdersGrpComp
        , 0 // NotAffectedOrderID#492
        , 1 // NotAffOrigClOrdID#493
        ///
        //// AffectedOrderRequestsGrpComp
        , 0 // AffectedOrderRequestID#494
        ///
        // DeleteAllOrderNRResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#495
        , 0 // TemplateID#496
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#497
        , 0 // TrdRegTSTimeIn#498
        , 0 // TrdRegTSTimeOut#499
        , 0 // ResponseIn#500
        , 0 // SendingTime#501
        , 0 // MsgSeqNum#502
        , 0 // LastFragment#503
        ///
        , 0 // MassActionReportID#504
        // DeleteAllOrderQuoteEventBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#505
        , 0 // TemplateID#506
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#507
        , 1 // NotificationIn#508
        , 0 // SendingTime#509
        , 1 // ApplSubID#510
        , 0 // PartitionID#511
        , 1 // ApplMsgID#512
        , 0 // ApplID#513
        , 0 // ApplResendFlag#514
        , 0 // LastFragment#515
        ///
        , 0 // MassActionReportID#516
        , 1 // SecurityID#517
        , 0 // MarketSegmentID#518
        , 0 // MassActionReason#519
        , 1 // ExecInst#520
        // DeleteAllOrderRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#521
        , 0 // TemplateID#522
        , 2 // NetworkMsgID#523
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#524
        , 0 // SenderSubID#525
        ///
        , 1 // SecurityID#526
        , 1 // Price#527
        , 1 // PartyIdInvestmentDecisionMaker#528
        , 1 // ExecutingTrader#529
        , 0 // MarketSegmentID#530
        , 1 // TargetPartyIDSessionID#531
        , 1 // TargetPartyIDExecutingTrader#532
        , 1 // Side#533
        , 1 // OrderOrigination#534
        , 1 // PartyIdInvestmentDecisionMakerQualifier#535
        , 0 // ExecutingTraderQualifier#536
        // DeleteAllOrderResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#537
        , 0 // TemplateID#538
        ///
        //// ResponseHeaderMEComp
        , 0 // RequestTime#539
        , 0 // TrdRegTSTimeIn#540
        , 0 // TrdRegTSTimeOut#541
        , 0 // ResponseIn#542
        , 0 // SendingTime#543
        , 0 // MsgSeqNum#544
        , 0 // PartitionID#545
        , 0 // ApplID#546
        , 0 // ApplMsgID#547
        , 0 // LastFragment#548
        ///
        , 0 // MassActionReportID#549
        , 0 // NoNotAffectedOrders#550
        , 0 // NoAffectedOrderRequests#551
        //// NotAffectedOrdersGrpComp
        , 0 // NotAffectedOrderID#552
        , 1 // NotAffOrigClOrdID#553
        ///
        //// AffectedOrderRequestsGrpComp
        , 0 // AffectedOrderRequestID#554
        ///
        // DeleteAllQuoteBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#555
        , 0 // TemplateID#556
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#557
        , 1 // NotificationIn#558
        , 0 // SendingTime#559
        , 2 // ApplSubID#560
        , 0 // PartitionID#561
        , 1 // ApplMsgID#562
        , 0 // ApplID#563
        , 0 // ApplResendFlag#564
        , 0 // LastFragment#565
        ///
        , 0 // MassActionReportID#566
        , 2 // SecurityID#567
        , 0 // MarketSegmentID#568
        , 0 // TargetPartyIDSessionID#569
        , 1 // PartyIDEnteringTrader#570
        , 1 // TargetPartyIDExecutingTrader#571
        , 0 // NoNotAffectedSecurities#572
        , 0 // MassActionReason#573
        , 1 // PartyIDEnteringFirm#574
        , 1 // TargetPartyIDDeskID#575
        //// NotAffectedSecuritiesGrpComp
        , 0 // NotAffectedSecurityID#576
        ///
        // DeleteAllQuoteRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#577
        , 0 // TemplateID#578
        , 2 // NetworkMsgID#579
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#580
        , 0 // SenderSubID#581
        ///
        , 1 // PartyIdInvestmentDecisionMaker#582
        , 1 // ExecutingTrader#583
        , 0 // MarketSegmentID#584
        , 1 // TargetPartyIDSessionID#585
        , 1 // PartyIdInvestmentDecisionMakerQualifier#586
        , 0 // ExecutingTraderQualifier#587
        // DeleteAllQuoteResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#588
        , 0 // TemplateID#589
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#590
        , 1 // TrdRegTSTimeIn#591
        , 1 // TrdRegTSTimeOut#592
        , 0 // ResponseIn#593
        , 0 // SendingTime#594
        , 0 // MsgSeqNum#595
        , 0 // LastFragment#596
        ///
        , 0 // MassActionReportID#597
        , 0 // NoNotAffectedSecurities#598
        //// NotAffectedSecuritiesGrpComp
        , 0 // NotAffectedSecurityID#599
        ///
        // DeleteBasketTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#600
        , 0 // TemplateID#601
        , 2 // NetworkMsgID#602
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#603
        , 0 // SenderSubID#604
        ///
        , 0 // BasketTrdMatchID#605
        , 0 // BasketExecID#606
        , 0 // MarketSegmentID#607
        , 0 // TrdType#608
        , 0 // TradeReportType#609
        , 1 // TradeReportID#610
        // DeleteCLIPRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#611
        , 0 // TemplateID#612
        , 2 // NetworkMsgID#613
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#614
        , 0 // SenderSubID#615
        ///
        , 0 // OrderID#616
        , 0 // SecurityID#617
        , 1 // PartyIdInvestmentDecisionMaker#618
        , 1 // ExecutingTrader#619
        , 0 // MarketSegmentID#620
        , 1 // CrossRequestID#621
        , 0 // ExecutingTraderQualifier#622
        , 1 // PartyIdInvestmentDecisionMakerQualifier#623
        // DeleteOrderBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#624
        , 0 // TemplateID#625
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#626
        , 1 // NotificationIn#627
        , 0 // SendingTime#628
        , 1 // ApplSubID#629
        , 0 // PartitionID#630
        , 1 // ApplMsgID#631
        , 1 // ApplID#632
        , 0 // ApplResendFlag#633
        , 0 // LastFragment#634
        ///
        , 0 // OrderID#635
        , 1 // ClOrdID#636
        , 1 // OrigClOrdID#637
        , 0 // SecurityID#638
        , 0 // ExecID#639
        , 0 // CumQty#640
        , 0 // CxlQty#641
        , 0 // MarketSegmentID#642
        , 1 // PartyIDEnteringTrader#643
        , 1 // PartyIDSessionID#644
        , 0 // ExecRestatementReason#645
        , 1 // PartyIDEnteringFirm#646
        , 0 // OrdStatus#647
        , 0 // ExecType#648
        , 0 // ProductComplex#649
        , 0 // Side#650
        , 1 // FIXClOrdID#651
        // DeleteOrderComplexRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#652
        , 0 // TemplateID#653
        , 2 // NetworkMsgID#654
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#655
        , 0 // SenderSubID#656
        ///
        , 1 // OrderID#657
        , 1 // ClOrdID#658
        , 1 // OrigClOrdID#659
        , 0 // SecurityID#660
        , 1 // PartyIdInvestmentDecisionMaker#661
        , 1 // ExecutingTrader#662
        , 0 // MarketSegmentID#663
        , 1 // TargetPartyIDSessionID#664
        , 1 // OrderOrigination#665
        , 1 // PartyIdInvestmentDecisionMakerQualifier#666
        , 0 // ExecutingTraderQualifier#667
        , 1 // FIXClOrdID#668
        // DeleteOrderNRResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#669
        , 0 // TemplateID#670
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#671
        , 0 // TrdRegTSTimeIn#672
        , 0 // TrdRegTSTimeOut#673
        , 0 // ResponseIn#674
        , 0 // SendingTime#675
        , 0 // MsgSeqNum#676
        , 0 // LastFragment#677
        ///
        , 0 // OrderID#678
        , 1 // ClOrdID#679
        , 1 // OrigClOrdID#680
        , 0 // SecurityID#681
        , 0 // ExecID#682
        , 0 // CumQty#683
        , 0 // CxlQty#684
        , 0 // OrdStatus#685
        , 0 // ExecType#686
        , 0 // ExecRestatementReason#687
        , 0 // ProductComplex#688
        , 0 // TransactionDelayIndicator#689
        // DeleteOrderResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#690
        , 0 // TemplateID#691
        ///
        //// ResponseHeaderMEComp
        , 0 // RequestTime#692
        , 0 // TrdRegTSTimeIn#693
        , 0 // TrdRegTSTimeOut#694
        , 0 // ResponseIn#695
        , 0 // SendingTime#696
        , 0 // MsgSeqNum#697
        , 0 // PartitionID#698
        , 0 // ApplID#699
        , 1 // ApplMsgID#700
        , 0 // LastFragment#701
        ///
        , 0 // OrderID#702
        , 1 // ClOrdID#703
        , 1 // OrigClOrdID#704
        , 0 // SecurityID#705
        , 0 // ExecID#706
        , 0 // CumQty#707
        , 0 // CxlQty#708
        , 0 // OrdStatus#709
        , 0 // ExecType#710
        , 0 // ExecRestatementReason#711
        , 0 // ProductComplex#712
        , 0 // TransactionDelayIndicator#713
        // DeleteOrderSingleRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#714
        , 0 // TemplateID#715
        , 2 // NetworkMsgID#716
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#717
        , 0 // SenderSubID#718
        ///
        , 1 // OrderID#719
        , 1 // ClOrdID#720
        , 1 // OrigClOrdID#721
        , 1 // PartyIdInvestmentDecisionMaker#722
        , 1 // ExecutingTrader#723
        , 0 // MarketSegmentID#724
        , 0 // SimpleSecurityID#725
        , 1 // TargetPartyIDSessionID#726
        , 1 // OrderOrigination#727
        , 1 // PartyIdInvestmentDecisionMakerQualifier#728
        , 1 // ExecutingTraderQualifier#729
        , 1 // FIXClOrdID#730
        // DeleteTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#731
        , 0 // TemplateID#732
        , 2 // NetworkMsgID#733
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#734
        , 0 // SenderSubID#735
        ///
        , 1 // CompressionID#736
        , 0 // PackageID#737
        , 0 // MarketSegmentID#738
        , 0 // TESExecID#739
        , 1 // RelatedMarketSegmentID#740
        , 0 // TrdType#741
        , 0 // TradeReportType#742
        , 1 // TradeReportID#743
        // EnterBasketTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#744
        , 0 // TemplateID#745
        , 2 // NetworkMsgID#746
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#747
        , 0 // SenderSubID#748
        ///
        , 0 // MarketSegmentID#749
        , 0 // MaturityMonthYear#750
        , 1 // BasketProfileID#751
        , 0 // TrdType#752
        , 0 // NoBasketSideAlloc#753
        , 0 // TradeReportType#754
        , 0 // BasketTradeReportType#755
        , 0 // NoBasketRootPartyGrps#756
        , 0 // NoInstrmtMatchSides#757
        , 1 // BasketTradeReportText#758
        , 1 // TradeReportID#759
        //// BasketRootPartyGrpComp
        , 0 // RootPartySubIDType#760
        , 0 // RootPartyContraFirm#761
        , 0 // RootPartyContraTrader#762
        , 1 // BasketSideTradeReportID#763
        ///
        //// InstrmtMatchSideGrpComp
        , 0 // SecurityID#764
        , 0 // LastPx#765
        , 1 // TransBkdTime#766
        , 1 // RelatedClosePrice#767
        , 2 // PackageID#768
        , 0 // SideMarketSegmentID#769
        , 0 // SideTrdSubTyp#770
        , 0 // ProductComplex#771
        , 0 // TradePublishIndicator#772
        , 0 // InstrmtMatchSideID#773
        , 1 // EffectOnBasket#774
        , 1 // TradeReportText#775
        ///
        //// BasketSideAllocGrpComp
        , 0 // AllocQty#776
        , 2 // IndividualAllocID#777
        , 0 // PartySubIDType#778
        , 0 // Side#779
        , 0 // InstrmtMatchSideID#780
        , 2 // TradeAllocStatus#781
        , 0 // PartyExecutingFirm#782
        , 0 // PartyExecutingTrader#783
        ///
        // EnterCLIPRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#784
        , 0 // TemplateID#785
        , 2 // NetworkMsgID#786
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#787
        , 0 // SenderSubID#788
        ///
        , 0 // SecurityID#789
        , 0 // Price#790
        , 0 // OrderQty#791
        , 0 // MarketSegmentID#792
        , 1 // CrossID#793
        , 1 // CrossRequestID#794
        , 0 // NoSides#795
        , 0 // NoCrossLegs#796
        , 1 // SideDisclosureInstruction#797
        , 1 // PriceDisclosureInstruction#798
        , 1 // OrderQtyDisclosureInstruction#799
        , 1 // RootPartyContraFirm#800
        , 1 // RootPartyContraTrader#801
        //// CrossRequestSideGrpComp
        , 1 // PartyIDClientID#802
        , 1 // PartyIdInvestmentDecisionMaker#803
        , 1 // ExecutingTrader#804
        , 1 // MaximumPrice#805
        , 0 // InputSource#806
        , 0 // Side#807
        , 0 // TradingCapacity#808
        , 0 // ExecutingTraderQualifier#809
        , 0 // OrderAttributeLiquidityProvision#810
        , 1 // PartyIdInvestmentDecisionMakerQualifier#811
        , 1 // PositionEffect#812
        , 1 // CustOrderHandlingInst#813
        , 1 // Account#814
        , 1 // PartyIDPositionAccount#815
        , 1 // FreeText1#816
        , 1 // FreeText2#817
        , 1 // FreeText3#818
        , 1 // PartyIDOrderOriginationFirm#819
        , 1 // PartyIDBeneficiary#820
        , 1 // PartyIDTakeUpTradingFirm#821
        , 1 // SideComplianceText#822
        , 1 // PartyIDLocationID#823
        ///
        //// SideCrossLegGrpComp
        , 0 // LegInputSource#824
        , 0 // LegPositionEffect#825
        , 1 // LegAccount#826
        ///
        // EnterTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#827
        , 0 // TemplateID#828
        , 2 // NetworkMsgID#829
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#830
        , 0 // SenderSubID#831
        ///
        , 0 // SecurityID#832
        , 0 // LastPx#833
        , 1 // TransBkdTime#834
        , 1 // UnderlyingPx#835
        , 1 // RelatedClosePrice#836
        , 1 // RelatedTradeQuantity#837
        , 1 // UnderlyingQty#838
        , 1 // CompressionID#839
        , 0 // MarketSegmentID#840
        , 1 // UnderlyingSettlementDate#841
        , 1 // UnderlyingMaturityDate#842
        , 1 // RelatedTradeID#843
        , 1 // RelatedMarketSegmentID#844
        , 0 // TrdType#845
        , 1 // ProductComplex#846
        , 0 // TradeReportType#847
        , 0 // TradePublishIndicator#848
        , 0 // NoSideAllocs#849
        , 0 // NoEvents#850
        , 0 // NoLegs#851
        , 0 // NoInstrAttrib#852
        , 0 // NoUnderlyingStips#853
        , 1 // PartyIDSettlementLocation#854
        , 1 // HedgeType#855
        , 1 // SwapClearer#856
        , 1 // TradeReportText#857
        , 1 // TradeReportID#858
        , 1 // UnderlyingSecurityID#859
        , 1 // UnderlyingSecurityDesc#860
        , 1 // UnderlyingCurrency#861
        , 1 // UnderlyingIssuer#862
        //// SideAllocGrpComp
        , 0 // AllocQty#863
        , 2 // IndividualAllocID#864
        , 1 // TESEnrichmentRuleID#865
        , 0 // Side#866
        , 0 // PartyExecutingFirm#867
        , 0 // PartyExecutingTrader#868
        ///
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#869
        , 0 // LegPrice#870
        , 2 // LegQty#871
        ///
        //// InstrumentEventGrpComp
        , 0 // EventDate#872
        , 0 // EventType#873
        ///
        //// InstrumentAttributeGrpComp
        , 0 // InstrAttribType#874
        , 0 // InstrAttribValue#875
        ///
        //// UnderlyingStipGrpComp
        , 0 // UnderlyingStipValue#876
        , 0 // UnderlyingStipType#877
        ///
        // ForcedLogoutNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#878
        , 0 // TemplateID#879
        ///
        //// NotifHeaderComp
        , 0 // SendingTime#880
        ///
        , 0 // VarTextLen#881
        , 0 // VarText#882
        // ForcedUserLogoutNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#883
        , 0 // TemplateID#884
        ///
        //// NotifHeaderComp
        , 0 // SendingTime#885
        ///
        , 0 // Username#886
        , 0 // VarTextLen#887
        , 0 // UserStatus#888
        , 0 // VarText#889
        // Heartbeat
        //// MessageHeaderInComp
        , 0 // BodyLen#890
        , 0 // TemplateID#891
        , 2 // NetworkMsgID#892
        ///
        // HeartbeatNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#893
        , 0 // TemplateID#894
        ///
        //// NotifHeaderComp
        , 0 // SendingTime#895
        ///
        // InquireEnrichmentRuleIDListRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#896
        , 0 // TemplateID#897
        , 2 // NetworkMsgID#898
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#899
        , 2 // SenderSubID#900
        ///
        , 1 // LastEntityProcessed#901
        // InquireEnrichmentRuleIDListResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#902
        , 0 // TemplateID#903
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#904
        , 0 // SendingTime#905
        , 0 // MsgSeqNum#906
        ///
        , 1 // LastEntityProcessed#907
        , 0 // NoEnrichmentRules#908
        //// EnrichmentRulesGrpComp
        , 0 // EnrichmentRuleID#909
        , 1 // PartyIDOriginationMarket#910
        , 1 // Account#911
        , 1 // PositionEffect#912
        , 1 // PartyIDTakeUpTradingFirm#913
        , 1 // PartyIDOrderOriginationFirm#914
        , 1 // PartyIDBeneficiary#915
        , 1 // FreeText1#916
        , 1 // FreeText2#917
        , 1 // FreeText3#918
        ///
        // InquireMMParameterRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#919
        , 0 // TemplateID#920
        , 2 // NetworkMsgID#921
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#922
        , 0 // SenderSubID#923
        ///
        , 0 // MarketSegmentID#924
        , 1 // TargetPartyIDSessionID#925
        // InquireMMParameterResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#926
        , 0 // TemplateID#927
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#928
        , 0 // TrdRegTSTimeIn#929
        , 0 // TrdRegTSTimeOut#930
        , 0 // ResponseIn#931
        , 0 // SendingTime#932
        , 0 // MsgSeqNum#933
        , 0 // LastFragment#934
        ///
        , 0 // MMParameterReportID#935
        , 0 // MarketSegmentID#936
        , 0 // NoMMParameters#937
        //// MMParameterGrpComp
        , 0 // ExposureDuration#938
        , 0 // CumQty#939
        , 0 // Delta#940
        , 0 // Vega#941
        , 0 // PctCount#942
        , 1 // TargetPartyIDSessionID#943
        ///
        // InquirePreTradeRiskLimitsRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#944
        , 0 // TemplateID#945
        , 2 // NetworkMsgID#946
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#947
        , 0 // SenderSubID#948
        ///
        , 0 // MarketSegmentID#949
        , 0 // RiskLimitPlatform#950
        , 1 // PartyExecutingUnit#951
        , 1 // RiskLimitGroup#952
        // InquireSessionListRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#953
        , 0 // TemplateID#954
        , 2 // NetworkMsgID#955
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#956
        , 2 // SenderSubID#957
        ///
        // InquireSessionListResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#958
        , 0 // TemplateID#959
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#960
        , 0 // SendingTime#961
        , 0 // MsgSeqNum#962
        ///
        , 0 // NoSessions#963
        //// SessionsGrpComp
        , 0 // PartyIDSessionID#964
        , 0 // SessionMode#965
        , 1 // SessionSubMode#966
        ///
        // InquireUserRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#967
        , 0 // TemplateID#968
        , 2 // NetworkMsgID#969
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#970
        , 2 // SenderSubID#971
        ///
        , 1 // LastEntityProcessed#972
        // InquireUserResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#973
        , 0 // TemplateID#974
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#975
        , 0 // SendingTime#976
        , 0 // MsgSeqNum#977
        ///
        , 1 // LastEntityProcessed#978
        , 0 // NoPartyDetails#979
        //// PartyDetailsGrpComp
        , 0 // PartyDetailIDExecutingTrader#980
        , 0 // PartyDetailExecutingTrader#981
        , 0 // PartyDetailRoleQualifier#982
        , 0 // PartyDetailStatus#983
        , 1 // PartyDetailDeskID#984
        ///
        // LegalNotificationBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#985
        , 0 // TemplateID#986
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#987
        , 0 // ApplSeqNum#988
        , 2 // ApplSubID#989
        , 0 // PartitionID#990
        , 0 // ApplResendFlag#991
        , 0 // ApplID#992
        , 0 // LastFragment#993
        ///
        , 0 // TransactTime#994
        , 0 // VarTextLen#995
        , 0 // UserStatus#996
        , 0 // VarText#997
        // LogonRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#998
        , 0 // TemplateID#999
        , 2 // NetworkMsgID#1000
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1001
        , 2 // SenderSubID#1002
        ///
        , 1 // HeartBtInt#1003
        , 0 // PartyIDSessionID#1004
        , 0 // DefaultCstmApplVerID#1005
        , 0 // Password#1006
        , 0 // ApplUsageOrders#1007
        , 0 // ApplUsageQuotes#1008
        , 0 // OrderRoutingIndicator#1009
        , 1 // FIXEngineName#1010
        , 1 // FIXEngineVersion#1011
        , 1 // FIXEngineVendor#1012
        , 0 // ApplicationSystemName#1013
        , 0 // ApplicationSystemVersion#1014
        , 0 // ApplicationSystemVendor#1015
        // LogonResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1016
        , 0 // TemplateID#1017
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#1018
        , 0 // SendingTime#1019
        , 0 // MsgSeqNum#1020
        ///
        , 0 // ThrottleTimeInterval#1021
        , 0 // ThrottleNoMsgs#1022
        , 0 // ThrottleDisconnectLimit#1023
        , 0 // HeartBtInt#1024
        , 0 // SessionInstanceID#1025
        , 0 // MarketID#1026
        , 0 // TradSesMode#1027
        , 0 // DefaultCstmApplVerID#1028
        , 0 // DefaultCstmApplVerSubID#1029
        // LogoutRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1030
        , 0 // TemplateID#1031
        , 2 // NetworkMsgID#1032
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1033
        , 2 // SenderSubID#1034
        ///
        // LogoutResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1035
        , 0 // TemplateID#1036
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#1037
        , 0 // SendingTime#1038
        , 0 // MsgSeqNum#1039
        ///
        // MMParameterDefinitionRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1040
        , 0 // TemplateID#1041
        , 2 // NetworkMsgID#1042
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1043
        , 0 // SenderSubID#1044
        ///
        , 0 // ExposureDuration#1045
        , 0 // CumQty#1046
        , 0 // Delta#1047
        , 0 // Vega#1048
        , 0 // MarketSegmentID#1049
        , 1 // TargetPartyIDSessionID#1050
        , 0 // PctCount#1051
        // MMParameterDefinitionResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1052
        , 0 // TemplateID#1053
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1054
        , 0 // TrdRegTSTimeIn#1055
        , 0 // TrdRegTSTimeOut#1056
        , 0 // ResponseIn#1057
        , 0 // SendingTime#1058
        , 0 // MsgSeqNum#1059
        , 0 // LastFragment#1060
        ///
        , 1 // ExecID#1061
        // MassQuoteRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1062
        , 0 // TemplateID#1063
        , 2 // NetworkMsgID#1064
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1065
        , 0 // SenderSubID#1066
        ///
        , 0 // QuoteID#1067
        , 1 // PartyIdInvestmentDecisionMaker#1068
        , 1 // ExecutingTrader#1069
        , 0 // MarketSegmentID#1070
        , 1 // MatchInstCrossID#1071
        , 1 // EnrichmentRuleID#1072
        , 0 // PriceValidityCheckType#1073
        , 0 // ValueCheckTypeValue#1074
        , 0 // QuoteSizeType#1075
        , 0 // QuoteType#1076
        , 0 // OrderAttributeLiquidityProvision#1077
        , 0 // NoQuoteEntries#1078
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1079
        , 0 // ExecutingTraderQualifier#1080
        //// QuoteEntryGrpComp
        , 0 // SecurityID#1081
        , 1 // BidPx#1082
        , 1 // BidSize#1083
        , 1 // OfferPx#1084
        , 1 // OfferSize#1085
        ///
        // MassQuoteResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1086
        , 0 // TemplateID#1087
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1088
        , 0 // TrdRegTSTimeIn#1089
        , 0 // TrdRegTSTimeOut#1090
        , 0 // ResponseIn#1091
        , 0 // SendingTime#1092
        , 0 // MsgSeqNum#1093
        , 0 // LastFragment#1094
        ///
        , 0 // QuoteID#1095
        , 0 // QuoteResponseID#1096
        , 0 // MarketSegmentID#1097
        , 0 // NoQuoteSideEntries#1098
        //// QuoteEntryAckGrpComp
        , 0 // SecurityID#1099
        , 1 // CxlSize#1100
        , 1 // QuoteEntryRejectReason#1101
        , 0 // QuoteEntryStatus#1102
        , 0 // Side#1103
        ///
        // ModifyBasketTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1104
        , 0 // TemplateID#1105
        , 2 // NetworkMsgID#1106
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1107
        , 0 // SenderSubID#1108
        ///
        , 0 // BasketTrdMatchID#1109
        , 0 // BasketExecID#1110
        , 0 // MarketSegmentID#1111
        , 0 // MaturityMonthYear#1112
        , 1 // BasketProfileID#1113
        , 0 // TrdType#1114
        , 0 // NoBasketSideAlloc#1115
        , 0 // TradeReportType#1116
        , 0 // NoBasketRootPartyGrps#1117
        , 0 // NoInstrmtMatchSides#1118
        , 1 // BasketTradeReportText#1119
        , 1 // TradeReportID#1120
        //// BasketRootPartyGrpComp
        , 0 // RootPartySubIDType#1121
        , 0 // RootPartyContraFirm#1122
        , 0 // RootPartyContraTrader#1123
        , 1 // BasketSideTradeReportID#1124
        ///
        //// InstrmtMatchSideGrpComp
        , 0 // SecurityID#1125
        , 0 // LastPx#1126
        , 1 // TransBkdTime#1127
        , 1 // RelatedClosePrice#1128
        , 1 // PackageID#1129
        , 0 // SideMarketSegmentID#1130
        , 0 // SideTrdSubTyp#1131
        , 0 // ProductComplex#1132
        , 0 // TradePublishIndicator#1133
        , 0 // InstrmtMatchSideID#1134
        , 1 // EffectOnBasket#1135
        , 1 // TradeReportText#1136
        ///
        //// BasketSideAllocGrpComp
        , 0 // AllocQty#1137
        , 1 // IndividualAllocID#1138
        , 0 // PartySubIDType#1139
        , 0 // Side#1140
        , 0 // InstrmtMatchSideID#1141
        , 2 // TradeAllocStatus#1142
        , 0 // PartyExecutingFirm#1143
        , 0 // PartyExecutingTrader#1144
        ///
        // ModifyOrderComplexRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1145
        , 0 // TemplateID#1146
        , 2 // NetworkMsgID#1147
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1148
        , 0 // SenderSubID#1149
        ///
        , 1 // OrderID#1150
        , 1 // ClOrdID#1151
        , 1 // OrigClOrdID#1152
        , 0 // SecurityID#1153
        , 0 // Price#1154
        , 0 // OrderQty#1155
        , 1 // PartyIDClientID#1156
        , 1 // PartyIdInvestmentDecisionMaker#1157
        , 1 // ExecutingTrader#1158
        , 0 // MarketSegmentID#1159
        , 1 // ExpireDate#1160
        , 1 // MatchInstCrossID#1161
        , 1 // TargetPartyIDSessionID#1162
        , 1 // PartyIDTakeUpTradingFirm#1163
        , 1 // PartyIDOrderOriginationFirm#1164
        , 1 // PartyIDBeneficiary#1165
        , 0 // ApplSeqIndicator#1166
        , 0 // ProductComplex#1167
        , 0 // Side#1168
        , 0 // OrdType#1169
        , 0 // PriceValidityCheckType#1170
        , 0 // ValueCheckTypeValue#1171
        , 0 // OrderAttributeLiquidityProvision#1172
        , 0 // ExecInst#1173
        , 0 // TimeInForce#1174
        , 0 // TradingCapacity#1175
        , 0 // OwnershipIndicator#1176
        , 1 // OrderOrigination#1177
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1178
        , 0 // ExecutingTraderQualifier#1179
        , 1 // PartyIDLocationID#1180
        , 1 // CustOrderHandlingInst#1181
        , 1 // ComplianceText#1182
        , 1 // PartyIDPositionAccount#1183
        , 1 // FreeText1#1184
        , 1 // FreeText2#1185
        , 1 // FreeText3#1186
        , 1 // FIXClOrdID#1187
        , 0 // NoLegs#1188
        //// LegOrdGrpComp
        , 1 // LegAccount#1189
        , 0 // LegPositionEffect#1190
        ///
        // ModifyOrderComplexShortRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1191
        , 0 // TemplateID#1192
        , 2 // NetworkMsgID#1193
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1194
        , 0 // SenderSubID#1195
        ///
        , 1 // ClOrdID#1196
        , 0 // OrigClOrdID#1197
        , 0 // SecurityID#1198
        , 0 // Price#1199
        , 0 // OrderQty#1200
        , 1 // PartyIDClientID#1201
        , 1 // PartyIdInvestmentDecisionMaker#1202
        , 1 // ExecutingTrader#1203
        , 0 // MarketSegmentID#1204
        , 1 // MatchInstCrossID#1205
        , 1 // EnrichmentRuleID#1206
        , 0 // ApplSeqIndicator#1207
        , 0 // ProductComplex#1208
        , 0 // Side#1209
        , 0 // PriceValidityCheckType#1210
        , 0 // ValueCheckTypeValue#1211
        , 0 // OrderAttributeLiquidityProvision#1212
        , 0 // ExecInst#1213
        , 0 // TimeInForce#1214
        , 0 // TradingCapacity#1215
        , 1 // OrderOrigination#1216
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1217
        , 0 // ExecutingTraderQualifier#1218
        // ModifyOrderNRResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1219
        , 0 // TemplateID#1220
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1221
        , 0 // TrdRegTSTimeIn#1222
        , 0 // TrdRegTSTimeOut#1223
        , 0 // ResponseIn#1224
        , 0 // SendingTime#1225
        , 0 // MsgSeqNum#1226
        , 0 // LastFragment#1227
        ///
        , 0 // OrderID#1228
        , 1 // ClOrdID#1229
        , 1 // OrigClOrdID#1230
        , 0 // SecurityID#1231
        , 0 // ExecID#1232
        , 0 // LeavesQty#1233
        , 0 // CumQty#1234
        , 0 // CxlQty#1235
        , 0 // OrdStatus#1236
        , 0 // ExecType#1237
        , 0 // ExecRestatementReason#1238
        , 0 // CrossedIndicator#1239
        , 0 // ProductComplex#1240
        , 0 // Triggered#1241
        , 0 // TransactionDelayIndicator#1242
        // ModifyOrderResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1243
        , 0 // TemplateID#1244
        ///
        //// ResponseHeaderMEComp
        , 0 // RequestTime#1245
        , 0 // TrdRegTSTimeIn#1246
        , 0 // TrdRegTSTimeOut#1247
        , 0 // ResponseIn#1248
        , 0 // SendingTime#1249
        , 0 // MsgSeqNum#1250
        , 0 // PartitionID#1251
        , 0 // ApplID#1252
        , 1 // ApplMsgID#1253
        , 0 // LastFragment#1254
        ///
        , 0 // OrderID#1255
        , 1 // ClOrdID#1256
        , 1 // OrigClOrdID#1257
        , 0 // SecurityID#1258
        , 0 // ExecID#1259
        , 0 // LeavesQty#1260
        , 0 // CumQty#1261
        , 0 // CxlQty#1262
        , 0 // TrdRegTSTimePriority#1263
        , 0 // OrdStatus#1264
        , 0 // ExecType#1265
        , 0 // ExecRestatementReason#1266
        , 0 // CrossedIndicator#1267
        , 0 // ProductComplex#1268
        , 0 // Triggered#1269
        , 0 // TransactionDelayIndicator#1270
        // ModifyOrderSingleRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1271
        , 0 // TemplateID#1272
        , 2 // NetworkMsgID#1273
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1274
        , 0 // SenderSubID#1275
        ///
        , 1 // OrderID#1276
        , 1 // ClOrdID#1277
        , 1 // OrigClOrdID#1278
        , 1 // Price#1279
        , 0 // OrderQty#1280
        , 1 // StopPx#1281
        , 1 // PartyIDClientID#1282
        , 1 // PartyIdInvestmentDecisionMaker#1283
        , 1 // ExecutingTrader#1284
        , 1 // ExpireDate#1285
        , 0 // MarketSegmentID#1286
        , 0 // SimpleSecurityID#1287
        , 1 // MatchInstCrossID#1288
        , 1 // TargetPartyIDSessionID#1289
        , 1 // PartyIDTakeUpTradingFirm#1290
        , 1 // PartyIDOrderOriginationFirm#1291
        , 1 // PartyIDBeneficiary#1292
        , 0 // ApplSeqIndicator#1293
        , 0 // Side#1294
        , 0 // OrdType#1295
        , 0 // PriceValidityCheckType#1296
        , 0 // ValueCheckTypeValue#1297
        , 0 // OrderAttributeLiquidityProvision#1298
        , 0 // TimeInForce#1299
        , 0 // ExecInst#1300
        , 1 // TradingSessionSubID#1301
        , 0 // TradingCapacity#1302
        , 1 // OrderOrigination#1303
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1304
        , 1 // ExecutingTraderQualifier#1305
        , 1 // Account#1306
        , 1 // PartyIDPositionAccount#1307
        , 0 // PositionEffect#1308
        , 0 // OwnershipIndicator#1309
        , 1 // PartyIDLocationID#1310
        , 1 // CustOrderHandlingInst#1311
        , 1 // ComplianceText#1312
        , 1 // FreeText1#1313
        , 1 // FreeText2#1314
        , 1 // FreeText3#1315
        , 1 // FIXClOrdID#1316
        // ModifyOrderSingleShortRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1317
        , 0 // TemplateID#1318
        , 2 // NetworkMsgID#1319
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1320
        , 0 // SenderSubID#1321
        ///
        , 1 // ClOrdID#1322
        , 0 // OrigClOrdID#1323
        , 0 // Price#1324
        , 0 // OrderQty#1325
        , 1 // PartyIDClientID#1326
        , 1 // PartyIdInvestmentDecisionMaker#1327
        , 1 // ExecutingTrader#1328
        , 0 // SimpleSecurityID#1329
        , 1 // MatchInstCrossID#1330
        , 1 // EnrichmentRuleID#1331
        , 0 // Side#1332
        , 0 // PriceValidityCheckType#1333
        , 0 // ValueCheckTypeValue#1334
        , 0 // OrderAttributeLiquidityProvision#1335
        , 0 // TimeInForce#1336
        , 0 // ApplSeqIndicator#1337
        , 0 // ExecInst#1338
        , 0 // TradingCapacity#1339
        , 1 // OrderOrigination#1340
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1341
        , 0 // ExecutingTraderQualifier#1342
        // ModifyTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1343
        , 0 // TemplateID#1344
        , 2 // NetworkMsgID#1345
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1346
        , 0 // SenderSubID#1347
        ///
        , 0 // LastPx#1348
        , 1 // TransBkdTime#1349
        , 1 // CompressionID#1350
        , 0 // MarketSegmentID#1351
        , 0 // PackageID#1352
        , 0 // TESExecID#1353
        , 1 // RelatedMarketSegmentID#1354
        , 0 // TrdType#1355
        , 0 // TradeReportType#1356
        , 0 // TradePublishIndicator#1357
        , 0 // NoSideAllocs#1358
        , 0 // NoLegs#1359
        , 1 // SwapClearer#1360
        , 1 // TradeReportText#1361
        , 1 // TradeReportID#1362
        //// SideAllocGrpComp
        , 0 // AllocQty#1363
        , 1 // IndividualAllocID#1364
        , 1 // TESEnrichmentRuleID#1365
        , 0 // Side#1366
        , 0 // PartyExecutingFirm#1367
        , 0 // PartyExecutingTrader#1368
        ///
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#1369
        , 0 // LegPrice#1370
        , 2 // LegQty#1371
        ///
        // NewOrderComplexRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1372
        , 0 // TemplateID#1373
        , 2 // NetworkMsgID#1374
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1375
        , 0 // SenderSubID#1376
        ///
        , 1 // ClOrdID#1377
        , 0 // SecurityID#1378
        , 0 // Price#1379
        , 0 // OrderQty#1380
        , 1 // PartyIDClientID#1381
        , 1 // PartyIdInvestmentDecisionMaker#1382
        , 1 // ExecutingTrader#1383
        , 0 // MarketSegmentID#1384
        , 1 // ExpireDate#1385
        , 1 // MatchInstCrossID#1386
        , 1 // PartyIDTakeUpTradingFirm#1387
        , 1 // PartyIDOrderOriginationFirm#1388
        , 1 // PartyIDBeneficiary#1389
        , 0 // ApplSeqIndicator#1390
        , 0 // ProductComplex#1391
        , 0 // Side#1392
        , 0 // OrdType#1393
        , 0 // PriceValidityCheckType#1394
        , 0 // ValueCheckTypeValue#1395
        , 0 // OrderAttributeLiquidityProvision#1396
        , 1 // OrderAttributeRiskReduction#1397
        , 0 // ExecInst#1398
        , 0 // TimeInForce#1399
        , 0 // TradingCapacity#1400
        , 1 // OrderOrigination#1401
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1402
        , 0 // ExecutingTraderQualifier#1403
        , 1 // PartyIDLocationID#1404
        , 1 // ComplianceText#1405
        , 1 // CustOrderHandlingInst#1406
        , 1 // PartyIDPositionAccount#1407
        , 1 // FreeText1#1408
        , 1 // FreeText2#1409
        , 1 // FreeText3#1410
        , 1 // FIXClOrdID#1411
        , 0 // NoLegs#1412
        //// LegOrdGrpComp
        , 1 // LegAccount#1413
        , 0 // LegPositionEffect#1414
        ///
        // NewOrderComplexShortRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1415
        , 0 // TemplateID#1416
        , 2 // NetworkMsgID#1417
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1418
        , 0 // SenderSubID#1419
        ///
        , 0 // SecurityID#1420
        , 0 // Price#1421
        , 0 // OrderQty#1422
        , 0 // ClOrdID#1423
        , 1 // PartyIDClientID#1424
        , 1 // PartyIdInvestmentDecisionMaker#1425
        , 1 // ExecutingTrader#1426
        , 0 // MarketSegmentID#1427
        , 1 // MatchInstCrossID#1428
        , 1 // EnrichmentRuleID#1429
        , 0 // ApplSeqIndicator#1430
        , 0 // ProductComplex#1431
        , 0 // Side#1432
        , 0 // PriceValidityCheckType#1433
        , 0 // ValueCheckTypeValue#1434
        , 0 // OrderAttributeLiquidityProvision#1435
        , 0 // ExecInst#1436
        , 0 // TimeInForce#1437
        , 0 // TradingCapacity#1438
        , 1 // OrderOrigination#1439
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1440
        , 0 // ExecutingTraderQualifier#1441
        // NewOrderNRResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1442
        , 0 // TemplateID#1443
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1444
        , 0 // TrdRegTSTimeIn#1445
        , 0 // TrdRegTSTimeOut#1446
        , 0 // ResponseIn#1447
        , 0 // SendingTime#1448
        , 0 // MsgSeqNum#1449
        , 0 // LastFragment#1450
        ///
        , 0 // OrderID#1451
        , 1 // ClOrdID#1452
        , 0 // SecurityID#1453
        , 0 // ExecID#1454
        , 0 // OrdStatus#1455
        , 0 // ExecType#1456
        , 0 // ExecRestatementReason#1457
        , 0 // CrossedIndicator#1458
        , 0 // ProductComplex#1459
        , 0 // Triggered#1460
        , 0 // TransactionDelayIndicator#1461
        // NewOrderResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1462
        , 0 // TemplateID#1463
        ///
        //// ResponseHeaderMEComp
        , 0 // RequestTime#1464
        , 0 // TrdRegTSTimeIn#1465
        , 0 // TrdRegTSTimeOut#1466
        , 0 // ResponseIn#1467
        , 0 // SendingTime#1468
        , 0 // MsgSeqNum#1469
        , 0 // PartitionID#1470
        , 0 // ApplID#1471
        , 1 // ApplMsgID#1472
        , 0 // LastFragment#1473
        ///
        , 0 // OrderID#1474
        , 1 // ClOrdID#1475
        , 0 // SecurityID#1476
        , 0 // ExecID#1477
        , 0 // TrdRegTSEntryTime#1478
        , 0 // TrdRegTSTimePriority#1479
        , 0 // OrdStatus#1480
        , 0 // ExecType#1481
        , 0 // ExecRestatementReason#1482
        , 0 // CrossedIndicator#1483
        , 0 // ProductComplex#1484
        , 0 // Triggered#1485
        , 0 // TransactionDelayIndicator#1486
        // NewOrderSingleRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1487
        , 0 // TemplateID#1488
        , 2 // NetworkMsgID#1489
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1490
        , 0 // SenderSubID#1491
        ///
        , 1 // Price#1492
        , 0 // OrderQty#1493
        , 1 // StopPx#1494
        , 1 // ClOrdID#1495
        , 1 // PartyIDClientID#1496
        , 1 // PartyIdInvestmentDecisionMaker#1497
        , 1 // ExecutingTrader#1498
        , 1 // ExpireDate#1499
        , 0 // MarketSegmentID#1500
        , 0 // SimpleSecurityID#1501
        , 1 // MatchInstCrossID#1502
        , 1 // PartyIDTakeUpTradingFirm#1503
        , 1 // PartyIDOrderOriginationFirm#1504
        , 1 // PartyIDBeneficiary#1505
        , 0 // ApplSeqIndicator#1506
        , 0 // Side#1507
        , 0 // OrdType#1508
        , 0 // PriceValidityCheckType#1509
        , 0 // ValueCheckTypeValue#1510
        , 0 // OrderAttributeLiquidityProvision#1511
        , 1 // OrderAttributeRiskReduction#1512
        , 0 // TimeInForce#1513
        , 0 // ExecInst#1514
        , 1 // TradingSessionSubID#1515
        , 0 // TradingCapacity#1516
        , 1 // OrderOrigination#1517
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1518
        , 0 // ExecutingTraderQualifier#1519
        , 1 // Account#1520
        , 1 // PartyIDPositionAccount#1521
        , 0 // PositionEffect#1522
        , 1 // PartyIDLocationID#1523
        , 1 // CustOrderHandlingInst#1524
        , 1 // ComplianceText#1525
        , 1 // FreeText1#1526
        , 1 // FreeText2#1527
        , 1 // FreeText3#1528
        , 1 // FIXClOrdID#1529
        // NewOrderSingleShortRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1530
        , 0 // TemplateID#1531
        , 2 // NetworkMsgID#1532
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1533
        , 0 // SenderSubID#1534
        ///
        , 0 // Price#1535
        , 0 // OrderQty#1536
        , 0 // ClOrdID#1537
        , 1 // PartyIDClientID#1538
        , 1 // PartyIdInvestmentDecisionMaker#1539
        , 1 // ExecutingTrader#1540
        , 0 // SimpleSecurityID#1541
        , 1 // MatchInstCrossID#1542
        , 1 // EnrichmentRuleID#1543
        , 0 // Side#1544
        , 0 // ApplSeqIndicator#1545
        , 0 // PriceValidityCheckType#1546
        , 0 // ValueCheckTypeValue#1547
        , 0 // OrderAttributeLiquidityProvision#1548
        , 0 // TimeInForce#1549
        , 0 // ExecInst#1550
        , 0 // TradingCapacity#1551
        , 1 // OrderOrigination#1552
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1553
        , 0 // ExecutingTraderQualifier#1554
        // NewsBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#1555
        , 0 // TemplateID#1556
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#1557
        , 1 // ApplSeqNum#1558
        , 1 // ApplSubID#1559
        , 0 // PartitionID#1560
        , 0 // ApplResendFlag#1561
        , 0 // ApplID#1562
        , 0 // LastFragment#1563
        ///
        , 0 // OrigTime#1564
        , 0 // VarTextLen#1565
        , 0 // Headline#1566
        , 1 // VarText#1567
        // OrderExecNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#1568
        , 0 // TemplateID#1569
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#1570
        , 1 // NotificationIn#1571
        , 0 // SendingTime#1572
        , 2 // ApplSubID#1573
        , 0 // PartitionID#1574
        , 0 // ApplMsgID#1575
        , 0 // ApplID#1576
        , 0 // ApplResendFlag#1577
        , 0 // LastFragment#1578
        ///
        , 0 // OrderID#1579
        , 1 // ClOrdID#1580
        , 1 // OrigClOrdID#1581
        , 0 // SecurityID#1582
        , 0 // ExecID#1583
        , 0 // LeavesQty#1584
        , 0 // CumQty#1585
        , 0 // CxlQty#1586
        , 0 // MarketSegmentID#1587
        , 0 // NoLegExecs#1588
        , 0 // ExecRestatementReason#1589
        , 0 // Side#1590
        , 0 // ProductComplex#1591
        , 0 // OrdStatus#1592
        , 0 // ExecType#1593
        , 0 // Triggered#1594
        , 0 // CrossedIndicator#1595
        , 1 // FIXClOrdID#1596
        , 0 // NoFills#1597
        //// FillsGrpComp
        , 0 // FillPx#1598
        , 0 // FillQty#1599
        , 0 // FillMatchID#1600
        , 0 // FillExecID#1601
        , 1 // FillLiquidityInd#1602
        ///
        //// InstrmntLegExecGrpComp
        , 0 // LegSecurityID#1603
        , 0 // LegLastPx#1604
        , 0 // LegLastQty#1605
        , 0 // LegExecID#1606
        , 0 // LegSide#1607
        , 0 // FillRefID#1608
        ///
        // OrderExecReportBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#1609
        , 0 // TemplateID#1610
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#1611
        , 1 // NotificationIn#1612
        , 0 // SendingTime#1613
        , 1 // ApplSubID#1614
        , 0 // PartitionID#1615
        , 1 // ApplMsgID#1616
        , 0 // ApplID#1617
        , 0 // ApplResendFlag#1618
        , 0 // LastFragment#1619
        ///
        , 0 // OrderID#1620
        , 1 // ClOrdID#1621
        , 1 // OrigClOrdID#1622
        , 0 // SecurityID#1623
        , 0 // ExecID#1624
        , 0 // TrdRegTSEntryTime#1625
        , 0 // TrdRegTSTimePriority#1626
        , 1 // Price#1627
        , 0 // LeavesQty#1628
        , 0 // CumQty#1629
        , 0 // CxlQty#1630
        , 0 // OrderQty#1631
        , 1 // StopPx#1632
        , 0 // MarketSegmentID#1633
        , 1 // ExpireDate#1634
        , 1 // MatchInstCrossID#1635
        , 1 // PartyIDExecutingUnit#1636
        , 1 // PartyIDSessionID#1637
        , 1 // PartyIDExecutingTrader#1638
        , 1 // PartyIDEnteringTrader#1639
        , 0 // NoLegExecs#1640
        , 0 // ExecRestatementReason#1641
        , 1 // PartyIDEnteringFirm#1642
        , 0 // ProductComplex#1643
        , 0 // OrdStatus#1644
        , 0 // ExecType#1645
        , 0 // Side#1646
        , 0 // OrdType#1647
        , 0 // TradingCapacity#1648
        , 1 // TimeInForce#1649
        , 1 // ExecInst#1650
        , 1 // TradingSessionSubID#1651
        , 1 // ApplSeqIndicator#1652
        , 1 // Account#1653
        , 1 // PartyIDPositionAccount#1654
        , 1 // PositionEffect#1655
        , 1 // PartyIDTakeUpTradingFirm#1656
        , 1 // PartyIDOrderOriginationFirm#1657
        , 1 // PartyIDBeneficiary#1658
        , 1 // PartyIDLocationID#1659
        , 1 // CustOrderHandlingInst#1660
        , 1 // ComplianceText#1661
        , 1 // FreeText1#1662
        , 1 // FreeText2#1663
        , 1 // FreeText3#1664
        , 1 // FIXClOrdID#1665
        , 0 // NoFills#1666
        , 0 // NoLegs#1667
        , 0 // Triggered#1668
        , 0 // CrossedIndicator#1669
        //// LegOrdGrpComp
        , 1 // LegAccount#1670
        , 0 // LegPositionEffect#1671
        ///
        //// FillsGrpComp
        , 0 // FillPx#1672
        , 1 // FillQty#1673
        , 0 // FillMatchID#1674
        , 0 // FillExecID#1675
        , 1 // FillLiquidityInd#1676
        ///
        //// InstrmntLegExecGrpComp
        , 0 // LegSecurityID#1677
        , 0 // LegLastPx#1678
        , 0 // LegLastQty#1679
        , 0 // LegExecID#1680
        , 0 // LegSide#1681
        , 0 // FillRefID#1682
        ///
        // OrderExecResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1683
        , 0 // TemplateID#1684
        ///
        //// ResponseHeaderMEComp
        , 0 // RequestTime#1685
        , 0 // TrdRegTSTimeIn#1686
        , 0 // TrdRegTSTimeOut#1687
        , 0 // ResponseIn#1688
        , 0 // SendingTime#1689
        , 0 // MsgSeqNum#1690
        , 0 // PartitionID#1691
        , 0 // ApplID#1692
        , 1 // ApplMsgID#1693
        , 0 // LastFragment#1694
        ///
        , 0 // OrderID#1695
        , 1 // ClOrdID#1696
        , 1 // OrigClOrdID#1697
        , 0 // SecurityID#1698
        , 0 // ExecID#1699
        , 1 // TrdRegTSEntryTime#1700
        , 1 // TrdRegTSTimePriority#1701
        , 0 // LeavesQty#1702
        , 0 // CumQty#1703
        , 0 // CxlQty#1704
        , 0 // MarketSegmentID#1705
        , 0 // NoLegExecs#1706
        , 0 // ExecRestatementReason#1707
        , 0 // Side#1708
        , 0 // ProductComplex#1709
        , 0 // OrdStatus#1710
        , 0 // ExecType#1711
        , 0 // Triggered#1712
        , 0 // CrossedIndicator#1713
        , 0 // TransactionDelayIndicator#1714
        , 0 // NoFills#1715
        //// FillsGrpComp
        , 0 // FillPx#1716
        , 0 // FillQty#1717
        , 0 // FillMatchID#1718
        , 0 // FillExecID#1719
        , 1 // FillLiquidityInd#1720
        ///
        //// InstrmntLegExecGrpComp
        , 0 // LegSecurityID#1721
        , 0 // LegLastPx#1722
        , 0 // LegLastQty#1723
        , 0 // LegExecID#1724
        , 0 // LegSide#1725
        , 0 // FillRefID#1726
        ///
        // PartyActionReport
        //// MessageHeaderOutComp
        , 0 // BodyLen#1727
        , 0 // TemplateID#1728
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#1729
        , 0 // ApplSeqNum#1730
        , 2 // ApplSubID#1731
        , 0 // PartitionID#1732
        , 0 // ApplResendFlag#1733
        , 0 // ApplID#1734
        , 0 // LastFragment#1735
        ///
        , 0 // TransactTime#1736
        , 1 // TradeDate#1737
        , 1 // RequestingPartyIDExecutingTrader#1738
        , 0 // PartyIDExecutingUnit#1739
        , 1 // PartyIDExecutingTrader#1740
        , 0 // RequestingPartyIDExecutingSystem#1741
        , 1 // MarketID#1742
        , 0 // PartyActionType#1743
        , 0 // RequestingPartyIDEnteringFirm#1744
        // PartyEntitlementsUpdateReport
        //// MessageHeaderOutComp
        , 0 // BodyLen#1745
        , 0 // TemplateID#1746
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#1747
        , 0 // ApplSeqNum#1748
        , 2 // ApplSubID#1749
        , 0 // PartitionID#1750
        , 0 // ApplResendFlag#1751
        , 0 // ApplID#1752
        , 0 // LastFragment#1753
        ///
        , 0 // TransactTime#1754
        , 0 // TradeDate#1755
        , 0 // PartyDetailIDExecutingUnit#1756
        , 0 // RequestingPartyIDExecutingSystem#1757
        , 1 // MarketID#1758
        , 0 // ListUpdateAction#1759
        , 0 // RequestingPartyEnteringFirm#1760
        , 1 // RequestingPartyClearingFirm#1761
        , 0 // PartyDetailStatus#1762
        // PingRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1763
        , 0 // TemplateID#1764
        , 2 // NetworkMsgID#1765
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1766
        , 0 // SenderSubID#1767
        ///
        , 0 // PartitionID#1768
        // PingResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1769
        , 0 // TemplateID#1770
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1771
        , 0 // TrdRegTSTimeIn#1772
        , 0 // TrdRegTSTimeOut#1773
        , 0 // ResponseIn#1774
        , 0 // SendingTime#1775
        , 0 // MsgSeqNum#1776
        , 0 // LastFragment#1777
        ///
        , 0 // TransactTime#1778
        // PreTradeRiskLimitResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1779
        , 0 // TemplateID#1780
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1781
        , 1 // TrdRegTSTimeIn#1782
        , 1 // TrdRegTSTimeOut#1783
        , 0 // ResponseIn#1784
        , 0 // SendingTime#1785
        , 0 // MsgSeqNum#1786
        , 0 // LastFragment#1787
        ///
        , 0 // RiskLimitReportID#1788
        , 0 // MarketSegmentID#1789
        , 0 // NoRiskLimits#1790
        , 0 // PartyDetailStatus#1791
        , 0 // RiskLimitPlatform#1792
        , 0 // PartyDetailExecutingUnit#1793
        //// RiskLimitsRptGrpComp
        , 1 // RiskLimitQty#1794
        , 1 // RiskLimitOpenQty#1795
        , 1 // RiskLimitNetPositionQty#1796
        , 0 // RiskLimitType#1797
        , 0 // RiskLimitRequestingPartyRole#1798
        , 1 // RiskLimitViolationIndicator#1799
        , 1 // RiskLimitGroup#1800
        ///
        // PreTradeRiskLimitsDefinitionRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1801
        , 0 // TemplateID#1802
        , 2 // NetworkMsgID#1803
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1804
        , 0 // SenderSubID#1805
        ///
        , 0 // MarketSegmentID#1806
        , 0 // RiskLimitPlatform#1807
        , 0 // NoRiskLimitsQty#1808
        , 1 // PartyDetailStatus#1809
        , 1 // RiskLimitGroup#1810
        , 1 // PartyDetailExecutingUnit#1811
        //// RiskLimitQtyGrpComp
        , 1 // RiskLimitQty#1812
        , 0 // RiskLimitType#1813
        ///
        // QuoteActivationNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#1814
        , 0 // TemplateID#1815
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#1816
        , 1 // NotificationIn#1817
        , 0 // SendingTime#1818
        , 2 // ApplSubID#1819
        , 0 // PartitionID#1820
        , 0 // ApplMsgID#1821
        , 0 // ApplID#1822
        , 0 // ApplResendFlag#1823
        , 0 // LastFragment#1824
        ///
        , 0 // MassActionReportID#1825
        , 0 // MarketSegmentID#1826
        , 1 // PartyIDEnteringTrader#1827
        , 0 // NoNotAffectedSecurities#1828
        , 1 // PartyIDEnteringFirm#1829
        , 0 // MassActionType#1830
        , 1 // MassActionSubType#1831
        , 0 // MassActionReason#1832
        //// NotAffectedSecuritiesGrpComp
        , 0 // NotAffectedSecurityID#1833
        ///
        // QuoteActivationRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1834
        , 0 // TemplateID#1835
        , 2 // NetworkMsgID#1836
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1837
        , 0 // SenderSubID#1838
        ///
        , 1 // PartyIdInvestmentDecisionMaker#1839
        , 1 // ExecutingTrader#1840
        , 0 // MarketSegmentID#1841
        , 0 // TargetPartyIDSessionID#1842
        , 0 // MassActionType#1843
        , 1 // MassActionSubType#1844
        , 1 // PartyIdInvestmentDecisionMakerQualifier#1845
        , 0 // ExecutingTraderQualifier#1846
        // QuoteActivationResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1847
        , 0 // TemplateID#1848
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1849
        , 1 // TrdRegTSTimeIn#1850
        , 1 // TrdRegTSTimeOut#1851
        , 0 // ResponseIn#1852
        , 0 // SendingTime#1853
        , 0 // MsgSeqNum#1854
        , 0 // LastFragment#1855
        ///
        , 0 // MassActionReportID#1856
        , 0 // NoNotAffectedSecurities#1857
        //// NotAffectedSecuritiesGrpComp
        , 0 // NotAffectedSecurityID#1858
        ///
        // QuoteExecutionReport
        //// MessageHeaderOutComp
        , 0 // BodyLen#1859
        , 0 // TemplateID#1860
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#1861
        , 1 // NotificationIn#1862
        , 0 // SendingTime#1863
        , 2 // ApplSubID#1864
        , 0 // PartitionID#1865
        , 1 // ApplMsgID#1866
        , 0 // ApplID#1867
        , 0 // ApplResendFlag#1868
        , 0 // LastFragment#1869
        ///
        , 0 // ExecID#1870
        , 0 // MarketSegmentID#1871
        , 0 // NoLegExecs#1872
        , 0 // NoQuoteEvents#1873
        //// QuoteEventGrpComp
        , 0 // SecurityID#1874
        , 1 // QuoteEventPx#1875
        , 1 // QuoteEventQty#1876
        , 0 // QuoteMsgID#1877
        , 1 // QuoteEventMatchID#1878
        , 1 // QuoteEventExecID#1879
        , 0 // QuoteEventType#1880
        , 0 // QuoteEventSide#1881
        , 1 // QuoteEventLiquidityInd#1882
        , 1 // QuoteEventReason#1883
        ///
        //// QuoteLegExecGrpComp
        , 0 // LegSecurityID#1884
        , 0 // LegLastPx#1885
        , 0 // LegLastQty#1886
        , 0 // LegExecID#1887
        , 0 // LegSide#1888
        , 0 // NoQuoteEventsIndex#1889
        ///
        // RFQRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1890
        , 0 // TemplateID#1891
        , 2 // NetworkMsgID#1892
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1893
        , 0 // SenderSubID#1894
        ///
        , 0 // SecurityID#1895
        , 1 // OrderQty#1896
        , 0 // MarketSegmentID#1897
        , 1 // Side#1898
        , 1 // ComplianceText#1899
        // RFQResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1900
        , 0 // TemplateID#1901
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1902
        , 0 // TrdRegTSTimeIn#1903
        , 0 // TrdRegTSTimeOut#1904
        , 0 // ResponseIn#1905
        , 0 // SendingTime#1906
        , 0 // MsgSeqNum#1907
        , 0 // LastFragment#1908
        ///
        , 0 // ExecID#1909
        // Reject
        //// MessageHeaderOutComp
        , 0 // BodyLen#1910
        , 0 // TemplateID#1911
        ///
        //// NRResponseHeaderMEComp
        , 0 // RequestTime#1912
        , 1 // TrdRegTSTimeIn#1913
        , 1 // TrdRegTSTimeOut#1914
        , 1 // ResponseIn#1915
        , 0 // SendingTime#1916
        , 0 // MsgSeqNum#1917
        , 0 // LastFragment#1918
        ///
        , 0 // SessionRejectReason#1919
        , 0 // VarTextLen#1920
        , 0 // SessionStatus#1921
        , 0 // VarText#1922
        // RetransmitMEMessageRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1923
        , 0 // TemplateID#1924
        , 2 // NetworkMsgID#1925
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1926
        , 2 // SenderSubID#1927
        ///
        , 1 // SubscriptionScope#1928
        , 0 // PartitionID#1929
        , 0 // RefApplID#1930
        , 1 // ApplBegMsgID#1931
        , 1 // ApplEndMsgID#1932
        // RetransmitMEMessageResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1933
        , 0 // TemplateID#1934
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#1935
        , 0 // SendingTime#1936
        , 0 // MsgSeqNum#1937
        ///
        , 0 // ApplTotalMessageCount#1938
        , 1 // ApplEndMsgID#1939
        , 1 // RefApplLastMsgID#1940
        // RetransmitRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1941
        , 0 // TemplateID#1942
        , 2 // NetworkMsgID#1943
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1944
        , 2 // SenderSubID#1945
        ///
        , 1 // ApplBegSeqNum#1946
        , 1 // ApplEndSeqNum#1947
        , 1 // PartitionID#1948
        , 0 // RefApplID#1949
        // RetransmitResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#1950
        , 0 // TemplateID#1951
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#1952
        , 0 // SendingTime#1953
        , 0 // MsgSeqNum#1954
        ///
        , 1 // ApplEndSeqNum#1955
        , 1 // RefApplLastSeqNum#1956
        , 0 // ApplTotalMessageCount#1957
        // ReverseTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#1958
        , 0 // TemplateID#1959
        , 2 // NetworkMsgID#1960
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#1961
        , 0 // SenderSubID#1962
        ///
        , 0 // MarketSegmentID#1963
        , 0 // PackageID#1964
        , 0 // TESExecID#1965
        , 1 // RelatedMarketSegmentID#1966
        , 0 // TrdType#1967
        , 1 // TradeReportID#1968
        , 1 // ReversalReasonText#1969
        // RiskNotificationBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#1970
        , 0 // TemplateID#1971
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#1972
        , 0 // ApplSeqNum#1973
        , 2 // ApplSubID#1974
        , 0 // PartitionID#1975
        , 0 // ApplResendFlag#1976
        , 0 // ApplID#1977
        , 0 // LastFragment#1978
        ///
        , 0 // TransactTime#1979
        , 0 // TradeDate#1980
        , 0 // PartyDetailIDExecutingUnit#1981
        , 0 // RequestingPartyIDExecutingSystem#1982
        , 1 // MarketID#1983
        , 0 // ListUpdateAction#1984
        , 1 // RiskLimitAction#1985
        , 0 // RequestingPartyEnteringFirm#1986
        , 1 // RequestingPartyClearingFirm#1987
        // SRQSCreateDealNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#1988
        , 0 // TemplateID#1989
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#1990
        , 0 // ApplSeqNum#1991
        , 1 // ApplSubID#1992
        , 2 // PartitionID#1993
        , 0 // ApplResendFlag#1994
        , 0 // ApplID#1995
        , 0 // LastFragment#1996
        ///
        , 0 // TransactTime#1997
        , 0 // LastPx#1998
        , 0 // LastQty#1999
        , 0 // QuoteID#2000
        , 1 // SecurityID#2001
        , 1 // ExpireTime#2002
        , 1 // UnderlyingPx#2003
        , 1 // UnderlyingDeltaPercentage#2004
        , 1 // UnderlyingEffectiveDeltaPercentage#2005
        , 1 // UnderlyingQty#2006
        , 1 // UnderlyingPriceStipValue#2007
        , 0 // NegotiationID#2008
        , 0 // TradeID#2009
        , 1 // OrigTradeID#2010
        , 1 // TargetPartyIDExecutingTrader#2011
        , 0 // TrdRptStatus#2012
        , 1 // TradeReportType#2013
        , 0 // MessageEventSource#2014
        , 0 // Side#2015
        , 1 // NoOrderBookItems#2016
        , 1 // TradingCapacity#2017
        , 1 // TradePublishIndicator#2018
        , 1 // HedgingInstruction#2019
        , 0 // RootPartyExecutingFirm#2020
        , 0 // RootPartyExecutingTrader#2021
        , 0 // RootPartyEnteringTrader#2022
        , 1 // TargetPartyExecutingFirm#2023
        , 1 // TargetPartyExecutingTrader#2024
        , 1 // FirmTradeID#2025
        , 1 // FirmNegotiationID#2026
        , 1 // FreeText1#2027
        , 1 // FreeText2#2028
        , 1 // FreeText3#2029
        , 1 // FreeText5#2030
        , 1 // PositionEffect#2031
        , 1 // Account#2032
        , 1 // PartyIDBeneficiary#2033
        , 1 // CustOrderHandlingInst#2034
        , 1 // PartyIDOrderOriginationFirm#2035
        , 1 // PartyIDPositionAccount#2036
        , 1 // PartyIDLocationID#2037
        , 1 // ComplianceText#2038
        , 1 // PartyIDTakeUpTradingFirm#2039
        //// OrderBookItemGrpComp
        , 0 // SecurityID#2040
        , 1 // BestBidPx#2041
        , 1 // BestBidSize#2042
        , 1 // BestOfferPx#2043
        , 1 // BestOfferSize#2044
        , 0 // MDBookType#2045
        , 1 // MDSubBookType#2046
        ///
        // SRQSDealNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2047
        , 0 // TemplateID#2048
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2049
        , 0 // ApplSeqNum#2050
        , 1 // ApplSubID#2051
        , 2 // PartitionID#2052
        , 0 // ApplResendFlag#2053
        , 0 // ApplID#2054
        , 0 // LastFragment#2055
        ///
        , 0 // TransactTime#2056
        , 1 // UnderlyingPriceStipValue#2057
        , 1 // UnderlyingPx#2058
        , 1 // LastPx#2059
        , 1 // LastQty#2060
        , 0 // NegotiationID#2061
        , 0 // TradeID#2062
        , 1 // TargetPartyIDExecutingTrader#2063
        , 1 // RequestingPartySubIDType#2064
        , 0 // TrdRptStatus#2065
        , 1 // TradeRequestResult#2066
        , 0 // MessageEventSource#2067
        , 1 // TradingCapacity#2068
        , 1 // RootPartyExecutingFirm#2069
        , 1 // RootPartyExecutingTrader#2070
        , 1 // RootPartyEnteringTrader#2071
        , 1 // TargetPartyExecutingFirm#2072
        , 1 // TargetPartyExecutingTrader#2073
        , 1 // TargetPartyEnteringTrader#2074
        , 1 // FirmTradeID#2075
        , 1 // FirmNegotiationID#2076
        , 1 // FreeText1#2077
        , 1 // FreeText2#2078
        , 1 // FreeText3#2079
        , 1 // FreeText5#2080
        , 1 // PositionEffect#2081
        , 1 // Account#2082
        , 1 // PartyIDBeneficiary#2083
        , 1 // CustOrderHandlingInst#2084
        , 1 // PartyIDOrderOriginationFirm#2085
        , 1 // PartyIDPositionAccount#2086
        , 1 // PartyIDLocationID#2087
        , 1 // ComplianceText#2088
        , 1 // PartyIDTakeUpTradingFirm#2089
        // SRQSDealResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2090
        , 0 // TemplateID#2091
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2092
        , 0 // SendingTime#2093
        , 0 // MsgSeqNum#2094
        ///
        , 1 // SecurityID#2095
        , 1 // QuoteID#2096
        , 0 // NegotiationID#2097
        , 1 // TradeID#2098
        , 1 // SecondaryTradeID#2099
        , 1 // FirmTradeID#2100
        , 1 // FirmNegotiationID#2101
        // SRQSEnterQuoteRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2102
        , 0 // TemplateID#2103
        , 2 // NetworkMsgID#2104
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2105
        , 0 // SenderSubID#2106
        ///
        , 1 // BidPx#2107
        , 1 // OfferPx#2108
        , 1 // UnderlyingDeltaPercentage#2109
        , 1 // BidSize#2110
        , 1 // OfferSize#2111
        , 1 // PartyIDClientID#2112
        , 1 // PartyIdInvestmentDecisionMaker#2113
        , 1 // ExecutingTrader#2114
        , 1 // QuoteRefPrice#2115
        , 0 // MarketSegmentID#2116
        , 0 // NegotiationID#2117
        , 1 // OrderAttributeLiquidityProvision#2118
        , 1 // ExecutingTraderQualifier#2119
        , 1 // PartyIdInvestmentDecisionMakerQualifier#2120
        , 1 // TradingCapacity#2121
        , 0 // PartyExecutingFirm#2122
        , 0 // PartyExecutingTrader#2123
        , 1 // FreeText1#2124
        , 1 // FreeText2#2125
        , 1 // FreeText3#2126
        , 1 // FreeText5#2127
        , 1 // PositionEffect#2128
        , 1 // Account#2129
        , 1 // PartyIDBeneficiary#2130
        , 1 // CustOrderHandlingInst#2131
        , 1 // PartyIDOrderOriginationFirm#2132
        , 1 // PartyIDPositionAccount#2133
        , 1 // PartyIDLocationID#2134
        , 1 // ComplianceText#2135
        , 1 // PartyIDTakeUpTradingFirm#2136
        // SRQSHitQuoteRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2137
        , 0 // TemplateID#2138
        , 2 // NetworkMsgID#2139
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2140
        , 0 // SenderSubID#2141
        ///
        , 0 // QuoteID#2142
        , 0 // OrderQty#2143
        , 1 // ValidUntilTime#2144
        , 1 // UnderlyingQty#2145
        , 1 // UnderlyingPriceStipValue#2146
        , 1 // PartyIDClientID#2147
        , 1 // PartyIdInvestmentDecisionMaker#2148
        , 1 // ExecutingTrader#2149
        , 0 // MarketSegmentID#2150
        , 0 // NegotiationID#2151
        , 0 // Side#2152
        , 1 // OrderAttributeLiquidityProvision#2153
        , 1 // ExecutingTraderQualifier#2154
        , 1 // PartyIdInvestmentDecisionMakerQualifier#2155
        , 1 // TradingCapacity#2156
        , 1 // TradePublishIndicator#2157
        , 1 // OrderOrigination#2158
        , 1 // HedgingInstruction#2159
        , 0 // PartyExecutingFirm#2160
        , 0 // PartyExecutingTrader#2161
        , 1 // FirmTradeID#2162
        , 1 // FreeText1#2163
        , 1 // FreeText2#2164
        , 1 // FreeText3#2165
        , 1 // FreeText5#2166
        , 1 // PositionEffect#2167
        , 1 // Account#2168
        , 1 // PartyIDBeneficiary#2169
        , 1 // CustOrderHandlingInst#2170
        , 1 // PartyIDOrderOriginationFirm#2171
        , 1 // PartyIDPositionAccount#2172
        , 1 // PartyIDLocationID#2173
        , 1 // ComplianceText#2174
        , 1 // PartyIDTakeUpTradingFirm#2175
        // SRQSInquireSmartRespondentRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2176
        , 0 // TemplateID#2177
        , 2 // NetworkMsgID#2178
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2179
        , 0 // SenderSubID#2180
        ///
        , 0 // MarketSegmentID#2181
        , 0 // EurexVolumeRanking#2182
        , 0 // EnlightRFQAvgRespTimeRanking#2183
        , 0 // EnlightRFQAvgRespRateRanking#2184
        , 0 // TradeToQuoteRatioRanking#2185
        // SRQSInquireSmartRespondentResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2186
        , 0 // TemplateID#2187
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2188
        , 0 // SendingTime#2189
        , 0 // MsgSeqNum#2190
        ///
        , 0 // MarketSegmentID#2191
        , 0 // NoPartyDetails#2192
        //// SmartPartyDetailGrpComp
        , 0 // PartyDetailExecutingUnit#2193
        , 0 // PartyDetailExecutingTrader#2194
        ///
        // SRQSNegotiationNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2195
        , 0 // TemplateID#2196
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2197
        , 0 // ApplSeqNum#2198
        , 1 // ApplSubID#2199
        , 2 // PartitionID#2200
        , 0 // ApplResendFlag#2201
        , 0 // ApplID#2202
        , 0 // LastFragment#2203
        ///
        , 0 // TransactTime#2204
        , 1 // QuoteRefPrice#2205
        , 1 // UnderlyingDeltaPercentage#2206
        , 1 // BidPx#2207
        , 1 // OfferPx#2208
        , 1 // LastPx#2209
        , 1 // LeavesQty#2210
        , 1 // LastQty#2211
        , 1 // EffectiveTime#2212
        , 1 // LastUpdateTime#2213
        , 1 // TradeToQuoteRatio#2214
        , 0 // NegotiationID#2215
        , 1 // NumberOfRespondents#2216
        , 1 // TradeToQuoteRatioPosition#2217
        , 0 // QuoteType#2218
        , 0 // QuoteSubType#2219
        , 0 // QuoteStatus#2220
        , 0 // QuoteInstruction#2221
        , 1 // Side#2222
        , 0 // PartyExecutingFirm#2223
        , 0 // PartyExecutingTrader#2224
        , 0 // PartyEnteringTrader#2225
        , 1 // TargetPartyExecutingFirm#2226
        , 1 // TargetPartyExecutingTrader#2227
        , 1 // FirmNegotiationID#2228
        , 1 // FreeText5#2229
        , 1 // PartyOrderOriginationTrader#2230
        // SRQSNegotiationRequesterNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2231
        , 0 // TemplateID#2232
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2233
        , 0 // ApplSeqNum#2234
        , 1 // ApplSubID#2235
        , 2 // PartitionID#2236
        , 0 // ApplResendFlag#2237
        , 0 // ApplID#2238
        , 0 // LastFragment#2239
        ///
        , 0 // TransactTime#2240
        , 1 // TrdRegTSExecutionTime#2241
        , 1 // QuoteRefPrice#2242
        , 1 // UnderlyingDeltaPercentage#2243
        , 1 // BidPx#2244
        , 1 // OfferPx#2245
        , 1 // OrderQty#2246
        , 1 // LastPx#2247
        , 1 // LeavesQty#2248
        , 1 // LastQty#2249
        , 1 // EffectiveTime#2250
        , 1 // LastUpdateTime#2251
        , 0 // NegotiationID#2252
        , 1 // NumberOfRespondents#2253
        , 0 // QuoteType#2254
        , 0 // QuoteSubType#2255
        , 0 // QuoteStatus#2256
        , 0 // NoTargetPartyIDs#2257
        , 0 // NumberOfRespDisclosureInstruction#2258
        , 1 // Side#2259
        , 0 // ShowLastDealOnClosure#2260
        , 0 // PartyExecutingFirm#2261
        , 0 // PartyExecutingTrader#2262
        , 0 // PartyEnteringTrader#2263
        , 1 // FirmNegotiationID#2264
        , 1 // FreeText5#2265
        , 1 // PartyOrderOriginationTrader#2266
        //// TargetPartiesComp
        , 1 // TargetPartyIDExecutingTrader#2267
        , 0 // SideDisclosureInstruction#2268
        , 0 // PriceDisclosureInstruction#2269
        , 0 // LeavesQtyDisclosureInstruction#2270
        , 0 // LastPxDisclosureInstruction#2271
        , 0 // LastQtyDisclosureInstruction#2272
        , 0 // FreeText5DisclosureInstruction#2273
        , 0 // PartyOrderOriginationDisclosureInstruction#2274
        , 0 // QuoteInstruction#2275
        , 1 // TargetPartyExecutingFirm#2276
        , 1 // TargetPartyExecutingTrader#2277
        , 1 // PartyDetailStatus#2278
        , 1 // PartyDetailStatusInformation#2279
        ///
        // SRQSNegotiationStatusNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2280
        , 0 // TemplateID#2281
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2282
        , 0 // ApplSeqNum#2283
        , 1 // ApplSubID#2284
        , 2 // PartitionID#2285
        , 0 // ApplResendFlag#2286
        , 0 // ApplID#2287
        , 0 // LastFragment#2288
        ///
        , 0 // TransactTime#2289
        , 1 // EffectiveTime#2290
        , 0 // NegotiationID#2291
        , 0 // QuoteStatus#2292
        , 1 // FirmNegotiationID#2293
        // SRQSOpenNegotiationNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2294
        , 0 // TemplateID#2295
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2296
        , 0 // ApplSeqNum#2297
        , 1 // ApplSubID#2298
        , 2 // PartitionID#2299
        , 0 // ApplResendFlag#2300
        , 0 // ApplID#2301
        , 0 // LastFragment#2302
        ///
        , 0 // TransactTime#2303
        , 1 // NegotiationStartTime#2304
        , 1 // SecurityID#2305
        , 1 // BidPx#2306
        , 1 // OfferPx#2307
        , 1 // LeavesQty#2308
        , 1 // LastQty#2309
        , 1 // LastPx#2310
        , 1 // QuoteRefPrice#2311
        , 1 // UnderlyingDeltaPercentage#2312
        , 1 // ExpireTime#2313
        , 1 // TradeToRequestRatio#2314
        , 1 // TradeToQuoteRatio#2315
        , 0 // NegotiationID#2316
        , 0 // MarketSegmentID#2317
        , 1 // SecuritySubType#2318
        , 1 // NumberOfRespondents#2319
        , 1 // TradeToQuoteRatioPosition#2320
        , 0 // QuoteType#2321
        , 0 // QuoteSubType#2322
        , 0 // QuoteStatus#2323
        , 0 // NoLegs#2324
        , 1 // Side#2325
        , 1 // ProductComplex#2326
        , 0 // RespondentType#2327
        , 1 // PartyExecutingFirm#2328
        , 1 // PartyExecutingTrader#2329
        , 1 // PartyEnteringTrader#2330
        , 1 // TargetPartyExecutingFirm#2331
        , 1 // TargetPartyExecutingTrader#2332
        , 1 // FirmNegotiationID#2333
        , 1 // FreeText5#2334
        , 1 // PartyOrderOriginationTrader#2335
        //// QuotReqLegsGrpComp
        , 0 // LegSecurityID#2336
        , 1 // LegRatioQty#2337
        , 1 // LegSymbol#2338
        , 0 // LegSecurityType#2339
        , 1 // LegSide#2340
        ///
        // SRQSOpenNegotiationRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2341
        , 0 // TemplateID#2342
        , 2 // NetworkMsgID#2343
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2344
        , 0 // SenderSubID#2345
        ///
        , 1 // SecurityID#2346
        , 1 // BidPx#2347
        , 1 // OfferPx#2348
        , 1 // OrderQty#2349
        , 1 // QuoteRefPrice#2350
        , 1 // UnderlyingDeltaPercentage#2351
        , 1 // ValidUntilTime#2352
        , 0 // MarketSegmentID#2353
        , 1 // SecuritySubType#2354
        , 0 // QuoteType#2355
        , 0 // QuoteSubType#2356
        , 1 // NoLegs#2357
        , 0 // NoTargetPartyIDs#2358
        , 0 // NumberOfRespDisclosureInstruction#2359
        , 1 // Side#2360
        , 1 // ProductComplex#2361
        , 0 // RespondentType#2362
        , 0 // ShowLastDealOnClosure#2363
        , 0 // PartyExecutingFirm#2364
        , 0 // PartyExecutingTrader#2365
        , 1 // FreeText5#2366
        , 1 // QuoteReqID#2367
        , 1 // PartyOrderOriginationTrader#2368
        //// QuotReqLegsGrpComp
        , 0 // LegSecurityID#2369
        , 1 // LegRatioQty#2370
        , 1 // LegSymbol#2371
        , 0 // LegSecurityType#2372
        , 1 // LegSide#2373
        ///
        //// TargetPartiesComp
        , 2 // TargetPartyIDExecutingTrader#2374
        , 0 // SideDisclosureInstruction#2375
        , 0 // PriceDisclosureInstruction#2376
        , 0 // LeavesQtyDisclosureInstruction#2377
        , 2 // LastPxDisclosureInstruction#2378
        , 2 // LastQtyDisclosureInstruction#2379
        , 0 // FreeText5DisclosureInstruction#2380
        , 0 // PartyOrderOriginationDisclosureInstruction#2381
        , 2 // QuoteInstruction#2382
        , 0 // TargetPartyExecutingFirm#2383
        , 0 // TargetPartyExecutingTrader#2384
        , 2 // PartyDetailStatus#2385
        , 2 // PartyDetailStatusInformation#2386
        ///
        // SRQSOpenNegotiationRequesterNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2387
        , 0 // TemplateID#2388
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2389
        , 0 // ApplSeqNum#2390
        , 1 // ApplSubID#2391
        , 2 // PartitionID#2392
        , 0 // ApplResendFlag#2393
        , 0 // ApplID#2394
        , 0 // LastFragment#2395
        ///
        , 0 // TransactTime#2396
        , 1 // SecurityID#2397
        , 1 // BidPx#2398
        , 1 // OfferPx#2399
        , 1 // OrderQty#2400
        , 1 // LastPx#2401
        , 1 // LastQty#2402
        , 1 // QuoteRefPrice#2403
        , 1 // UnderlyingDeltaPercentage#2404
        , 1 // ExpireTime#2405
        , 1 // TradeToRequestRatio#2406
        , 0 // NegotiationID#2407
        , 0 // MarketSegmentID#2408
        , 1 // SecuritySubType#2409
        , 1 // NumberOfRespondents#2410
        , 0 // QuoteType#2411
        , 0 // QuoteSubType#2412
        , 0 // QuoteStatus#2413
        , 0 // NoLegs#2414
        , 0 // NoTargetPartyIDs#2415
        , 1 // Side#2416
        , 1 // ProductComplex#2417
        , 0 // NumberOfRespDisclosureInstruction#2418
        , 0 // RespondentType#2419
        , 0 // ShowLastDealOnClosure#2420
        , 0 // PartyExecutingFirm#2421
        , 0 // PartyExecutingTrader#2422
        , 0 // PartyEnteringTrader#2423
        , 1 // FirmNegotiationID#2424
        , 1 // FreeText5#2425
        , 1 // PartyOrderOriginationTrader#2426
        //// QuotReqLegsGrpComp
        , 0 // LegSecurityID#2427
        , 1 // LegRatioQty#2428
        , 1 // LegSymbol#2429
        , 0 // LegSecurityType#2430
        , 1 // LegSide#2431
        ///
        //// TargetPartiesComp
        , 1 // TargetPartyIDExecutingTrader#2432
        , 0 // SideDisclosureInstruction#2433
        , 0 // PriceDisclosureInstruction#2434
        , 0 // LeavesQtyDisclosureInstruction#2435
        , 2 // LastPxDisclosureInstruction#2436
        , 2 // LastQtyDisclosureInstruction#2437
        , 0 // FreeText5DisclosureInstruction#2438
        , 0 // PartyOrderOriginationDisclosureInstruction#2439
        , 2 // QuoteInstruction#2440
        , 1 // TargetPartyExecutingFirm#2441
        , 1 // TargetPartyExecutingTrader#2442
        , 1 // PartyDetailStatus#2443
        , 1 // PartyDetailStatusInformation#2444
        ///
        // SRQSQuoteNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#2445
        , 0 // TemplateID#2446
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2447
        , 0 // ApplSeqNum#2448
        , 1 // ApplSubID#2449
        , 2 // PartitionID#2450
        , 0 // ApplResendFlag#2451
        , 0 // ApplID#2452
        , 0 // LastFragment#2453
        ///
        , 0 // TransactTime#2454
        , 1 // QuoteID#2455
        , 1 // SecondaryQuoteID#2456
        , 1 // BidPx#2457
        , 1 // BidSize#2458
        , 1 // OfferPx#2459
        , 1 // OfferSize#2460
        , 1 // UnderlyingDeltaPercentage#2461
        , 1 // QuoteRefPrice#2462
        , 0 // NegotiationID#2463
        , 0 // QuotingStatus#2464
        , 1 // TradingCapacity#2465
        , 1 // PartyIDExecutingTrader#2466
        , 1 // PartyExecutingFirm#2467
        , 1 // PartyExecutingTrader#2468
        , 1 // PartyEnteringTrader#2469
        , 1 // QuoteReqID#2470
        , 1 // FreeText1#2471
        , 1 // FreeText2#2472
        , 1 // FreeText3#2473
        , 1 // FreeText5#2474
        , 1 // PositionEffect#2475
        , 1 // Account#2476
        , 1 // PartyIDBeneficiary#2477
        , 1 // CustOrderHandlingInst#2478
        , 1 // PartyIDOrderOriginationFirm#2479
        , 1 // PartyIDPositionAccount#2480
        , 1 // PartyIDLocationID#2481
        , 1 // ComplianceText#2482
        , 1 // PartyIDTakeUpTradingFirm#2483
        // SRQSQuoteResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2484
        , 0 // TemplateID#2485
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2486
        , 0 // SendingTime#2487
        , 0 // MsgSeqNum#2488
        ///
        , 1 // QuoteID#2489
        , 0 // NegotiationID#2490
        , 1 // QuoteReqID#2491
        // SRQSQuotingStatusRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2492
        , 0 // TemplateID#2493
        , 2 // NetworkMsgID#2494
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2495
        , 0 // SenderSubID#2496
        ///
        , 0 // MarketSegmentID#2497
        , 0 // NegotiationID#2498
        , 1 // QuotingStatus#2499
        , 0 // PartyExecutingFirm#2500
        , 0 // PartyExecutingTrader#2501
        , 1 // FreeText5#2502
        // SRQSStatusBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2503
        , 0 // TemplateID#2504
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2505
        , 0 // ApplSeqNum#2506
        , 1 // ApplSubID#2507
        , 2 // PartitionID#2508
        , 0 // ApplResendFlag#2509
        , 0 // ApplID#2510
        , 0 // LastFragment#2511
        ///
        , 1 // TradeDate#2512
        , 0 // TradSesEvent#2513
        // SRQSUpdateDealStatusRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2514
        , 0 // TemplateID#2515
        , 2 // NetworkMsgID#2516
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2517
        , 0 // SenderSubID#2518
        ///
        , 1 // UnderlyingPriceStipValue#2519
        , 1 // UnderlyingPx#2520
        , 1 // LastQty#2521
        , 0 // MarketSegmentID#2522
        , 0 // NegotiationID#2523
        , 0 // TradeID#2524
        , 0 // TradeReportType#2525
        , 1 // TrdRptStatus#2526
        , 0 // PartyExecutingFirm#2527
        , 0 // PartyExecutingTrader#2528
        , 1 // FreeText5#2529
        // SRQSUpdateNegotiationRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2530
        , 0 // TemplateID#2531
        , 2 // NetworkMsgID#2532
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2533
        , 0 // SenderSubID#2534
        ///
        , 1 // QuoteRefPrice#2535
        , 1 // UnderlyingDeltaPercentage#2536
        , 1 // BidPx#2537
        , 1 // OfferPx#2538
        , 1 // OrderQty#2539
        , 0 // MarketSegmentID#2540
        , 0 // NegotiationID#2541
        , 0 // NoTargetPartyIDs#2542
        , 0 // NumberOfRespDisclosureInstruction#2543
        , 1 // Side#2544
        , 1 // QuoteCancelType#2545
        , 0 // ShowLastDealOnClosure#2546
        , 0 // PartyExecutingFirm#2547
        , 0 // PartyExecutingTrader#2548
        , 1 // FreeText5#2549
        , 1 // PartyOrderOriginationTrader#2550
        //// TargetPartiesComp
        , 1 // TargetPartyIDExecutingTrader#2551
        , 0 // SideDisclosureInstruction#2552
        , 0 // PriceDisclosureInstruction#2553
        , 0 // LeavesQtyDisclosureInstruction#2554
        , 0 // LastPxDisclosureInstruction#2555
        , 0 // LastQtyDisclosureInstruction#2556
        , 0 // FreeText5DisclosureInstruction#2557
        , 0 // PartyOrderOriginationDisclosureInstruction#2558
        , 0 // QuoteInstruction#2559
        , 1 // TargetPartyExecutingFirm#2560
        , 1 // TargetPartyExecutingTrader#2561
        , 2 // PartyDetailStatus#2562
        , 2 // PartyDetailStatusInformation#2563
        ///
        // ServiceAvailabilityBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2564
        , 0 // TemplateID#2565
        ///
        //// NRBCHeaderComp
        , 0 // SendingTime#2566
        , 0 // ApplSubID#2567
        , 0 // ApplID#2568
        , 0 // LastFragment#2569
        ///
        , 1 // MatchingEngineTradeDate#2570
        , 1 // TradeManagerTradeDate#2571
        , 1 // ApplSeqTradeDate#2572
        , 1 // T7EntryServiceTradeDate#2573
        , 1 // T7EntryServiceRtmTradeDate#2574
        , 0 // PartitionID#2575
        , 0 // MatchingEngineStatus#2576
        , 0 // TradeManagerStatus#2577
        , 0 // ApplSeqStatus#2578
        , 0 // T7EntryServiceStatus#2579
        , 0 // T7EntryServiceRtmStatus#2580
        // ServiceAvailabilityMarketBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2581
        , 0 // TemplateID#2582
        ///
        //// NRBCHeaderComp
        , 0 // SendingTime#2583
        , 0 // ApplSubID#2584
        , 0 // ApplID#2585
        , 0 // LastFragment#2586
        ///
        , 1 // SelectiveRequestForQuoteServiceTradeDate#2587
        , 0 // SelectiveRequestForQuoteServiceStatus#2588
        // SubscribeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2589
        , 0 // TemplateID#2590
        , 2 // NetworkMsgID#2591
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2592
        , 2 // SenderSubID#2593
        ///
        , 1 // SubscriptionScope#2594
        , 0 // RefApplID#2595
        // SubscribeResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2596
        , 0 // TemplateID#2597
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2598
        , 0 // SendingTime#2599
        , 0 // MsgSeqNum#2600
        ///
        , 0 // ApplSubID#2601
        // TESApproveBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2602
        , 0 // TemplateID#2603
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2604
        , 0 // ApplSeqNum#2605
        , 1 // ApplSubID#2606
        , 0 // PartitionID#2607
        , 0 // ApplResendFlag#2608
        , 0 // ApplID#2609
        , 0 // LastFragment#2610
        ///
        , 0 // SecurityID#2611
        , 0 // LastPx#2612
        , 0 // AllocQty#2613
        , 0 // TransactTime#2614
        , 1 // UnderlyingPx#2615
        , 1 // TransBkdTime#2616
        , 1 // RelatedClosePrice#2617
        , 1 // RelatedTradeQuantity#2618
        , 1 // UnderlyingQty#2619
        , 1 // CompressionID#2620
        , 0 // MarketSegmentID#2621
        , 0 // PackageID#2622
        , 0 // TESExecID#2623
        , 0 // AllocID#2624
        , 1 // UnderlyingSettlementDate#2625
        , 1 // UnderlyingMaturityDate#2626
        , 1 // RelatedTradeID#2627
        , 1 // RelatedMarketSegmentID#2628
        , 1 // NegotiationID#2629
        , 1 // TESEnrichmentRuleID#2630
        , 1 // AutoApprovalRuleID#2631
        , 0 // TrdType#2632
        , 1 // VarTextLen#2633
        , 0 // Side#2634
        , 0 // TradePublishIndicator#2635
        , 1 // ProductComplex#2636
        , 0 // TradeReportType#2637
        , 1 // TrdRptStatus#2638
        , 0 // TradingCapacity#2639
        , 1 // PartyIDSettlementLocation#2640
        , 0 // TradeAllocStatus#2641
        , 1 // HedgeType#2642
        , 0 // NoLegs#2643
        , 0 // NoEvents#2644
        , 0 // NoInstrAttrib#2645
        , 0 // NoUnderlyingStips#2646
        , 0 // MessageEventSource#2647
        , 1 // TradeReportID#2648
        , 0 // PartyExecutingFirm#2649
        , 0 // PartyExecutingTrader#2650
        , 0 // PartyIDEnteringFirm#2651
        , 0 // PartyEnteringTrader#2652
        , 0 // PositionEffect#2653
        , 1 // RootPartyExecutingFirm#2654
        , 1 // RootPartyExecutingTrader#2655
        , 1 // FreeText1#2656
        , 1 // FreeText2#2657
        , 1 // FreeText3#2658
        , 1 // PartyIDTakeUpTradingFirm#2659
        , 1 // Account#2660
        , 1 // PartyIDPositionAccount#2661
        , 1 // PartyIDOrderOriginationFirm#2662
        , 1 // PartyIDBeneficiary#2663
        , 1 // PartyIDLocationID#2664
        , 1 // CustOrderHandlingInst#2665
        , 1 // ComplianceText#2666
        , 1 // UnderlyingSecurityID#2667
        , 1 // UnderlyingSecurityDesc#2668
        , 1 // UnderlyingCurrency#2669
        , 1 // UnderlyingIssuer#2670
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#2671
        , 0 // LegPrice#2672
        , 2 // LegQty#2673
        ///
        //// InstrumentEventGrpComp
        , 0 // EventDate#2674
        , 0 // EventType#2675
        ///
        //// InstrumentAttributeGrpComp
        , 0 // InstrAttribType#2676
        , 0 // InstrAttribValue#2677
        ///
        //// UnderlyingStipGrpComp
        , 0 // UnderlyingStipValue#2678
        , 0 // UnderlyingStipType#2679
        ///
        , 1 // VarText#2680
        // TESBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2681
        , 0 // TemplateID#2682
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2683
        , 0 // ApplSeqNum#2684
        , 1 // ApplSubID#2685
        , 0 // PartitionID#2686
        , 0 // ApplResendFlag#2687
        , 0 // ApplID#2688
        , 0 // LastFragment#2689
        ///
        , 0 // SecurityID#2690
        , 0 // LastPx#2691
        , 0 // TransactTime#2692
        , 1 // UnderlyingPx#2693
        , 1 // TransBkdTime#2694
        , 1 // RelatedClosePrice#2695
        , 1 // RelatedTradeQuantity#2696
        , 1 // UnderlyingQty#2697
        , 1 // CompressionID#2698
        , 0 // MarketSegmentID#2699
        , 0 // PackageID#2700
        , 0 // TESExecID#2701
        , 1 // UnderlyingSettlementDate#2702
        , 1 // UnderlyingMaturityDate#2703
        , 1 // RelatedTradeID#2704
        , 1 // RelatedMarketSegmentID#2705
        , 1 // AutoApprovalRuleID#2706
        , 0 // TrdType#2707
        , 1 // VarTextLen#2708
        , 0 // TradeReportType#2709
        , 1 // TrdRptStatus#2710
        , 1 // ProductComplex#2711
        , 0 // TradePublishIndicator#2712
        , 0 // NoEvents#2713
        , 0 // NoInstrAttrib#2714
        , 0 // NoUnderlyingStips#2715
        , 0 // NoSideAllocs#2716
        , 0 // NoLegs#2717
        , 1 // PartyIDSettlementLocation#2718
        , 1 // HedgeType#2719
        , 1 // SwapClearer#2720
        , 0 // MessageEventSource#2721
        , 1 // TradeReportText#2722
        , 1 // TradeReportID#2723
        , 0 // RootPartyExecutingFirm#2724
        , 0 // RootPartyExecutingTrader#2725
        , 1 // UnderlyingSecurityID#2726
        , 1 // UnderlyingSecurityDesc#2727
        , 1 // UnderlyingCurrency#2728
        , 1 // UnderlyingIssuer#2729
        //// SideAllocGrpBCComp
        , 0 // AllocQty#2730
        , 2 // ReversalApprovalTime#2731
        , 0 // IndividualAllocID#2732
        , 1 // TESEnrichmentRuleID#2733
        , 0 // PartyExecutingFirm#2734
        , 0 // PartyExecutingTrader#2735
        , 0 // Side#2736
        , 0 // TradeAllocStatus#2737
        ///
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#2738
        , 0 // LegPrice#2739
        , 1 // LegQty#2740
        ///
        //// InstrumentEventGrpComp
        , 0 // EventDate#2741
        , 0 // EventType#2742
        ///
        //// InstrumentAttributeGrpComp
        , 0 // InstrAttribType#2743
        , 0 // InstrAttribValue#2744
        ///
        //// UnderlyingStipGrpComp
        , 0 // UnderlyingStipValue#2745
        , 0 // UnderlyingStipType#2746
        ///
        , 1 // VarText#2747
        // TESCompressionRunStatusBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2748
        , 0 // TemplateID#2749
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2750
        , 0 // ApplSeqNum#2751
        , 1 // ApplSubID#2752
        , 0 // PartitionID#2753
        , 0 // ApplResendFlag#2754
        , 0 // ApplID#2755
        , 0 // LastFragment#2756
        ///
        , 1 // CompressionID#2757
        , 1 // MarketSegmentID#2758
        , 1 // CompressionAction#2759
        , 1 // CompressionStatus#2760
        , 1 // TradePublishIndicator#2761
        // TESCompressionRunStatusRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#2762
        , 0 // TemplateID#2763
        , 2 // NetworkMsgID#2764
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#2765
        , 0 // SenderSubID#2766
        ///
        , 1 // CompressionID#2767
        , 0 // MarketSegmentID#2768
        , 0 // CompressionAction#2769
        , 1 // TradePublishIndicator#2770
        // TESCompressionRunStatusResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2771
        , 0 // TemplateID#2772
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2773
        , 0 // SendingTime#2774
        , 0 // MsgSeqNum#2775
        ///
        , 1 // CompressionID#2776
        , 1 // MarketSegmentID#2777
        , 1 // CompressionAction#2778
        , 1 // CompressionStatus#2779
        , 1 // TradePublishIndicator#2780
        // TESDeleteBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2781
        , 0 // TemplateID#2782
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2783
        , 0 // ApplSeqNum#2784
        , 1 // ApplSubID#2785
        , 0 // PartitionID#2786
        , 0 // ApplResendFlag#2787
        , 0 // ApplID#2788
        , 0 // LastFragment#2789
        ///
        , 0 // TransactTime#2790
        , 1 // CompressionID#2791
        , 0 // MarketSegmentID#2792
        , 0 // PackageID#2793
        , 0 // TESExecID#2794
        , 0 // TrdType#2795
        , 0 // DeleteReason#2796
        , 0 // TradeReportType#2797
        , 1 // TrdRptStatus#2798
        , 0 // MessageEventSource#2799
        , 1 // TradeReportID#2800
        // TESExecutionBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2801
        , 0 // TemplateID#2802
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2803
        , 0 // ApplSeqNum#2804
        , 1 // ApplSubID#2805
        , 0 // PartitionID#2806
        , 0 // ApplResendFlag#2807
        , 0 // ApplID#2808
        , 0 // LastFragment#2809
        ///
        , 0 // TransactTime#2810
        , 1 // CompressionID#2811
        , 0 // MarketSegmentID#2812
        , 0 // PackageID#2813
        , 0 // TESExecID#2814
        , 0 // AllocID#2815
        , 0 // TrdType#2816
        , 0 // TradeReportType#2817
        , 0 // Side#2818
        , 1 // TrdRptStatus#2819
        , 0 // MessageEventSource#2820
        // TESResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#2821
        , 0 // TemplateID#2822
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#2823
        , 0 // SendingTime#2824
        , 0 // MsgSeqNum#2825
        ///
        , 0 // TESExecID#2826
        , 0 // TradeReportID#2827
        // TESReversalBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2828
        , 0 // TemplateID#2829
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2830
        , 0 // ApplSeqNum#2831
        , 1 // ApplSubID#2832
        , 0 // PartitionID#2833
        , 0 // ApplResendFlag#2834
        , 0 // ApplID#2835
        , 0 // LastFragment#2836
        ///
        , 1 // ReversalInitiationTime#2837
        , 0 // MarketSegmentID#2838
        , 0 // PackageID#2839
        , 0 // TESExecID#2840
        , 1 // RelatedMarketSegmentID#2841
        , 0 // TrdType#2842
        , 1 // TrdRptStatus#2843
        , 1 // ReversalCancellationReason#2844
        , 0 // NoSideAllocs#2845
        , 1 // TradeReportID#2846
        , 1 // ReversalReasonText#2847
        //// SideAllocGrpBCComp
        , 2 // AllocQty#2848
        , 1 // ReversalApprovalTime#2849
        , 0 // IndividualAllocID#2850
        , 2 // TESEnrichmentRuleID#2851
        , 2 // PartyExecutingFirm#2852
        , 2 // PartyExecutingTrader#2853
        , 2 // Side#2854
        , 0 // TradeAllocStatus#2855
        ///
        // TESTradeBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2856
        , 0 // TemplateID#2857
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2858
        , 0 // ApplSeqNum#2859
        , 1 // ApplSubID#2860
        , 0 // PartitionID#2861
        , 0 // ApplResendFlag#2862
        , 0 // ApplID#2863
        , 0 // LastFragment#2864
        ///
        , 0 // SecurityID#2865
        , 0 // LastPx#2866
        , 0 // LastQty#2867
        , 1 // ClearingTradePrice#2868
        , 1 // ClearingTradeQty#2869
        , 0 // TransactTime#2870
        , 1 // RelatedSecurityID#2871
        , 1 // CompressionID#2872
        , 1 // RootPartyIDClientID#2873
        , 1 // ExecutingTrader#2874
        , 1 // RootPartyIDInvestmentDecisionMaker#2875
        , 1 // BasketTrdMatchID#2876
        , 1 // SideLastPx#2877
        , 1 // SideLastQty#2878
        , 0 // PackageID#2879
        , 0 // MarketSegmentID#2880
        , 0 // TradeID#2881
        , 0 // TradeDate#2882
        , 0 // SideTradeID#2883
        , 1 // RootPartyIDSessionID#2884
        , 1 // OrigTradeID#2885
        , 0 // RootPartyIDExecutingUnit#2886
        , 0 // RootPartyIDExecutingTrader#2887
        , 0 // RootPartyIDClearingUnit#2888
        , 1 // StrategyLinkID#2889
        , 1 // RelatedSymbol#2890
        , 1 // TotNumTradeReports#2891
        , 1 // NegotiationID#2892
        , 1 // NoSRQSRelatedTradeIDs#2893
        , 1 // BasketProfileID#2894
        , 1 // TrdType#2895
        , 1 // ProductComplex#2896
        , 1 // RelatedProductComplex#2897
        , 0 // Side#2898
        , 1 // TradingCapacity#2899
        , 0 // TradeReportType#2900
        , 0 // TransferReason#2901
        , 0 // TradePublishIndicator#2902
        , 1 // MultiLegReportingType#2903
        , 1 // PositionEffect#2904
        , 1 // MultilegPriceModel#2905
        , 1 // OrderAttributeLiquidityProvision#2906
        , 1 // OrderAttributeRiskReduction#2907
        , 1 // ExecutingTraderQualifier#2908
        , 1 // RootPartyIDInvestmentDecisionMakerQualifier#2909
        , 1 // OrderOrigination#2910
        , 1 // ReversalIndicator#2911
        , 1 // Account#2912
        , 1 // RootPartyIDPositionAccount#2913
        , 1 // CustOrderHandlingInst#2914
        , 1 // FreeText1#2915
        , 1 // FreeText2#2916
        , 1 // FreeText3#2917
        , 0 // RootPartyExecutingFirm#2918
        , 0 // RootPartyExecutingTrader#2919
        , 0 // RootPartyClearingFirm#2920
        , 0 // RootPartyClearingOrganization#2921
        , 1 // RootPartyIDBeneficiary#2922
        , 1 // RootPartyIDTakeUpTradingFirm#2923
        , 1 // RootPartyIDOrderOriginationFirm#2924
        , 1 // RootPartyIDExecutionVenue#2925
        , 1 // RegulatoryTradeID#2926
        , 1 // BasketPartyContraFirm#2927
        , 1 // BasketSideTradeReportID#2928
        //// SRQSRelatedTradeIDGrpComp
        , 0 // SRQSRelatedTradeID#2929
        ///
        // TESTradingSessionStatusBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2930
        , 0 // TemplateID#2931
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2932
        , 0 // ApplSeqNum#2933
        , 1 // ApplSubID#2934
        , 0 // PartitionID#2935
        , 0 // ApplResendFlag#2936
        , 0 // ApplID#2937
        , 0 // LastFragment#2938
        ///
        , 0 // TradeDate#2939
        , 0 // TradSesEvent#2940
        // TESUploadBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#2941
        , 0 // TemplateID#2942
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#2943
        , 0 // ApplSeqNum#2944
        , 1 // ApplSubID#2945
        , 0 // PartitionID#2946
        , 0 // ApplResendFlag#2947
        , 0 // ApplID#2948
        , 0 // LastFragment#2949
        ///
        , 0 // SecurityID#2950
        , 0 // LastPx#2951
        , 1 // TransBkdTime#2952
        , 0 // TransactTime#2953
        , 1 // UnderlyingPx#2954
        , 1 // RelatedClosePrice#2955
        , 1 // RelatedTradeQuantity#2956
        , 1 // UnderlyingQty#2957
        , 1 // CompressionID#2958
        , 0 // MarketSegmentID#2959
        , 0 // PackageID#2960
        , 0 // TESExecID#2961
        , 1 // UnderlyingSettlementDate#2962
        , 1 // UnderlyingMaturityDate#2963
        , 1 // RelatedTradeID#2964
        , 1 // RelatedMarketSegmentID#2965
        , 0 // TrdType#2966
        , 1 // ProductComplex#2967
        , 0 // TradeReportType#2968
        , 1 // TrdRptStatus#2969
        , 0 // TradePublishIndicator#2970
        , 0 // TradePlatform#2971
        , 0 // NoSideAllocs#2972
        , 0 // NoLegs#2973
        , 0 // NoEvents#2974
        , 0 // NoInstrAttrib#2975
        , 0 // NoUnderlyingStips#2976
        , 1 // HedgeType#2977
        , 1 // PartyIDSettlementLocation#2978
        , 0 // MessageEventSource#2979
        , 1 // TradeReportID#2980
        , 1 // RootPartyExecutingFirm#2981
        , 1 // RootPartyExecutingTrader#2982
        , 1 // UnderlyingSecurityID#2983
        , 1 // UnderlyingSecurityDesc#2984
        , 1 // UnderlyingCurrency#2985
        , 1 // UnderlyingIssuer#2986
        //// SideAllocExtGrpComp
        , 0 // AllocQty#2987
        , 1 // PartyIDClientID#2988
        , 1 // PartyIdInvestmentDecisionMaker#2989
        , 1 // ExecutingTrader#2990
        , 0 // IndividualAllocID#2991
        , 0 // PartyExecutingFirm#2992
        , 0 // PartyExecutingTrader#2993
        , 1 // TESEnrichmentRuleID#2994
        , 0 // Side#2995
        , 0 // TradeAllocStatus#2996
        , 0 // TradingCapacity#2997
        , 0 // PositionEffect#2998
        , 1 // OrderAttributeLiquidityProvision#2999
        , 1 // ExecutingTraderQualifier#3000
        , 1 // PartyIdInvestmentDecisionMakerQualifier#3001
        , 1 // OrderAttributeRiskReduction#3002
        , 1 // OrderOrigination#3003
        , 1 // Account#3004
        , 1 // PartyIDPositionAccount#3005
        , 1 // PartyIDTakeUpTradingFirm#3006
        , 1 // FreeText1#3007
        , 1 // FreeText2#3008
        , 1 // FreeText3#3009
        , 1 // PartyIDOrderOriginationFirm#3010
        , 1 // PartyIDBeneficiary#3011
        , 1 // PartyIDLocationID#3012
        , 1 // CustOrderHandlingInst#3013
        , 1 // ComplianceText#3014
        ///
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#3015
        , 0 // LegPrice#3016
        , 1 // LegQty#3017
        ///
        //// InstrumentEventGrpComp
        , 0 // EventDate#3018
        , 0 // EventType#3019
        ///
        //// InstrumentAttributeGrpComp
        , 0 // InstrAttribType#3020
        , 0 // InstrAttribValue#3021
        ///
        //// UnderlyingStipGrpComp
        , 0 // UnderlyingStipValue#3022
        , 0 // UnderlyingStipType#3023
        ///
        // TMTradingSessionStatusBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#3024
        , 0 // TemplateID#3025
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#3026
        , 0 // ApplSeqNum#3027
        , 1 // ApplSubID#3028
        , 0 // PartitionID#3029
        , 0 // ApplResendFlag#3030
        , 0 // ApplID#3031
        , 0 // LastFragment#3032
        ///
        , 0 // TradSesEvent#3033
        // ThrottleUpdateNotification
        //// MessageHeaderOutComp
        , 0 // BodyLen#3034
        , 0 // TemplateID#3035
        ///
        //// NotifHeaderComp
        , 0 // SendingTime#3036
        ///
        , 0 // ThrottleTimeInterval#3037
        , 0 // ThrottleNoMsgs#3038
        , 0 // ThrottleDisconnectLimit#3039
        // TradeBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#3040
        , 0 // TemplateID#3041
        ///
        //// RBCHeaderComp
        , 0 // SendingTime#3042
        , 0 // ApplSeqNum#3043
        , 1 // ApplSubID#3044
        , 0 // PartitionID#3045
        , 0 // ApplResendFlag#3046
        , 0 // ApplID#3047
        , 0 // LastFragment#3048
        ///
        , 0 // SecurityID#3049
        , 1 // RelatedSecurityID#3050
        , 1 // Price#3051
        , 0 // LastPx#3052
        , 0 // LastQty#3053
        , 1 // SideLastPx#3054
        , 1 // SideLastQty#3055
        , 1 // ClearingTradePrice#3056
        , 1 // ClearingTradeQty#3057
        , 0 // TransactTime#3058
        , 1 // OrderID#3059
        , 1 // ClOrdID#3060
        , 1 // LeavesQty#3061
        , 1 // CumQty#3062
        , 1 // RootPartyIDClientID#3063
        , 1 // ExecutingTrader#3064
        , 1 // RootPartyIDInvestmentDecisionMaker#3065
        , 0 // TradeID#3066
        , 1 // OrigTradeID#3067
        , 0 // RootPartyIDExecutingUnit#3068
        , 1 // RootPartyIDSessionID#3069
        , 1 // RootPartyIDExecutingTrader#3070
        , 0 // RootPartyIDClearingUnit#3071
        , 0 // MarketSegmentID#3072
        , 1 // RelatedSymbol#3073
        , 0 // SideTradeID#3074
        , 0 // MatchDate#3075
        , 0 // TrdMatchID#3076
        , 1 // StrategyLinkID#3077
        , 1 // TotNumTradeReports#3078
        , 1 // MultiLegReportingType#3079
        , 0 // TradeReportType#3080
        , 0 // TransferReason#3081
        , 1 // RootPartyIDBeneficiary#3082
        , 1 // RootPartyIDTakeUpTradingFirm#3083
        , 1 // RootPartyIDOrderOriginationFirm#3084
        , 1 // MatchType#3085
        , 1 // MatchSubType#3086
        , 0 // Side#3087
        , 1 // SideLiquidityInd#3088
        , 0 // TradingCapacity#3089
        , 1 // OrderOrigination#3090
        , 1 // OrderAttributeLiquidityProvision#3091
        , 1 // OrderAttributeRiskReduction#3092
        , 1 // ExecutingTraderQualifier#3093
        , 1 // RootPartyIDInvestmentDecisionMakerQualifier#3094
        , 1 // Account#3095
        , 1 // RootPartyIDPositionAccount#3096
        , 1 // PositionEffect#3097
        , 1 // CustOrderHandlingInst#3098
        , 1 // FreeText1#3099
        , 1 // FreeText2#3100
        , 1 // FreeText3#3101
        , 1 // OrderCategory#3102
        , 1 // OrdType#3103
        , 1 // RelatedProductComplex#3104
        , 1 // OrderSide#3105
        , 0 // RootPartyClearingOrganization#3106
        , 0 // RootPartyExecutingFirm#3107
        , 1 // RootPartyExecutingTrader#3108
        , 0 // RootPartyClearingFirm#3109
        , 1 // RegulatoryTradeID#3110
        , 1 // RootPartyIDExecutionVenue#3111
        // TradingSessionStatusBroadcast
        //// MessageHeaderOutComp
        , 0 // BodyLen#3112
        , 0 // TemplateID#3113
        ///
        //// RBCHeaderMEComp
        , 1 // TrdRegTSTimeOut#3114
        , 1 // NotificationIn#3115
        , 0 // SendingTime#3116
        , 1 // ApplSubID#3117
        , 0 // PartitionID#3118
        , 0 // ApplMsgID#3119
        , 0 // ApplID#3120
        , 0 // ApplResendFlag#3121
        , 0 // LastFragment#3122
        ///
        , 1 // MarketSegmentID#3123
        , 1 // TradeDate#3124
        , 0 // TradSesEvent#3125
        , 1 // RefApplLastMsgID#3126
        // UnsubscribeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#3127
        , 0 // TemplateID#3128
        , 2 // NetworkMsgID#3129
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#3130
        , 2 // SenderSubID#3131
        ///
        , 0 // RefApplSubID#3132
        // UnsubscribeResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#3133
        , 0 // TemplateID#3134
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#3135
        , 0 // SendingTime#3136
        , 0 // MsgSeqNum#3137
        ///
        // UploadTESTradeRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#3138
        , 0 // TemplateID#3139
        , 2 // NetworkMsgID#3140
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#3141
        , 0 // SenderSubID#3142
        ///
        , 0 // SecurityID#3143
        , 0 // LastPx#3144
        , 1 // TransBkdTime#3145
        , 1 // UnderlyingPx#3146
        , 1 // RelatedClosePrice#3147
        , 1 // RelatedTradeQuantity#3148
        , 1 // UnderlyingQty#3149
        , 1 // CompressionID#3150
        , 0 // MarketSegmentID#3151
        , 1 // UnderlyingSettlementDate#3152
        , 1 // UnderlyingMaturityDate#3153
        , 1 // RelatedTradeID#3154
        , 1 // RelatedMarketSegmentID#3155
        , 0 // TrdType#3156
        , 1 // ProductComplex#3157
        , 0 // TradeReportType#3158
        , 0 // TradePublishIndicator#3159
        , 0 // NoSideAllocs#3160
        , 0 // NoLegs#3161
        , 0 // NoEvents#3162
        , 0 // NoInstrAttrib#3163
        , 0 // NoUnderlyingStips#3164
        , 0 // SkipValidations#3165
        , 1 // TrdRptStatus#3166
        , 1 // TradePlatform#3167
        , 1 // HedgeType#3168
        , 1 // PartyIDSettlementLocation#3169
        , 1 // ValueCheckTypeMinLotSize#3170
        , 1 // TradeReportID#3171
        , 1 // TradeReportText#3172
        , 1 // UnderlyingSecurityID#3173
        , 1 // UnderlyingSecurityDesc#3174
        , 1 // UnderlyingCurrency#3175
        , 1 // UnderlyingIssuer#3176
        //// SideAllocExtGrpComp
        , 0 // AllocQty#3177
        , 1 // PartyIDClientID#3178
        , 1 // PartyIdInvestmentDecisionMaker#3179
        , 1 // ExecutingTrader#3180
        , 2 // IndividualAllocID#3181
        , 0 // PartyExecutingFirm#3182
        , 0 // PartyExecutingTrader#3183
        , 1 // TESEnrichmentRuleID#3184
        , 0 // Side#3185
        , 2 // TradeAllocStatus#3186
        , 1 // TradingCapacity#3187
        , 1 // PositionEffect#3188
        , 1 // OrderAttributeLiquidityProvision#3189
        , 1 // ExecutingTraderQualifier#3190
        , 1 // PartyIdInvestmentDecisionMakerQualifier#3191
        , 1 // OrderAttributeRiskReduction#3192
        , 1 // OrderOrigination#3193
        , 1 // Account#3194
        , 1 // PartyIDPositionAccount#3195
        , 1 // PartyIDTakeUpTradingFirm#3196
        , 1 // FreeText1#3197
        , 1 // FreeText2#3198
        , 1 // FreeText3#3199
        , 1 // PartyIDOrderOriginationFirm#3200
        , 1 // PartyIDBeneficiary#3201
        , 1 // PartyIDLocationID#3202
        , 1 // CustOrderHandlingInst#3203
        , 1 // ComplianceText#3204
        ///
        //// TrdInstrmntLegGrpComp
        , 0 // LegSecurityID#3205
        , 0 // LegPrice#3206
        , 2 // LegQty#3207
        ///
        //// InstrumentEventGrpComp
        , 0 // EventDate#3208
        , 0 // EventType#3209
        ///
        //// InstrumentAttributeGrpComp
        , 0 // InstrAttribType#3210
        , 0 // InstrAttribValue#3211
        ///
        //// UnderlyingStipGrpComp
        , 0 // UnderlyingStipValue#3212
        , 0 // UnderlyingStipType#3213
        ///
        // UserLoginRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#3214
        , 0 // TemplateID#3215
        , 2 // NetworkMsgID#3216
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#3217
        , 2 // SenderSubID#3218
        ///
        , 0 // Username#3219
        , 0 // Password#3220
        // UserLoginResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#3221
        , 0 // TemplateID#3222
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#3223
        , 0 // SendingTime#3224
        , 0 // MsgSeqNum#3225
        ///
        // UserLogoutRequest
        //// MessageHeaderInComp
        , 0 // BodyLen#3226
        , 0 // TemplateID#3227
        , 2 // NetworkMsgID#3228
        ///
        //// RequestHeaderComp
        , 0 // MsgSeqNum#3229
        , 2 // SenderSubID#3230
        ///
        , 0 // Username#3231
        // UserLogoutResponse
        //// MessageHeaderOutComp
        , 0 // BodyLen#3232
        , 0 // TemplateID#3233
        ///
        //// ResponseHeaderComp
        , 0 // RequestTime#3234
        , 0 // SendingTime#3235
        , 0 // MsgSeqNum#3236
        ///
        , 0 // filler
    };
    static const int16_t tid2uidx[] = {
        998 /* LogonRequest */
        , 1016 /* LogonResponse */
        , 1030 /* LogoutRequest */
        , 1035 /* LogoutResponse */
        , -1
        , 2596 /* SubscribeResponse */
        , 3127 /* UnsubscribeRequest */
        , 3133 /* UnsubscribeResponse */
        , 1941 /* RetransmitRequest */
        , 1950 /* RetransmitResponse */
        , 1910 /* Reject */
        , 890 /* Heartbeat */
        , 878 /* ForcedLogoutNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , 3214 /* UserLoginRequest */
        , 3221 /* UserLoginResponse */
        , -1
        , -1
        , -1
        , 893 /* HeartbeatNotification */
        , 3232 /* UserLogoutResponse */
        , 2589 /* SubscribeRequest */
        , 1923 /* RetransmitMEMessageRequest */
        , 1933 /* RetransmitMEMessageResponse */
        , 3034 /* ThrottleUpdateNotification */
        , 3226 /* UserLogoutRequest */
        , 2564 /* ServiceAvailabilityBroadcast */
        , 1555 /* NewsBroadcast */
        , 365 /* BroadcastErrorNotification */
        , 1970 /* RiskNotificationBroadcast */
        , 1745 /* PartyEntitlementsUpdateReport */
        , 953 /* InquireSessionListRequest */
        , 958 /* InquireSessionListResponse */
        , 985 /* LegalNotificationBroadcast */
        , 967 /* InquireUserRequest */
        , 973 /* InquireUserResponse */
        , 896 /* InquireEnrichmentRuleIDListRequest */
        , 902 /* InquireEnrichmentRuleIDListResponse */
        , 1727 /* PartyActionReport */
        , 883 /* ForcedUserLogoutNotification */
        , 2581 /* ServiceAvailabilityMarketBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1487 /* NewOrderSingleRequest */
        , 1462 /* NewOrderResponse */
        , 1442 /* NewOrderNRResponse */
        , 1683 /* OrderExecResponse */
        , 1568 /* OrderExecNotification */
        , -1
        , 1271 /* ModifyOrderSingleRequest */
        , 1243 /* ModifyOrderResponse */
        , 1219 /* ModifyOrderNRResponse */
        , 714 /* DeleteOrderSingleRequest */
        , 690 /* DeleteOrderResponse */
        , 669 /* DeleteOrderNRResponse */
        , 624 /* DeleteOrderBroadcast */
        , 1372 /* NewOrderComplexRequest */
        , 1145 /* ModifyOrderComplexRequest */
        , -1
        , -1
        , 1609 /* OrderExecReportBroadcast */
        , 449 /* CrossRequest */
        , 458 /* CrossRequestResponse */
        , 521 /* DeleteAllOrderRequest */
        , 537 /* DeleteAllOrderResponse */
        , 468 /* DeleteAllOrderBroadcast */
        , 652 /* DeleteOrderComplexRequest */
        , 495 /* DeleteAllOrderNRResponse */
        , 1530 /* NewOrderSingleShortRequest */
        , 1317 /* ModifyOrderSingleShortRequest */
        , -1
        , -1
        , 1415 /* NewOrderComplexShortRequest */
        , 1191 /* ModifyOrderComplexShortRequest */
        , 784 /* EnterCLIPRequest */
        , 611 /* DeleteCLIPRequest */
        , 432 /* CLIPResponse */
        , 374 /* CLIPDeletionNotification */
        , 395 /* CLIPExecutionNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 0 /* AddComplexInstrumentRequest */
        , 16 /* AddComplexInstrumentResponse */
        , 1040 /* MMParameterDefinitionRequest */
        , 1052 /* MMParameterDefinitionResponse */
        , 919 /* InquireMMParameterRequest */
        , 926 /* InquireMMParameterResponse */
        , 3112 /* TradingSessionStatusBroadcast */
        , 505 /* DeleteAllOrderQuoteEventBroadcast */
        , 43 /* AddFlexibleInstrumentRequest */
        , 56 /* AddFlexibleInstrumentResponse */
        , 944 /* InquirePreTradeRiskLimitsRequest */
        , 1801 /* PreTradeRiskLimitsDefinitionRequest */
        , 1779 /* PreTradeRiskLimitResponse */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1763 /* PingRequest */
        , 1769 /* PingResponse */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 1890 /* RFQRequest */
        , 1900 /* RFQResponse */
        , 1834 /* QuoteActivationRequest */
        , 1847 /* QuoteActivationResponse */
        , 1062 /* MassQuoteRequest */
        , 1086 /* MassQuoteResponse */
        , 1859 /* QuoteExecutionReport */
        , 577 /* DeleteAllQuoteRequest */
        , 588 /* DeleteAllQuoteResponse */
        , 555 /* DeleteAllQuoteBroadcast */
        , 1814 /* QuoteActivationNotification */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 3040 /* TradeBroadcast */
        , 3024 /* TMTradingSessionStatusBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 827 /* EnterTESTradeRequest */
        , 1343 /* ModifyTESTradeRequest */
        , 731 /* DeleteTESTradeRequest */
        , 172 /* ApproveTESTradeRequest */
        , 2681 /* TESBroadcast */
        , -1
        , 2781 /* TESDeleteBroadcast */
        , 2602 /* TESApproveBroadcast */
        , -1
        , -1
        , 2801 /* TESExecutionBroadcast */
        , 2821 /* TESResponse */
        , 3138 /* UploadTESTradeRequest */
        , 2941 /* TESUploadBroadcast */
        , 2856 /* TESTradeBroadcast */
        , 2930 /* TESTradingSessionStatusBroadcast */
        , 2762 /* TESCompressionRunStatusRequest */
        , 2771 /* TESCompressionRunStatusResponse */
        , 2748 /* TESCompressionRunStatusBroadcast */
        , -1
        , 744 /* EnterBasketTradeRequest */
        , 1104 /* ModifyBasketTradeRequest */
        , 600 /* DeleteBasketTradeRequest */
        , 117 /* ApproveBasketTradeRequest */
        , 358 /* BasketResponse */
        , 269 /* BasketBroadcast */
        , 317 /* BasketDeleteBroadcast */
        , 211 /* BasketApproveBroadcast */
        , 335 /* BasketExecutionBroadcast */
        , 75 /* AmendBasketTradeRequest */
        , 1958 /* ReverseTESTradeRequest */
        , 160 /* ApproveReverseTESTradeRequest */
        , 2828 /* TESReversalBroadcast */
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , -1
        , 2341 /* SRQSOpenNegotiationRequest */
        , 2530 /* SRQSUpdateNegotiationRequest */
        , 2102 /* SRQSEnterQuoteRequest */
        , 2484 /* SRQSQuoteResponse */
        , 2137 /* SRQSHitQuoteRequest */
        , 2090 /* SRQSDealResponse */
        , 2514 /* SRQSUpdateDealStatusRequest */
        , 2445 /* SRQSQuoteNotification */
        , 1988 /* SRQSCreateDealNotification */
        , 2047 /* SRQSDealNotification */
        , 2387 /* SRQSOpenNegotiationRequesterNotification */
        , 2294 /* SRQSOpenNegotiationNotification */
        , 2231 /* SRQSNegotiationRequesterNotification */
        , 2195 /* SRQSNegotiationNotification */
        , 2503 /* SRQSStatusBroadcast */
        , 2280 /* SRQSNegotiationStatusNotification */
        , -1
        , 2492 /* SRQSQuotingStatusRequest */
        , 2176 /* SRQSInquireSmartRespondentRequest */
        , 2186 /* SRQSInquireSmartRespondentResponse */
    };
    static int * const dscp_bits[] = {
        &hf_eti_dscp_exec_summary,
        &hf_eti_dscp_improved,
        &hf_eti_dscp_widened,
        NULL
    };
    if (templateid < 10000 || templateid > 10719) {
        proto_tree_add_expert_format(root, pinfo, &ei_eti_invalid_template, tvb, 4, 4,
            "Template ID out of range: %" PRIu16, templateid);
        return tvb_captured_length(tvb);
    }
    int fidx = tid2fidx[templateid - 10000];
    if (fidx == -1) {
        proto_tree_add_expert_format(root, pinfo, &ei_eti_invalid_template, tvb, 4, 4,
            "Unallocated Template ID: %" PRIu16, templateid);
        return tvb_captured_length(tvb);
    }
    if (bodylen < tid2size[templateid - 10000][0] || bodylen > tid2size[templateid - 10000][1]) {
        if (tid2size[templateid - 10000][0] != tid2size[templateid - 10000][1])
            proto_tree_add_expert_format(root, pinfo, &ei_eti_invalid_length, tvb, 0, 4,
                    "Unexpected BodyLen value of %" PRIu32 ", expected:  %" PRIu32 "..%" PRIu32, bodylen, tid2size[templateid - 10000][0], tid2size[templateid - 10000][1]);
        else
            proto_tree_add_expert_format(root, pinfo, &ei_eti_invalid_length, tvb, 0, 4,
                    "Unexpected BodyLen value of %" PRIu32 ", expected:  %" PRIu32, bodylen, tid2size[templateid - 10000][0]);
    }
    if (bodylen % 8)
        proto_tree_add_expert_format(root, pinfo, &ei_eti_unaligned, tvb, 0, 4,
                "BodyLen value of %" PRIu32 " is not divisible by 8", bodylen);

    int uidx = tid2uidx[templateid - 10000];
    DISSECTOR_ASSERT_CMPINT(uidx, >=, 0);
    DISSECTOR_ASSERT_CMPUINT(((size_t)uidx), <, (sizeof usages / sizeof usages[0]));

    int old_fidx = 0;
    int old_uidx = 0;
    unsigned top = 1;
    unsigned counter[8] = {0};
    unsigned off = 0;
    unsigned struct_off = 0;
    unsigned repeats = 0;
    proto_tree *t = root;
    while (top) {
        DISSECTOR_ASSERT_CMPINT(fidx, >=, 0);
        DISSECTOR_ASSERT_CMPUINT(((size_t)fidx), <, (sizeof fields / sizeof fields[0]));
        DISSECTOR_ASSERT_CMPINT(uidx, >=, 0);
        DISSECTOR_ASSERT_CMPUINT(((size_t)uidx), <, (sizeof usages / sizeof usages[0]));

        switch (fields[fidx].type) {
            case ETI_EOF:
                DISSECTOR_ASSERT_CMPUINT(top, >=, 1);
                DISSECTOR_ASSERT_CMPUINT(top, <=, 2);
                if (t != root)
                    proto_item_set_len(t, off - struct_off);
                if (repeats) {
                    --repeats;
                    fidx = fields[old_fidx].field_handle_idx;
                    uidx = old_uidx;
                    t = proto_tree_add_subtree(root, tvb, off, -1, ett_eti[fields[old_fidx].ett_idx], NULL, &struct_names[fields[old_fidx].size]);
                    struct_off = off;
                } else {
                    fidx = old_fidx + 1;
                    t = root;
                    --top;
                }
                break;
            case ETI_VAR_STRUCT:
            case ETI_STRUCT:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].counter_off, <, sizeof counter / sizeof counter[0]);
                repeats = fields[fidx].type == ETI_VAR_STRUCT ? counter[fields[fidx].counter_off] : 1;
                if (repeats) {
                    --repeats;
                    t = proto_tree_add_subtree(root, tvb, off, -1, ett_eti[fields[fidx].ett_idx], NULL, &struct_names[fields[fidx].size]);
                    struct_off = off;
                    old_fidx = fidx;
                    old_uidx = uidx;
                    fidx = fields[fidx].field_handle_idx;
                    DISSECTOR_ASSERT_CMPUINT(top, ==, 1);
                    ++top;
                } else {
                    ++fidx;
                }
                break;
            case ETI_PADDING:
                off += fields[fidx].size;
                ++fidx;
                break;
            case ETI_CHAR:
                proto_tree_add_item(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, ENC_ASCII);
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_STRING:
                {
                    guint8 c = tvb_get_guint8(tvb, off);
                    if (c)
                        proto_tree_add_item(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, ENC_ASCII);
                    else {
                        proto_item *e = proto_tree_add_string(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, "NO_VALUE ('0x00...')");
                        if (!usages[uidx])
                            expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                    }
                }
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_VAR_STRING:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].counter_off, <, sizeof counter / sizeof counter[0]);
                proto_tree_add_item(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, counter[fields[fidx].counter_off], ENC_ASCII);
                off += counter[fields[fidx].counter_off];
                ++fidx;
                ++uidx;
                break;
            case ETI_COUNTER:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].counter_off, <, sizeof counter / sizeof counter[0]);
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].size, <=, 2);
                {
                    switch (fields[fidx].size) {
                        case 1:
                            {
                                guint8 x = tvb_get_guint8(tvb, off);
                                if (x == UINT8_MAX) {
                                    proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xff)");
                                    counter[fields[fidx].counter_off] = 0;
                                } else {
                                    proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu8, x);
                                    if (x > fields[fidx].ett_idx) {
                                        counter[fields[fidx].counter_off] = fields[fidx].ett_idx;
                                        expert_add_info_format(pinfo, e, &ei_eti_counter_overflow, "Counter overflow: %" PRIu8 " > %" PRIu16, x, fields[fidx].ett_idx);
                                    } else {
                                        counter[fields[fidx].counter_off] = x;
                                    }
                                }
                            }
                            break;
                        case 2:
                            {
                                guint16 x = tvb_get_letohs(tvb, off);
                                if (x == UINT16_MAX) {
                                    proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xffff)");
                                    counter[fields[fidx].counter_off] = 0;
                                } else {
                                    proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu16, x);
                                    if (x > fields[fidx].ett_idx) {
                                        counter[fields[fidx].counter_off] = fields[fidx].ett_idx;
                                        expert_add_info_format(pinfo, e, &ei_eti_counter_overflow, "Counter overflow: %" PRIu16 " > %" PRIu16, x, fields[fidx].ett_idx);
                                    } else {
                                        counter[fields[fidx].counter_off] = x;
                                    }
                                }
                            }
                            break;
                    }
                }
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_UINT:
                switch (fields[fidx].size) {
                    case 1:
                        {
                            guint8 x = tvb_get_guint8(tvb, off);
                            if (x == UINT8_MAX) {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xff)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu8, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 2:
                        {
                            guint16 x = tvb_get_letohs(tvb, off);
                            if (x == UINT16_MAX) {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xffff)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu16, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 4:
                        {
                            guint32 x = tvb_get_letohl(tvb, off);
                            if (x == UINT32_MAX) {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xffffffff)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_uint_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu32, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 8:
                        {
                            guint64 x = tvb_get_letoh64(tvb, off);
                            if (x == UINT64_MAX) {
                                proto_item *e = proto_tree_add_uint64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0xffffffffffffffff)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_uint64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIu64, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                }
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_INT:
                switch (fields[fidx].size) {
                    case 1:
                        {
                            gint8 x = tvb_get_gint8(tvb, off);
                            if (x == INT8_MIN) {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0x80)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIi8, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 2:
                        {
                            gint16 x = tvb_get_letohis(tvb, off);
                            if (x == INT16_MIN) {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0x8000)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIi16, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 4:
                        {
                            gint32 x = tvb_get_letohil(tvb, off);
                            if (x == INT32_MIN) {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0x80000000)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_int_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIi32, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                    case 8:
                        {
                            gint64 x = tvb_get_letohi64(tvb, off);
                            if (x == INT64_MIN) {
                                proto_item *e = proto_tree_add_int64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0x8000000000000000)");
                                if (!usages[uidx])
                                    expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                            } else {
                                proto_item *e = proto_tree_add_int64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%" PRIi64, x);
                                if (usages[uidx] == 2)
                                    expert_add_info_format(pinfo, e, &ei_eti_overused, "unused value is set");
                            }
                        }
                        break;
                }
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_UINT_ENUM:
            case ETI_INT_ENUM:
                proto_tree_add_item(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, ENC_LITTLE_ENDIAN);
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_FIXED_POINT:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].size, ==, 8);
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].counter_off, >, 0);
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].counter_off, <=, 16);
                {
                    gint64 x = tvb_get_letohi64(tvb, off);
                    if (x == INT64_MIN) {
                        proto_item *e = proto_tree_add_int64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "NO_VALUE (0x8000000000000000)");
                        if (!usages[uidx])
                            expert_add_info_format(pinfo, e, &ei_eti_missing, "required value is missing");
                    } else {
                        unsigned slack = fields[fidx].counter_off + 1;
                        if (x < 0)
                            slack += 1;
                        char s[21];
                        int n = snprintf(s, sizeof s, "%0*" PRIi64, slack, x);
                        DISSECTOR_ASSERT_CMPUINT(n, >, 0);
                        unsigned k = n - fields[fidx].counter_off;
                        proto_tree_add_int64_format_value(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, x, "%.*s.%s", k, s, s + k);
                    }
                }
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_TIMESTAMP_NS:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].size, ==, 8);
                proto_tree_add_item(t, hf_eti[fields[fidx].field_handle_idx], tvb, off, fields[fidx].size, ENC_LITTLE_ENDIAN | ENC_TIME_NSECS);
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
            case ETI_DSCP:
                DISSECTOR_ASSERT_CMPUINT(fields[fidx].size, ==, 1);
                proto_tree_add_bitmask(t, tvb, off, hf_eti[fields[fidx].field_handle_idx], ett_eti_dscp, dscp_bits, ENC_LITTLE_ENDIAN);
                off += fields[fidx].size;
                ++fidx;
                ++uidx;
                break;
        }
    }

    return tvb_captured_length(tvb);
}

/* determine PDU length of protocol ETI */
static guint
get_eti_message_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset, void *data _U_)
{
    return (guint)tvb_get_letohl(tvb, offset);
}

static int
dissect_eti(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
        void *data)
{
    tcp_dissect_pdus(tvb, pinfo, tree, TRUE, 4 /* bytes to read for bodylen */,
            get_eti_message_len, dissect_eti_message, data);
    return tvb_captured_length(tvb);
}

void
proto_register_eti(void)
{
    static hf_register_info hf[] ={
          { &hf_eti[ACCOUNT_FH_IDX],
              { "Account", "eti.account",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[AFFECTEDORDERREQUESTID_FH_IDX],
              { "AffectedOrderRequestID", "eti.affectedorderrequestid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ALLOCID_FH_IDX],
              { "AllocID", "eti.allocid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ALLOCQTY_FH_IDX],
              { "AllocQty", "eti.allocqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLBEGMSGID_FH_IDX],
              { "ApplBegMsgID", "eti.applbegmsgid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLBEGSEQNUM_FH_IDX],
              { "ApplBegSeqNum", "eti.applbegseqnum",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLENDMSGID_FH_IDX],
              { "ApplEndMsgID", "eti.applendmsgid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLENDSEQNUM_FH_IDX],
              { "ApplEndSeqNum", "eti.applendseqnum",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLID_FH_IDX],
              { "ApplID", "eti.applid",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &appl_id_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLIDSTATUS_FH_IDX],
              { "ApplIDStatus", "eti.applidstatus",
                FT_UINT32, BASE_DEC, VALS(appl_idstatus_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLMSGID_FH_IDX],
              { "ApplMsgID", "eti.applmsgid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLRESENDFLAG_FH_IDX],
              { "ApplResendFlag", "eti.applresendflag",
                FT_UINT8, BASE_DEC, VALS(appl_resend_flag_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLSEQINDICATOR_FH_IDX],
              { "ApplSeqIndicator", "eti.applseqindicator",
                FT_UINT8, BASE_DEC, VALS(appl_seq_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLSEQNUM_FH_IDX],
              { "ApplSeqNum", "eti.applseqnum",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLSEQSTATUS_FH_IDX],
              { "ApplSeqStatus", "eti.applseqstatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLSEQTRADEDATE_FH_IDX],
              { "ApplSeqTradeDate", "eti.applseqtradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLSUBID_FH_IDX],
              { "ApplSubID", "eti.applsubid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLTOTALMESSAGECOUNT_FH_IDX],
              { "ApplTotalMessageCount", "eti.appltotalmessagecount",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLUSAGEORDERS_FH_IDX],
              { "ApplUsageOrders", "eti.applusageorders",
                FT_CHAR, BASE_HEX, VALS(appl_usage_orders_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLUSAGEQUOTES_FH_IDX],
              { "ApplUsageQuotes", "eti.applusagequotes",
                FT_CHAR, BASE_HEX, VALS(appl_usage_orders_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLICATIONSYSTEMNAME_FH_IDX],
              { "ApplicationSystemName", "eti.applicationsystemname",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLICATIONSYSTEMVENDOR_FH_IDX],
              { "ApplicationSystemVendor", "eti.applicationsystemvendor",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[APPLICATIONSYSTEMVERSION_FH_IDX],
              { "ApplicationSystemVersion", "eti.applicationsystemversion",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[AUTOAPPROVALRULEID_FH_IDX],
              { "AutoApprovalRuleID", "eti.autoapprovalruleid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETEXECID_FH_IDX],
              { "BasketExecID", "eti.basketexecid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETPARTYCONTRAFIRM_FH_IDX],
              { "BasketPartyContraFirm", "eti.basketpartycontrafirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETPROFILEID_FH_IDX],
              { "BasketProfileID", "eti.basketprofileid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETSIDETRADEREPORTID_FH_IDX],
              { "BasketSideTradeReportID", "eti.basketsidetradereportid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETTRADEREPORTTEXT_FH_IDX],
              { "BasketTradeReportText", "eti.baskettradereporttext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETTRADEREPORTTYPE_FH_IDX],
              { "BasketTradeReportType", "eti.baskettradereporttype",
                FT_UINT8, BASE_DEC, VALS(basket_trade_report_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BASKETTRDMATCHID_FH_IDX],
              { "BasketTrdMatchID", "eti.baskettrdmatchid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BESTBIDPX_FH_IDX],
              { "BestBidPx", "eti.bestbidpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BESTBIDSIZE_FH_IDX],
              { "BestBidSize", "eti.bestbidsize",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BESTOFFERPX_FH_IDX],
              { "BestOfferPx", "eti.bestofferpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BESTOFFERSIZE_FH_IDX],
              { "BestOfferSize", "eti.bestoffersize",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BIDPX_FH_IDX],
              { "BidPx", "eti.bidpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BIDSIZE_FH_IDX],
              { "BidSize", "eti.bidsize",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[BODYLEN_FH_IDX],
              { "BodyLen", "eti.bodylen",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CLORDID_FH_IDX],
              { "ClOrdID", "eti.clordid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CLEARINGTRADEPRICE_FH_IDX],
              { "ClearingTradePrice", "eti.clearingtradeprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CLEARINGTRADEQTY_FH_IDX],
              { "ClearingTradeQty", "eti.clearingtradeqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[COMPLIANCETEXT_FH_IDX],
              { "ComplianceText", "eti.compliancetext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[COMPRESSIONACTION_FH_IDX],
              { "CompressionAction", "eti.compressionaction",
                FT_UINT8, BASE_DEC, VALS(compression_action_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[COMPRESSIONID_FH_IDX],
              { "CompressionID", "eti.compressionid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[COMPRESSIONSTATUS_FH_IDX],
              { "CompressionStatus", "eti.compressionstatus",
                FT_UINT8, BASE_DEC, VALS(compression_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CROSSID_FH_IDX],
              { "CrossID", "eti.crossid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CROSSREQUESTID_FH_IDX],
              { "CrossRequestID", "eti.crossrequestid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CROSSEDINDICATOR_FH_IDX],
              { "CrossedIndicator", "eti.crossedindicator",
                FT_UINT8, BASE_DEC, VALS(crossed_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CUMQTY_FH_IDX],
              { "CumQty", "eti.cumqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CUSTORDERHANDLINGINST_FH_IDX],
              { "CustOrderHandlingInst", "eti.custorderhandlinginst",
                FT_CHAR, BASE_HEX, VALS(cust_order_handling_inst_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CXLQTY_FH_IDX],
              { "CxlQty", "eti.cxlqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[CXLSIZE_FH_IDX],
              { "CxlSize", "eti.cxlsize",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[DEFAULTCSTMAPPLVERID_FH_IDX],
              { "DefaultCstmApplVerID", "eti.defaultcstmapplverid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[DEFAULTCSTMAPPLVERSUBID_FH_IDX],
              { "DefaultCstmApplVerSubID", "eti.defaultcstmapplversubid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[DELETEREASON_FH_IDX],
              { "DeleteReason", "eti.deletereason",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &delete_reason_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[DELTA_FH_IDX],
              { "Delta", "eti.delta",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EFFECTONBASKET_FH_IDX],
              { "EffectOnBasket", "eti.effectonbasket",
                FT_UINT8, BASE_DEC, VALS(effect_on_basket_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EFFECTIVETIME_FH_IDX],
              { "EffectiveTime", "eti.effectivetime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ENLIGHTRFQAVGRESPRATERANKING_FH_IDX],
              { "EnlightRFQAvgRespRateRanking", "eti.enlightrfqavgresprateranking",
                FT_UINT8, BASE_DEC, VALS(enlight_rfqavg_resp_rate_ranking_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ENLIGHTRFQAVGRESPTIMERANKING_FH_IDX],
              { "EnlightRFQAvgRespTimeRanking", "eti.enlightrfqavgresptimeranking",
                FT_UINT8, BASE_DEC, VALS(enlight_rfqavg_resp_rate_ranking_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ENRICHMENTRULEID_FH_IDX],
              { "EnrichmentRuleID", "eti.enrichmentruleid",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EUREXVOLUMERANKING_FH_IDX],
              { "EurexVolumeRanking", "eti.eurexvolumeranking",
                FT_UINT8, BASE_DEC, VALS(enlight_rfqavg_resp_rate_ranking_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EVENTDATE_FH_IDX],
              { "EventDate", "eti.eventdate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EVENTTYPE_FH_IDX],
              { "EventType", "eti.eventtype",
                FT_UINT8, BASE_DEC, VALS(event_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECID_FH_IDX],
              { "ExecID", "eti.execid",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECINST_FH_IDX],
              { "ExecInst", "eti.execinst",
                FT_UINT8, BASE_DEC, VALS(exec_inst_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECRESTATEMENTREASON_FH_IDX],
              { "ExecRestatementReason", "eti.execrestatementreason",
                FT_UINT16, BASE_DEC| BASE_EXT_STRING, &exec_restatement_reason_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECTYPE_FH_IDX],
              { "ExecType", "eti.exectype",
                FT_CHAR, BASE_HEX| BASE_EXT_STRING, &exec_type_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECUTINGTRADER_FH_IDX],
              { "ExecutingTrader", "eti.executingtrader",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXECUTINGTRADERQUALIFIER_FH_IDX],
              { "ExecutingTraderQualifier", "eti.executingtraderqualifier",
                FT_UINT8, BASE_DEC, VALS(executing_trader_qualifier_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXERCISESTYLE_FH_IDX],
              { "ExerciseStyle", "eti.exercisestyle",
                FT_UINT8, BASE_DEC, VALS(exercise_style_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXPIREDATE_FH_IDX],
              { "ExpireDate", "eti.expiredate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXPIRETIME_FH_IDX],
              { "ExpireTime", "eti.expiretime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[EXPOSUREDURATION_FH_IDX],
              { "ExposureDuration", "eti.exposureduration",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIXCLORDID_FH_IDX],
              { "FIXClOrdID", "eti.fixclordid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIXENGINENAME_FH_IDX],
              { "FIXEngineName", "eti.fixenginename",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIXENGINEVENDOR_FH_IDX],
              { "FIXEngineVendor", "eti.fixenginevendor",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIXENGINEVERSION_FH_IDX],
              { "FIXEngineVersion", "eti.fixengineversion",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLEXECID_FH_IDX],
              { "FillExecID", "eti.fillexecid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLLIQUIDITYIND_FH_IDX],
              { "FillLiquidityInd", "eti.fillliquidityind",
                FT_UINT8, BASE_DEC, VALS(fill_liquidity_ind_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLMATCHID_FH_IDX],
              { "FillMatchID", "eti.fillmatchid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLPX_FH_IDX],
              { "FillPx", "eti.fillpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLQTY_FH_IDX],
              { "FillQty", "eti.fillqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FILLREFID_FH_IDX],
              { "FillRefID", "eti.fillrefid",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIRMNEGOTIATIONID_FH_IDX],
              { "FirmNegotiationID", "eti.firmnegotiationid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FIRMTRADEID_FH_IDX],
              { "FirmTradeID", "eti.firmtradeid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FREETEXT1_FH_IDX],
              { "FreeText1", "eti.freetext1",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FREETEXT2_FH_IDX],
              { "FreeText2", "eti.freetext2",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FREETEXT3_FH_IDX],
              { "FreeText3", "eti.freetext3",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FREETEXT5_FH_IDX],
              { "FreeText5", "eti.freetext5",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[FREETEXT5DISCLOSUREINSTRUCTION_FH_IDX],
              { "FreeText5DisclosureInstruction", "eti.freetext5disclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[HEADLINE_FH_IDX],
              { "Headline", "eti.headline",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[HEARTBTINT_FH_IDX],
              { "HeartBtInt", "eti.heartbtint",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[HEDGETYPE_FH_IDX],
              { "HedgeType", "eti.hedgetype",
                FT_UINT8, BASE_DEC, VALS(hedge_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[HEDGINGINSTRUCTION_FH_IDX],
              { "HedgingInstruction", "eti.hedginginstruction",
                FT_UINT8, BASE_DEC, VALS(hedging_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[HIGHLIMITPRICE_FH_IDX],
              { "HighLimitPrice", "eti.highlimitprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[IMPLIEDMARKETINDICATOR_FH_IDX],
              { "ImpliedMarketIndicator", "eti.impliedmarketindicator",
                FT_UINT8, BASE_DEC, VALS(implied_market_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[INDIVIDUALALLOCID_FH_IDX],
              { "IndividualAllocID", "eti.individualallocid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[INPUTSOURCE_FH_IDX],
              { "InputSource", "eti.inputsource",
                FT_UINT8, BASE_DEC, VALS(input_source_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[INSTRATTRIBTYPE_FH_IDX],
              { "InstrAttribType", "eti.instrattribtype",
                FT_UINT8, BASE_DEC, VALS(instr_attrib_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[INSTRATTRIBVALUE_FH_IDX],
              { "InstrAttribValue", "eti.instrattribvalue",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[INSTRMTMATCHSIDEID_FH_IDX],
              { "InstrmtMatchSideID", "eti.instrmtmatchsideid",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTENTITYPROCESSED_FH_IDX],
              { "LastEntityProcessed", "eti.lastentityprocessed",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTFRAGMENT_FH_IDX],
              { "LastFragment", "eti.lastfragment",
                FT_UINT8, BASE_DEC, VALS(last_fragment_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTPX_FH_IDX],
              { "LastPx", "eti.lastpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTPXDISCLOSUREINSTRUCTION_FH_IDX],
              { "LastPxDisclosureInstruction", "eti.lastpxdisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTQTY_FH_IDX],
              { "LastQty", "eti.lastqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTQTYDISCLOSUREINSTRUCTION_FH_IDX],
              { "LastQtyDisclosureInstruction", "eti.lastqtydisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LASTUPDATETIME_FH_IDX],
              { "LastUpdateTime", "eti.lastupdatetime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEAVESQTY_FH_IDX],
              { "LeavesQty", "eti.leavesqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEAVESQTYDISCLOSUREINSTRUCTION_FH_IDX],
              { "LeavesQtyDisclosureInstruction", "eti.leavesqtydisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGACCOUNT_FH_IDX],
              { "LegAccount", "eti.legaccount",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGEXECID_FH_IDX],
              { "LegExecID", "eti.legexecid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGINPUTSOURCE_FH_IDX],
              { "LegInputSource", "eti.leginputsource",
                FT_UINT8, BASE_DEC, VALS(input_source_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGLASTPX_FH_IDX],
              { "LegLastPx", "eti.leglastpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGLASTQTY_FH_IDX],
              { "LegLastQty", "eti.leglastqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGPOSITIONEFFECT_FH_IDX],
              { "LegPositionEffect", "eti.legpositioneffect",
                FT_CHAR, BASE_HEX, VALS(leg_position_effect_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGPRICE_FH_IDX],
              { "LegPrice", "eti.legprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGQTY_FH_IDX],
              { "LegQty", "eti.legqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGRATIOQTY_FH_IDX],
              { "LegRatioQty", "eti.legratioqty",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGSECURITYID_FH_IDX],
              { "LegSecurityID", "eti.legsecurityid",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGSECURITYTYPE_FH_IDX],
              { "LegSecurityType", "eti.legsecuritytype",
                FT_UINT8, BASE_DEC, VALS(leg_security_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGSIDE_FH_IDX],
              { "LegSide", "eti.legside",
                FT_UINT8, BASE_DEC, VALS(leg_side_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LEGSYMBOL_FH_IDX],
              { "LegSymbol", "eti.legsymbol",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LISTUPDATEACTION_FH_IDX],
              { "ListUpdateAction", "eti.listupdateaction",
                FT_CHAR, BASE_HEX, VALS(list_update_action_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[LOWLIMITPRICE_FH_IDX],
              { "LowLimitPrice", "eti.lowlimitprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MDBOOKTYPE_FH_IDX],
              { "MDBookType", "eti.mdbooktype",
                FT_UINT8, BASE_DEC, VALS(mdbook_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MDSUBBOOKTYPE_FH_IDX],
              { "MDSubBookType", "eti.mdsubbooktype",
                FT_UINT8, BASE_DEC, VALS(mdsub_book_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MMPARAMETERREPORTID_FH_IDX],
              { "MMParameterReportID", "eti.mmparameterreportid",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MARKETID_FH_IDX],
              { "MarketID", "eti.marketid",
                FT_UINT16, BASE_DEC, VALS(market_id_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MARKETSEGMENTID_FH_IDX],
              { "MarketSegmentID", "eti.marketsegmentid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MASSACTIONREASON_FH_IDX],
              { "MassActionReason", "eti.massactionreason",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &mass_action_reason_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MASSACTIONREPORTID_FH_IDX],
              { "MassActionReportID", "eti.massactionreportid",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MASSACTIONSUBTYPE_FH_IDX],
              { "MassActionSubType", "eti.massactionsubtype",
                FT_UINT8, BASE_DEC, VALS(mass_action_sub_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MASSACTIONTYPE_FH_IDX],
              { "MassActionType", "eti.massactiontype",
                FT_UINT8, BASE_DEC, VALS(mass_action_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHDATE_FH_IDX],
              { "MatchDate", "eti.matchdate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHINSTCROSSID_FH_IDX],
              { "MatchInstCrossID", "eti.matchinstcrossid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHSUBTYPE_FH_IDX],
              { "MatchSubType", "eti.matchsubtype",
                FT_UINT8, BASE_DEC, VALS(match_sub_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHTYPE_FH_IDX],
              { "MatchType", "eti.matchtype",
                FT_UINT8, BASE_DEC, VALS(match_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHINGENGINESTATUS_FH_IDX],
              { "MatchingEngineStatus", "eti.matchingenginestatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATCHINGENGINETRADEDATE_FH_IDX],
              { "MatchingEngineTradeDate", "eti.matchingenginetradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATURITYDATE_FH_IDX],
              { "MaturityDate", "eti.maturitydate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MATURITYMONTHYEAR_FH_IDX],
              { "MaturityMonthYear", "eti.maturitymonthyear",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MAXIMUMPRICE_FH_IDX],
              { "MaximumPrice", "eti.maximumprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MESSAGEEVENTSOURCE_FH_IDX],
              { "MessageEventSource", "eti.messageeventsource",
                FT_CHAR, BASE_HEX, VALS(message_event_source_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MSGSEQNUM_FH_IDX],
              { "MsgSeqNum", "eti.msgseqnum",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MULTILEGREPORTINGTYPE_FH_IDX],
              { "MultiLegReportingType", "eti.multilegreportingtype",
                FT_UINT8, BASE_DEC, VALS(multi_leg_reporting_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MULTILEGMODEL_FH_IDX],
              { "MultilegModel", "eti.multilegmodel",
                FT_UINT8, BASE_DEC, VALS(multileg_model_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[MULTILEGPRICEMODEL_FH_IDX],
              { "MultilegPriceModel", "eti.multilegpricemodel",
                FT_UINT8, BASE_DEC, VALS(multileg_price_model_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NEGOTIATIONID_FH_IDX],
              { "NegotiationID", "eti.negotiationid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NEGOTIATIONSTARTTIME_FH_IDX],
              { "NegotiationStartTime", "eti.negotiationstarttime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NETWORKMSGID_FH_IDX],
              { "NetworkMsgID", "eti.networkmsgid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOAFFECTEDORDERREQUESTS_FH_IDX],
              { "NoAffectedOrderRequests", "eti.noaffectedorderrequests",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOBASKETROOTPARTYGRPS_FH_IDX],
              { "NoBasketRootPartyGrps", "eti.nobasketrootpartygrps",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOBASKETROOTPARTYGRPSBC_FH_IDX],
              { "NoBasketRootPartyGrpsBC", "eti.nobasketrootpartygrpsbc",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOBASKETSIDEALLOC_FH_IDX],
              { "NoBasketSideAlloc", "eti.nobasketsidealloc",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOCROSSLEGS_FH_IDX],
              { "NoCrossLegs", "eti.nocrosslegs",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOENRICHMENTRULES_FH_IDX],
              { "NoEnrichmentRules", "eti.noenrichmentrules",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOEVENTS_FH_IDX],
              { "NoEvents", "eti.noevents",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOFILLS_FH_IDX],
              { "NoFills", "eti.nofills",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOINSTRATTRIB_FH_IDX],
              { "NoInstrAttrib", "eti.noinstrattrib",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOINSTRMTMATCHSIDES_FH_IDX],
              { "NoInstrmtMatchSides", "eti.noinstrmtmatchsides",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOLEGEXECS_FH_IDX],
              { "NoLegExecs", "eti.nolegexecs",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOLEGS_FH_IDX],
              { "NoLegs", "eti.nolegs",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOMMPARAMETERS_FH_IDX],
              { "NoMMParameters", "eti.nommparameters",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NONOTAFFECTEDORDERS_FH_IDX],
              { "NoNotAffectedOrders", "eti.nonotaffectedorders",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NONOTAFFECTEDSECURITIES_FH_IDX],
              { "NoNotAffectedSecurities", "eti.nonotaffectedsecurities",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOORDERBOOKITEMS_FH_IDX],
              { "NoOrderBookItems", "eti.noorderbookitems",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOPARTYDETAILS_FH_IDX],
              { "NoPartyDetails", "eti.nopartydetails",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOQUOTEENTRIES_FH_IDX],
              { "NoQuoteEntries", "eti.noquoteentries",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOQUOTEEVENTS_FH_IDX],
              { "NoQuoteEvents", "eti.noquoteevents",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOQUOTEEVENTSINDEX_FH_IDX],
              { "NoQuoteEventsIndex", "eti.noquoteeventsindex",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOQUOTESIDEENTRIES_FH_IDX],
              { "NoQuoteSideEntries", "eti.noquotesideentries",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NORISKLIMITS_FH_IDX],
              { "NoRiskLimits", "eti.norisklimits",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NORISKLIMITSQTY_FH_IDX],
              { "NoRiskLimitsQty", "eti.norisklimitsqty",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOSRQSRELATEDTRADEIDS_FH_IDX],
              { "NoSRQSRelatedTradeIDs", "eti.nosrqsrelatedtradeids",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOSESSIONS_FH_IDX],
              { "NoSessions", "eti.nosessions",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOSIDEALLOCS_FH_IDX],
              { "NoSideAllocs", "eti.nosideallocs",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOSIDES_FH_IDX],
              { "NoSides", "eti.nosides",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOTARGETPARTYIDS_FH_IDX],
              { "NoTargetPartyIDs", "eti.notargetpartyids",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOUNDERLYINGSTIPS_FH_IDX],
              { "NoUnderlyingStips", "eti.nounderlyingstips",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOTAFFORIGCLORDID_FH_IDX],
              { "NotAffOrigClOrdID", "eti.notafforigclordid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOTAFFECTEDORDERID_FH_IDX],
              { "NotAffectedOrderID", "eti.notaffectedorderid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOTAFFECTEDSECURITYID_FH_IDX],
              { "NotAffectedSecurityID", "eti.notaffectedsecurityid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NOTIFICATIONIN_FH_IDX],
              { "NotificationIn", "eti.notificationin",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NUMBEROFRESPDISCLOSUREINSTRUCTION_FH_IDX],
              { "NumberOfRespDisclosureInstruction", "eti.numberofrespdisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NUMBEROFRESPONDENTS_FH_IDX],
              { "NumberOfRespondents", "eti.numberofrespondents",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[NUMBEROFSECURITIES_FH_IDX],
              { "NumberOfSecurities", "eti.numberofsecurities",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[OFFERPX_FH_IDX],
              { "OfferPx", "eti.offerpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[OFFERSIZE_FH_IDX],
              { "OfferSize", "eti.offersize",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[OPTATTRIBUTE_FH_IDX],
              { "OptAttribute", "eti.optattribute",
                FT_UINT8, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDSTATUS_FH_IDX],
              { "OrdStatus", "eti.ordstatus",
                FT_CHAR, BASE_HEX, VALS(ord_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDTYPE_FH_IDX],
              { "OrdType", "eti.ordtype",
                FT_UINT8, BASE_DEC, VALS(ord_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERATTRIBUTELIQUIDITYPROVISION_FH_IDX],
              { "OrderAttributeLiquidityProvision", "eti.orderattributeliquidityprovision",
                FT_UINT8, BASE_DEC, VALS(order_attribute_liquidity_provision_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERATTRIBUTERISKREDUCTION_FH_IDX],
              { "OrderAttributeRiskReduction", "eti.orderattributeriskreduction",
                FT_UINT8, BASE_DEC, VALS(order_attribute_liquidity_provision_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERCATEGORY_FH_IDX],
              { "OrderCategory", "eti.ordercategory",
                FT_CHAR, BASE_HEX, VALS(order_category_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERID_FH_IDX],
              { "OrderID", "eti.orderid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERORIGINATION_FH_IDX],
              { "OrderOrigination", "eti.orderorigination",
                FT_UINT8, BASE_DEC, VALS(order_origination_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERQTY_FH_IDX],
              { "OrderQty", "eti.orderqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERQTYDISCLOSUREINSTRUCTION_FH_IDX],
              { "OrderQtyDisclosureInstruction", "eti.orderqtydisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERROUTINGINDICATOR_FH_IDX],
              { "OrderRoutingIndicator", "eti.orderroutingindicator",
                FT_CHAR, BASE_HEX, VALS(order_routing_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORDERSIDE_FH_IDX],
              { "OrderSide", "eti.orderside",
                FT_UINT8, BASE_DEC, VALS(leg_side_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORIGCLORDID_FH_IDX],
              { "OrigClOrdID", "eti.origclordid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORIGTIME_FH_IDX],
              { "OrigTime", "eti.origtime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ORIGTRADEID_FH_IDX],
              { "OrigTradeID", "eti.origtradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[OWNERSHIPINDICATOR_FH_IDX],
              { "OwnershipIndicator", "eti.ownershipindicator",
                FT_UINT8, BASE_DEC, VALS(ownership_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PACKAGEID_FH_IDX],
              { "PackageID", "eti.packageid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTITIONID_FH_IDX],
              { "PartitionID", "eti.partitionid",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYACTIONTYPE_FH_IDX],
              { "PartyActionType", "eti.partyactiontype",
                FT_UINT8, BASE_DEC, VALS(party_action_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILDESKID_FH_IDX],
              { "PartyDetailDeskID", "eti.partydetaildeskid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILEXECUTINGTRADER_FH_IDX],
              { "PartyDetailExecutingTrader", "eti.partydetailexecutingtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILEXECUTINGUNIT_FH_IDX],
              { "PartyDetailExecutingUnit", "eti.partydetailexecutingunit",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILIDEXECUTINGTRADER_FH_IDX],
              { "PartyDetailIDExecutingTrader", "eti.partydetailidexecutingtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILIDEXECUTINGUNIT_FH_IDX],
              { "PartyDetailIDExecutingUnit", "eti.partydetailidexecutingunit",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILROLEQUALIFIER_FH_IDX],
              { "PartyDetailRoleQualifier", "eti.partydetailrolequalifier",
                FT_UINT8, BASE_DEC, VALS(party_detail_role_qualifier_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILSTATUS_FH_IDX],
              { "PartyDetailStatus", "eti.partydetailstatus",
                FT_UINT8, BASE_DEC, VALS(party_detail_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYDETAILSTATUSINFORMATION_FH_IDX],
              { "PartyDetailStatusInformation", "eti.partydetailstatusinformation",
                FT_UINT8, BASE_DEC, VALS(party_detail_status_information_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYENTERINGTRADER_FH_IDX],
              { "PartyEnteringTrader", "eti.partyenteringtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYEXECUTINGFIRM_FH_IDX],
              { "PartyExecutingFirm", "eti.partyexecutingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYEXECUTINGTRADER_FH_IDX],
              { "PartyExecutingTrader", "eti.partyexecutingtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYEXECUTINGUNIT_FH_IDX],
              { "PartyExecutingUnit", "eti.partyexecutingunit",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDBENEFICIARY_FH_IDX],
              { "PartyIDBeneficiary", "eti.partyidbeneficiary",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDCLIENTID_FH_IDX],
              { "PartyIDClientID", "eti.partyidclientid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDENTERINGFIRM_FH_IDX],
              { "PartyIDEnteringFirm", "eti.partyidenteringfirm",
                FT_UINT8, BASE_DEC, VALS(party_identering_firm_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDENTERINGTRADER_FH_IDX],
              { "PartyIDEnteringTrader", "eti.partyidenteringtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDEXECUTINGTRADER_FH_IDX],
              { "PartyIDExecutingTrader", "eti.partyidexecutingtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDEXECUTINGUNIT_FH_IDX],
              { "PartyIDExecutingUnit", "eti.partyidexecutingunit",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDLOCATIONID_FH_IDX],
              { "PartyIDLocationID", "eti.partyidlocationid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDORDERORIGINATIONFIRM_FH_IDX],
              { "PartyIDOrderOriginationFirm", "eti.partyidorderoriginationfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDORIGINATIONMARKET_FH_IDX],
              { "PartyIDOriginationMarket", "eti.partyidoriginationmarket",
                FT_UINT8, BASE_DEC, VALS(party_idorigination_market_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDPOSITIONACCOUNT_FH_IDX],
              { "PartyIDPositionAccount", "eti.partyidpositionaccount",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDSESSIONID_FH_IDX],
              { "PartyIDSessionID", "eti.partyidsessionid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDSETTLEMENTLOCATION_FH_IDX],
              { "PartyIDSettlementLocation", "eti.partyidsettlementlocation",
                FT_UINT8, BASE_DEC, VALS(party_idsettlement_location_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDTAKEUPTRADINGFIRM_FH_IDX],
              { "PartyIDTakeUpTradingFirm", "eti.partyidtakeuptradingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDINVESTMENTDECISIONMAKER_FH_IDX],
              { "PartyIdInvestmentDecisionMaker", "eti.partyidinvestmentdecisionmaker",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX],
              { "PartyIdInvestmentDecisionMakerQualifier", "eti.partyidinvestmentdecisionmakerqualifier",
                FT_UINT8, BASE_DEC, VALS(executing_trader_qualifier_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYORDERORIGINATIONDISCLOSUREINSTRUCTION_FH_IDX],
              { "PartyOrderOriginationDisclosureInstruction", "eti.partyorderoriginationdisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYORDERORIGINATIONTRADER_FH_IDX],
              { "PartyOrderOriginationTrader", "eti.partyorderoriginationtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PARTYSUBIDTYPE_FH_IDX],
              { "PartySubIDType", "eti.partysubidtype",
                FT_UINT16, BASE_DEC, VALS(party_sub_idtype_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PASSWORD_FH_IDX],
              { "Password", "eti.password",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PCTCOUNT_FH_IDX],
              { "PctCount", "eti.pctcount",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[POSITIONEFFECT_FH_IDX],
              { "PositionEffect", "eti.positioneffect",
                FT_CHAR, BASE_HEX, VALS(leg_position_effect_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PRICE_FH_IDX],
              { "Price", "eti.price",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PRICEDISCLOSUREINSTRUCTION_FH_IDX],
              { "PriceDisclosureInstruction", "eti.pricedisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PRICEVALIDITYCHECKTYPE_FH_IDX],
              { "PriceValidityCheckType", "eti.pricevaliditychecktype",
                FT_UINT8, BASE_DEC, VALS(price_validity_check_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PRODUCTCOMPLEX_FH_IDX],
              { "ProductComplex", "eti.productcomplex",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &product_complex_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[PUTORCALL_FH_IDX],
              { "PutOrCall", "eti.putorcall",
                FT_UINT8, BASE_DEC, VALS(put_or_call_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTECANCELTYPE_FH_IDX],
              { "QuoteCancelType", "eti.quotecanceltype",
                FT_UINT8, BASE_DEC, VALS(quote_cancel_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEENTRYREJECTREASON_FH_IDX],
              { "QuoteEntryRejectReason", "eti.quoteentryrejectreason",
                FT_UINT32, BASE_DEC| BASE_EXT_STRING, &quote_entry_reject_reason_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEENTRYSTATUS_FH_IDX],
              { "QuoteEntryStatus", "eti.quoteentrystatus",
                FT_UINT8, BASE_DEC, VALS(quote_entry_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTEXECID_FH_IDX],
              { "QuoteEventExecID", "eti.quoteeventexecid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTLIQUIDITYIND_FH_IDX],
              { "QuoteEventLiquidityInd", "eti.quoteeventliquidityind",
                FT_UINT8, BASE_DEC, VALS(quote_event_liquidity_ind_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTMATCHID_FH_IDX],
              { "QuoteEventMatchID", "eti.quoteeventmatchid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTPX_FH_IDX],
              { "QuoteEventPx", "eti.quoteeventpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTQTY_FH_IDX],
              { "QuoteEventQty", "eti.quoteeventqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTREASON_FH_IDX],
              { "QuoteEventReason", "eti.quoteeventreason",
                FT_UINT8, BASE_DEC, VALS(quote_event_reason_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTSIDE_FH_IDX],
              { "QuoteEventSide", "eti.quoteeventside",
                FT_UINT8, BASE_DEC, VALS(leg_side_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEEVENTTYPE_FH_IDX],
              { "QuoteEventType", "eti.quoteeventtype",
                FT_UINT8, BASE_DEC, VALS(quote_event_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEID_FH_IDX],
              { "QuoteID", "eti.quoteid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEINSTRUCTION_FH_IDX],
              { "QuoteInstruction", "eti.quoteinstruction",
                FT_UINT8, BASE_DEC, VALS(quote_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEMSGID_FH_IDX],
              { "QuoteMsgID", "eti.quotemsgid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEREFPRICE_FH_IDX],
              { "QuoteRefPrice", "eti.quoterefprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTEREQID_FH_IDX],
              { "QuoteReqID", "eti.quotereqid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTERESPONSEID_FH_IDX],
              { "QuoteResponseID", "eti.quoteresponseid",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTESIZETYPE_FH_IDX],
              { "QuoteSizeType", "eti.quotesizetype",
                FT_UINT8, BASE_DEC, VALS(quote_size_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTESTATUS_FH_IDX],
              { "QuoteStatus", "eti.quotestatus",
                FT_UINT8, BASE_DEC, VALS(quote_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTESUBTYPE_FH_IDX],
              { "QuoteSubType", "eti.quotesubtype",
                FT_UINT8, BASE_DEC, VALS(quote_sub_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTETYPE_FH_IDX],
              { "QuoteType", "eti.quotetype",
                FT_UINT8, BASE_DEC, VALS(quote_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[QUOTINGSTATUS_FH_IDX],
              { "QuotingStatus", "eti.quotingstatus",
                FT_UINT8, BASE_DEC, VALS(quoting_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REFAPPLID_FH_IDX],
              { "RefApplID", "eti.refapplid",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &appl_id_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REFAPPLLASTMSGID_FH_IDX],
              { "RefApplLastMsgID", "eti.refappllastmsgid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REFAPPLLASTSEQNUM_FH_IDX],
              { "RefApplLastSeqNum", "eti.refappllastseqnum",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REFAPPLSUBID_FH_IDX],
              { "RefApplSubID", "eti.refapplsubid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REGULATORYTRADEID_FH_IDX],
              { "RegulatoryTradeID", "eti.regulatorytradeid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDCLOSEPRICE_FH_IDX],
              { "RelatedClosePrice", "eti.relatedcloseprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDMARKETSEGMENTID_FH_IDX],
              { "RelatedMarketSegmentID", "eti.relatedmarketsegmentid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDPRODUCTCOMPLEX_FH_IDX],
              { "RelatedProductComplex", "eti.relatedproductcomplex",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &related_product_complex_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDSECURITYID_FH_IDX],
              { "RelatedSecurityID", "eti.relatedsecurityid",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDSYMBOL_FH_IDX],
              { "RelatedSymbol", "eti.relatedsymbol",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDTRADEID_FH_IDX],
              { "RelatedTradeID", "eti.relatedtradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RELATEDTRADEQUANTITY_FH_IDX],
              { "RelatedTradeQuantity", "eti.relatedtradequantity",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTTIME_FH_IDX],
              { "RequestTime", "eti.requesttime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYCLEARINGFIRM_FH_IDX],
              { "RequestingPartyClearingFirm", "eti.requestingpartyclearingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYENTERINGFIRM_FH_IDX],
              { "RequestingPartyEnteringFirm", "eti.requestingpartyenteringfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYIDENTERINGFIRM_FH_IDX],
              { "RequestingPartyIDEnteringFirm", "eti.requestingpartyidenteringfirm",
                FT_UINT8, BASE_DEC, VALS(party_identering_firm_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYIDEXECUTINGSYSTEM_FH_IDX],
              { "RequestingPartyIDExecutingSystem", "eti.requestingpartyidexecutingsystem",
                FT_UINT32, BASE_DEC, VALS(requesting_party_idexecuting_system_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYIDEXECUTINGTRADER_FH_IDX],
              { "RequestingPartyIDExecutingTrader", "eti.requestingpartyidexecutingtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REQUESTINGPARTYSUBIDTYPE_FH_IDX],
              { "RequestingPartySubIDType", "eti.requestingpartysubidtype",
                FT_UINT16, BASE_DEC, VALS(requesting_party_sub_idtype_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RESPONDENTTYPE_FH_IDX],
              { "RespondentType", "eti.respondenttype",
                FT_UINT8, BASE_DEC, VALS(respondent_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RESPONSEIN_FH_IDX],
              { "ResponseIn", "eti.responsein",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REVERSALAPPROVALTIME_FH_IDX],
              { "ReversalApprovalTime", "eti.reversalapprovaltime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REVERSALCANCELLATIONREASON_FH_IDX],
              { "ReversalCancellationReason", "eti.reversalcancellationreason",
                FT_UINT8, BASE_DEC, VALS(reversal_cancellation_reason_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REVERSALINDICATOR_FH_IDX],
              { "ReversalIndicator", "eti.reversalindicator",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REVERSALINITIATIONTIME_FH_IDX],
              { "ReversalInitiationTime", "eti.reversalinitiationtime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[REVERSALREASONTEXT_FH_IDX],
              { "ReversalReasonText", "eti.reversalreasontext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITACTION_FH_IDX],
              { "RiskLimitAction", "eti.risklimitaction",
                FT_UINT8, BASE_DEC, VALS(risk_limit_action_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITGROUP_FH_IDX],
              { "RiskLimitGroup", "eti.risklimitgroup",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITNETPOSITIONQTY_FH_IDX],
              { "RiskLimitNetPositionQty", "eti.risklimitnetpositionqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITOPENQTY_FH_IDX],
              { "RiskLimitOpenQty", "eti.risklimitopenqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITPLATFORM_FH_IDX],
              { "RiskLimitPlatform", "eti.risklimitplatform",
                FT_UINT8, BASE_DEC, VALS(risk_limit_platform_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITQTY_FH_IDX],
              { "RiskLimitQty", "eti.risklimitqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITREPORTID_FH_IDX],
              { "RiskLimitReportID", "eti.risklimitreportid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITREQUESTINGPARTYROLE_FH_IDX],
              { "RiskLimitRequestingPartyRole", "eti.risklimitrequestingpartyrole",
                FT_UINT8, BASE_DEC, VALS(risk_limit_requesting_party_role_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITTYPE_FH_IDX],
              { "RiskLimitType", "eti.risklimittype",
                FT_UINT8, BASE_DEC, VALS(risk_limit_type_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[RISKLIMITVIOLATIONINDICATOR_FH_IDX],
              { "RiskLimitViolationIndicator", "eti.risklimitviolationindicator",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYCLEARINGFIRM_FH_IDX],
              { "RootPartyClearingFirm", "eti.rootpartyclearingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYCLEARINGORGANIZATION_FH_IDX],
              { "RootPartyClearingOrganization", "eti.rootpartyclearingorganization",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYCONTRAFIRM_FH_IDX],
              { "RootPartyContraFirm", "eti.rootpartycontrafirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYCONTRATRADER_FH_IDX],
              { "RootPartyContraTrader", "eti.rootpartycontratrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYENTERINGTRADER_FH_IDX],
              { "RootPartyEnteringTrader", "eti.rootpartyenteringtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYEXECUTINGFIRM_FH_IDX],
              { "RootPartyExecutingFirm", "eti.rootpartyexecutingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYEXECUTINGTRADER_FH_IDX],
              { "RootPartyExecutingTrader", "eti.rootpartyexecutingtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDBENEFICIARY_FH_IDX],
              { "RootPartyIDBeneficiary", "eti.rootpartyidbeneficiary",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDCLEARINGUNIT_FH_IDX],
              { "RootPartyIDClearingUnit", "eti.rootpartyidclearingunit",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDCLIENTID_FH_IDX],
              { "RootPartyIDClientID", "eti.rootpartyidclientid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDEXECUTINGTRADER_FH_IDX],
              { "RootPartyIDExecutingTrader", "eti.rootpartyidexecutingtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDEXECUTINGUNIT_FH_IDX],
              { "RootPartyIDExecutingUnit", "eti.rootpartyidexecutingunit",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDEXECUTIONVENUE_FH_IDX],
              { "RootPartyIDExecutionVenue", "eti.rootpartyidexecutionvenue",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDINVESTMENTDECISIONMAKER_FH_IDX],
              { "RootPartyIDInvestmentDecisionMaker", "eti.rootpartyidinvestmentdecisionmaker",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDINVESTMENTDECISIONMAKERQUALIFIER_FH_IDX],
              { "RootPartyIDInvestmentDecisionMakerQualifier", "eti.rootpartyidinvestmentdecisionmakerqualifier",
                FT_UINT8, BASE_DEC, VALS(executing_trader_qualifier_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDORDERORIGINATIONFIRM_FH_IDX],
              { "RootPartyIDOrderOriginationFirm", "eti.rootpartyidorderoriginationfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDPOSITIONACCOUNT_FH_IDX],
              { "RootPartyIDPositionAccount", "eti.rootpartyidpositionaccount",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDSESSIONID_FH_IDX],
              { "RootPartyIDSessionID", "eti.rootpartyidsessionid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYIDTAKEUPTRADINGFIRM_FH_IDX],
              { "RootPartyIDTakeUpTradingFirm", "eti.rootpartyidtakeuptradingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[ROOTPARTYSUBIDTYPE_FH_IDX],
              { "RootPartySubIDType", "eti.rootpartysubidtype",
                FT_UINT16, BASE_DEC, VALS(root_party_sub_idtype_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SRQSRELATEDTRADEID_FH_IDX],
              { "SRQSRelatedTradeID", "eti.srqsrelatedtradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SECONDARYQUOTEID_FH_IDX],
              { "SecondaryQuoteID", "eti.secondaryquoteid",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SECONDARYTRADEID_FH_IDX],
              { "SecondaryTradeID", "eti.secondarytradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SECURITYID_FH_IDX],
              { "SecurityID", "eti.securityid",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SECURITYRESPONSEID_FH_IDX],
              { "SecurityResponseID", "eti.securityresponseid",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SECURITYSUBTYPE_FH_IDX],
              { "SecuritySubType", "eti.securitysubtype",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SELECTIVEREQUESTFORQUOTESERVICESTATUS_FH_IDX],
              { "SelectiveRequestForQuoteServiceStatus", "eti.selectiverequestforquoteservicestatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SELECTIVEREQUESTFORQUOTESERVICETRADEDATE_FH_IDX],
              { "SelectiveRequestForQuoteServiceTradeDate", "eti.selectiverequestforquoteservicetradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SENDERSUBID_FH_IDX],
              { "SenderSubID", "eti.sendersubid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SENDINGTIME_FH_IDX],
              { "SendingTime", "eti.sendingtime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SESSIONINSTANCEID_FH_IDX],
              { "SessionInstanceID", "eti.sessioninstanceid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SESSIONMODE_FH_IDX],
              { "SessionMode", "eti.sessionmode",
                FT_UINT8, BASE_DEC, VALS(session_mode_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SESSIONREJECTREASON_FH_IDX],
              { "SessionRejectReason", "eti.sessionrejectreason",
                FT_UINT32, BASE_DEC| BASE_EXT_STRING, &session_reject_reason_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SESSIONSTATUS_FH_IDX],
              { "SessionStatus", "eti.sessionstatus",
                FT_UINT8, BASE_DEC, VALS(session_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SESSIONSUBMODE_FH_IDX],
              { "SessionSubMode", "eti.sessionsubmode",
                FT_UINT8, BASE_DEC, VALS(session_sub_mode_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SETTLMETHOD_FH_IDX],
              { "SettlMethod", "eti.settlmethod",
                FT_CHAR, BASE_HEX, VALS(settl_method_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SHOWLASTDEALONCLOSURE_FH_IDX],
              { "ShowLastDealOnClosure", "eti.showlastdealonclosure",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDE_FH_IDX],
              { "Side", "eti.side",
                FT_UINT8, BASE_DEC, VALS(leg_side_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDECOMPLIANCETEXT_FH_IDX],
              { "SideComplianceText", "eti.sidecompliancetext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDEDISCLOSUREINSTRUCTION_FH_IDX],
              { "SideDisclosureInstruction", "eti.sidedisclosureinstruction",
                FT_UINT8, BASE_DEC, VALS(free_text5_disclosure_instruction_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDELASTPX_FH_IDX],
              { "SideLastPx", "eti.sidelastpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDELASTQTY_FH_IDX],
              { "SideLastQty", "eti.sidelastqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDELIQUIDITYIND_FH_IDX],
              { "SideLiquidityInd", "eti.sideliquidityind",
                FT_UINT8, BASE_DEC, VALS(quote_event_liquidity_ind_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDEMARKETSEGMENTID_FH_IDX],
              { "SideMarketSegmentID", "eti.sidemarketsegmentid",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDETRADEID_FH_IDX],
              { "SideTradeID", "eti.sidetradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIDETRDSUBTYP_FH_IDX],
              { "SideTrdSubTyp", "eti.sidetrdsubtyp",
                FT_UINT16, BASE_DEC, VALS(side_trd_sub_typ_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SIMPLESECURITYID_FH_IDX],
              { "SimpleSecurityID", "eti.simplesecurityid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SKIPVALIDATIONS_FH_IDX],
              { "SkipValidations", "eti.skipvalidations",
                FT_UINT8, BASE_DEC, VALS(appl_resend_flag_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[STOPPX_FH_IDX],
              { "StopPx", "eti.stoppx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[STRATEGYLINKID_FH_IDX],
              { "StrategyLinkID", "eti.strategylinkid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[STRIKEPRICE_FH_IDX],
              { "StrikePrice", "eti.strikeprice",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SUBSCRIPTIONSCOPE_FH_IDX],
              { "SubscriptionScope", "eti.subscriptionscope",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[SWAPCLEARER_FH_IDX],
              { "SwapClearer", "eti.swapclearer",
                FT_UINT8, BASE_DEC, VALS(swap_clearer_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[T7ENTRYSERVICERTMSTATUS_FH_IDX],
              { "T7EntryServiceRtmStatus", "eti.t7entryservicertmstatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[T7ENTRYSERVICERTMTRADEDATE_FH_IDX],
              { "T7EntryServiceRtmTradeDate", "eti.t7entryservicertmtradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[T7ENTRYSERVICESTATUS_FH_IDX],
              { "T7EntryServiceStatus", "eti.t7entryservicestatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[T7ENTRYSERVICETRADEDATE_FH_IDX],
              { "T7EntryServiceTradeDate", "eti.t7entryservicetradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TESENRICHMENTRULEID_FH_IDX],
              { "TESEnrichmentRuleID", "eti.tesenrichmentruleid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TESEXECID_FH_IDX],
              { "TESExecID", "eti.tesexecid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYENTERINGTRADER_FH_IDX],
              { "TargetPartyEnteringTrader", "eti.targetpartyenteringtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYEXECUTINGFIRM_FH_IDX],
              { "TargetPartyExecutingFirm", "eti.targetpartyexecutingfirm",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYEXECUTINGTRADER_FH_IDX],
              { "TargetPartyExecutingTrader", "eti.targetpartyexecutingtrader",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYIDDESKID_FH_IDX],
              { "TargetPartyIDDeskID", "eti.targetpartyiddeskid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYIDEXECUTINGTRADER_FH_IDX],
              { "TargetPartyIDExecutingTrader", "eti.targetpartyidexecutingtrader",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TARGETPARTYIDSESSIONID_FH_IDX],
              { "TargetPartyIDSessionID", "eti.targetpartyidsessionid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TEMPLATEID_FH_IDX],
              { "TemplateID", "eti.templateid",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[THROTTLEDISCONNECTLIMIT_FH_IDX],
              { "ThrottleDisconnectLimit", "eti.throttledisconnectlimit",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[THROTTLENOMSGS_FH_IDX],
              { "ThrottleNoMsgs", "eti.throttlenomsgs",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[THROTTLETIMEINTERVAL_FH_IDX],
              { "ThrottleTimeInterval", "eti.throttletimeinterval",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TIMEINFORCE_FH_IDX],
              { "TimeInForce", "eti.timeinforce",
                FT_UINT8, BASE_DEC, VALS(time_in_force_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TOTNUMTRADEREPORTS_FH_IDX],
              { "TotNumTradeReports", "eti.totnumtradereports",
                FT_INT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADSESEVENT_FH_IDX],
              { "TradSesEvent", "eti.tradsesevent",
                FT_UINT8, BASE_DEC, VALS(trad_ses_event_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADSESMODE_FH_IDX],
              { "TradSesMode", "eti.tradsesmode",
                FT_UINT8, BASE_DEC, VALS(trad_ses_mode_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEALLOCSTATUS_FH_IDX],
              { "TradeAllocStatus", "eti.tradeallocstatus",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &trade_alloc_status_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEDATE_FH_IDX],
              { "TradeDate", "eti.tradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEID_FH_IDX],
              { "TradeID", "eti.tradeid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEMANAGERSTATUS_FH_IDX],
              { "TradeManagerStatus", "eti.trademanagerstatus",
                FT_UINT8, BASE_DEC, VALS(appl_seq_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEMANAGERTRADEDATE_FH_IDX],
              { "TradeManagerTradeDate", "eti.trademanagertradedate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEPLATFORM_FH_IDX],
              { "TradePlatform", "eti.tradeplatform",
                FT_UINT8, BASE_DEC, VALS(trade_platform_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEPUBLISHINDICATOR_FH_IDX],
              { "TradePublishIndicator", "eti.tradepublishindicator",
                FT_UINT8, BASE_DEC, VALS(trade_publish_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEREPORTID_FH_IDX],
              { "TradeReportID", "eti.tradereportid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEREPORTTEXT_FH_IDX],
              { "TradeReportText", "eti.tradereporttext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEREPORTTYPE_FH_IDX],
              { "TradeReportType", "eti.tradereporttype",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &trade_report_type_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADEREQUESTRESULT_FH_IDX],
              { "TradeRequestResult", "eti.traderequestresult",
                FT_UINT8, BASE_DEC, VALS(trade_request_result_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADETOQUOTERATIO_FH_IDX],
              { "TradeToQuoteRatio", "eti.tradetoquoteratio",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADETOQUOTERATIOPOSITION_FH_IDX],
              { "TradeToQuoteRatioPosition", "eti.tradetoquoteratioposition",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADETOQUOTERATIORANKING_FH_IDX],
              { "TradeToQuoteRatioRanking", "eti.tradetoquoteratioranking",
                FT_UINT8, BASE_DEC, VALS(enlight_rfqavg_resp_rate_ranking_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADETOREQUESTRATIO_FH_IDX],
              { "TradeToRequestRatio", "eti.tradetorequestratio",
                FT_UINT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADINGCAPACITY_FH_IDX],
              { "TradingCapacity", "eti.tradingcapacity",
                FT_UINT8, BASE_DEC, VALS(trading_capacity_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRADINGSESSIONSUBID_FH_IDX],
              { "TradingSessionSubID", "eti.tradingsessionsubid",
                FT_UINT8, BASE_DEC, VALS(trading_session_sub_id_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRANSBKDTIME_FH_IDX],
              { "TransBkdTime", "eti.transbkdtime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRANSACTTIME_FH_IDX],
              { "TransactTime", "eti.transacttime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRANSACTIONDELAYINDICATOR_FH_IDX],
              { "TransactionDelayIndicator", "eti.transactiondelayindicator",
                FT_UINT8, BASE_DEC, VALS(transaction_delay_indicator_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRANSFERREASON_FH_IDX],
              { "TransferReason", "eti.transferreason",
                FT_UINT8, BASE_DEC, VALS(transfer_reason_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDMATCHID_FH_IDX],
              { "TrdMatchID", "eti.trdmatchid",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDREGTSENTRYTIME_FH_IDX],
              { "TrdRegTSEntryTime", "eti.trdregtsentrytime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDREGTSEXECUTIONTIME_FH_IDX],
              { "TrdRegTSExecutionTime", "eti.trdregtsexecutiontime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDREGTSTIMEIN_FH_IDX],
              { "TrdRegTSTimeIn", "eti.trdregtstimein",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDREGTSTIMEOUT_FH_IDX],
              { "TrdRegTSTimeOut", "eti.trdregtstimeout",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDREGTSTIMEPRIORITY_FH_IDX],
              { "TrdRegTSTimePriority", "eti.trdregtstimepriority",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDRPTSTATUS_FH_IDX],
              { "TrdRptStatus", "eti.trdrptstatus",
                FT_UINT8, BASE_DEC| BASE_EXT_STRING, &trd_rpt_status_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRDTYPE_FH_IDX],
              { "TrdType", "eti.trdtype",
                FT_UINT16, BASE_DEC| BASE_EXT_STRING, &trd_type_vals_ext, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[TRIGGERED_FH_IDX],
              { "Triggered", "eti.triggered",
                FT_UINT8, BASE_DEC, VALS(triggered_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGCURRENCY_FH_IDX],
              { "UnderlyingCurrency", "eti.underlyingcurrency",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGDELTAPERCENTAGE_FH_IDX],
              { "UnderlyingDeltaPercentage", "eti.underlyingdeltapercentage",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGEFFECTIVEDELTAPERCENTAGE_FH_IDX],
              { "UnderlyingEffectiveDeltaPercentage", "eti.underlyingeffectivedeltapercentage",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGISSUER_FH_IDX],
              { "UnderlyingIssuer", "eti.underlyingissuer",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGMATURITYDATE_FH_IDX],
              { "UnderlyingMaturityDate", "eti.underlyingmaturitydate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGPRICESTIPVALUE_FH_IDX],
              { "UnderlyingPriceStipValue", "eti.underlyingpricestipvalue",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGPX_FH_IDX],
              { "UnderlyingPx", "eti.underlyingpx",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGQTY_FH_IDX],
              { "UnderlyingQty", "eti.underlyingqty",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGSECURITYDESC_FH_IDX],
              { "UnderlyingSecurityDesc", "eti.underlyingsecuritydesc",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGSECURITYID_FH_IDX],
              { "UnderlyingSecurityID", "eti.underlyingsecurityid",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGSETTLEMENTDATE_FH_IDX],
              { "UnderlyingSettlementDate", "eti.underlyingsettlementdate",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGSTIPTYPE_FH_IDX],
              { "UnderlyingStipType", "eti.underlyingstiptype",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[UNDERLYINGSTIPVALUE_FH_IDX],
              { "UnderlyingStipValue", "eti.underlyingstipvalue",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[USERSTATUS_FH_IDX],
              { "UserStatus", "eti.userstatus",
                FT_UINT8, BASE_DEC, VALS(user_status_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[USERNAME_FH_IDX],
              { "Username", "eti.username",
                FT_UINT32, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VALIDUNTILTIME_FH_IDX],
              { "ValidUntilTime", "eti.validuntiltime",
                FT_ABSOLUTE_TIME, ABSOLUTE_TIME_UTC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VALUECHECKTYPEMINLOTSIZE_FH_IDX],
              { "ValueCheckTypeMinLotSize", "eti.valuechecktypeminlotsize",
                FT_UINT8, BASE_DEC, VALS(value_check_type_min_lot_size_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VALUECHECKTYPEVALUE_FH_IDX],
              { "ValueCheckTypeValue", "eti.valuechecktypevalue",
                FT_UINT8, BASE_DEC, VALS(value_check_type_min_lot_size_vals), 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VARTEXT_FH_IDX],
              { "VarText", "eti.vartext",
                FT_STRINGZTRUNC, STR_ASCII, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VARTEXTLEN_FH_IDX],
              { "VarTextLen", "eti.vartextlen",
                FT_UINT16, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti[VEGA_FH_IDX],
              { "Vega", "eti.vega",
                FT_INT64, BASE_DEC, NULL, 0x0,
                NULL, HFILL }
          }
        , { &hf_eti_dscp_exec_summary,
              { "DSCP_ExecSummary", "eti.dscp_execsummary",
                FT_BOOLEAN, 8, NULL, 0x10,
                NULL, HFILL }
          }
        , { &hf_eti_dscp_improved,
              { "DSCP_Improved", "eti.dscp_improved",
                FT_BOOLEAN, 8, NULL, 0x20,
                NULL, HFILL }
          }
        , { &hf_eti_dscp_widened,
              { "DSCP_Widened", "eti.dscp_widened",
                FT_BOOLEAN, 8, NULL, 0x40,
                NULL, HFILL }
          }
    };
    static ei_register_info ei[] = {
        {
            &ei_eti_counter_overflow,
            { "eti.counter_overflow", PI_PROTOCOL, PI_WARN, "Counter Overflow", EXPFILL }
        },
        {
            &ei_eti_invalid_template,
            { "eti.invalid_template", PI_PROTOCOL, PI_ERROR, "Invalid Template ID", EXPFILL }
        },
        {
            &ei_eti_invalid_length,
            { "eti.invalid_length", PI_PROTOCOL, PI_ERROR, "Invalid Body Length", EXPFILL }
        },
        {
            &ei_eti_unaligned,
            { "eti.unaligned", PI_PROTOCOL, PI_ERROR, "A Body Length not divisible by 8 leads to unaligned followup messages", EXPFILL }
        },
        {
            &ei_eti_missing,
            { "eti.missing", PI_PROTOCOL, PI_WARN, "A required value is missing", EXPFILL }
        },
        {
            &ei_eti_overused,
            { "eti.overused", PI_PROTOCOL, PI_WARN, "An unused value is set", EXPFILL }
        }
    };
    proto_eti = proto_register_protocol("Enhanced Trading Interface 9.1",
            "ETI", "eti");
    expert_module_t *expert_eti = expert_register_protocol(proto_eti);
    expert_register_field_array(expert_eti, ei, array_length(ei));
    proto_register_field_array(proto_eti, hf, array_length(hf));
    static gint * const ett[] = { &ett_eti[0], &ett_eti[1], &ett_eti[2], &ett_eti[3], &ett_eti[4], &ett_eti[5], &ett_eti[6], &ett_eti[7], &ett_eti[8], &ett_eti[9], &ett_eti[10], &ett_eti[11], &ett_eti[12], &ett_eti[13], &ett_eti[14], &ett_eti[15], &ett_eti[16], &ett_eti[17], &ett_eti[18], &ett_eti[19], &ett_eti[20], &ett_eti[21], &ett_eti[22], &ett_eti[23], &ett_eti[24], &ett_eti[25], &ett_eti[26], &ett_eti[27], &ett_eti[28], &ett_eti[29], &ett_eti[30], &ett_eti[31], &ett_eti[32], &ett_eti[33], &ett_eti[34], &ett_eti[35], &ett_eti[36], &ett_eti[37], &ett_eti[38], &ett_eti[39], &ett_eti[40], &ett_eti[41], &ett_eti[42], &ett_eti[43], &ett_eti[44], &ett_eti[45], &ett_eti[46], &ett_eti[47], &ett_eti[48], &ett_eti_dscp };
    proto_register_subtree_array(ett, array_length(ett));
}

void
proto_reg_handoff_eti(void)
{
    dissector_handle_t eti_handle = create_dissector_handle(dissect_eti,
            proto_eti);

    // cf. N7 Network Access Guide, e.g.
    // https://www.xetra.com/xetra-en/technology/t7/system-documentation/release10-0/Release-10.0-2692700?frag=2692724
    // https://www.xetra.com/resource/blob/2762078/388b727972b5122945eedf0e63c36920/data/N7-Network-Access-Guide-v2.0.59.pdf


    // NB: can only be called once for a port/handle pair ...
    // dissector_add_uint_with_preference("tcp.port", 19006 /* LF PROD */, eti_handle);

    dissector_add_uint("tcp.port", 19006 /* LF PROD */, eti_handle);
    dissector_add_uint("tcp.port", 19043 /* PS PROD */, eti_handle);
    dissector_add_uint("tcp.port", 19506 /* LF SIMU */, eti_handle);
    dissector_add_uint("tcp.port", 19543 /* PS SIMU */, eti_handle);
}

